@@ -287,32 +287,34 @@ void PGABinaryCreateString(PGAContext *ctx, int p, int pop, int initflag)
 ****************************************************************************I*/
 int PGABinaryMutation( PGAContext *ctx, int p, int pop, double mr )
 {
-     int i,wi;
-     int count = 0;
-     PGABinary *c;
+    int i,wi;
+    int count = 0;
+    PGABinary *c;
 
-     PGADebugEntered("PGABinaryMutation");
+    PGADebugEntered ("PGABinaryMutation");
 
-     c = (PGABinary *)PGAGetIndividual(ctx, p, pop)->chrom;
-     for(wi=0; wi<ctx->ga.fw; wi++)
-          for(i=0; i<WL; ++i)
-               if ( PGARandomFlip(ctx, mr) )
-               {
-                    TOGGLE(i,c[wi]);
-                    count++;
-               }
+    c = (PGABinary *)PGAGetIndividual (ctx, p, pop)->chrom;
+    for (wi=0; wi<ctx->ga.fw; wi++) {
+        for (i=0; i<(int)WL; ++i) {
+            if (PGARandomFlip (ctx, mr)) {
+                TOGGLE (i,c[wi]);
+                count++;
+            }
+        }
+    }
 
-     /* clean up the partial word if eb > 0 */
-     if (ctx->ga.eb > 0 )
-          for(i=0;i<ctx->ga.eb;++i)
-               if ( PGARandomFlip(ctx, mr) )
-               {
-                    TOGGLE(i,c[ctx->ga.fw]);
-                    count++;
-               }
+    /* clean up the partial word if eb > 0 */
+    if (ctx->ga.eb > 0) {
+        for (i=0; i<ctx->ga.eb; ++i) {
+            if (PGARandomFlip (ctx, mr)) {
+                TOGGLE (i,c[ctx->ga.fw]);
+                count++;
+            }
+        }
+    }
 
-    PGADebugExited("PGABinaryMutation");
-    return(count);
+    PGADebugExited ("PGABinaryMutation");
+    return (count);
 
 }
 
@@ -539,34 +541,35 @@ void PGABinaryTwoptCrossover(PGAContext *ctx, int p1, int p2, int pop1, int c1,
       PGABinaryUniformCrossover( ctx, m, d, PGA_OLDPOP, s, b, PGA_NEWPOP );
 
 ****************************************************************************I*/
-void PGABinaryUniformCrossover(PGAContext *ctx, int p1, int p2, int pop1,
-                               int c1, int c2, int pop2)
+void PGABinaryUniformCrossover (PGAContext *ctx, int p1, int p2, int pop1,
+                                int c1, int c2, int pop2)
 {
-     PGABinary *parent1 = (PGABinary *)PGAGetIndividual(ctx, p1, pop1)->chrom;
-     PGABinary *parent2 = (PGABinary *)PGAGetIndividual(ctx, p2, pop1)->chrom;
-     PGABinary *child1  = (PGABinary *)PGAGetIndividual(ctx, c1, pop2)->chrom;
-     PGABinary *child2  = (PGABinary *)PGAGetIndividual(ctx, c2, pop2)->chrom;
-     PGABinary mask;
-     int j,wi;
+    PGABinary *parent1 = (PGABinary *)PGAGetIndividual (ctx, p1, pop1)->chrom;
+    PGABinary *parent2 = (PGABinary *)PGAGetIndividual (ctx, p2, pop1)->chrom;
+    PGABinary *child1  = (PGABinary *)PGAGetIndividual (ctx, c1, pop2)->chrom;
+    PGABinary *child2  = (PGABinary *)PGAGetIndividual (ctx, c2, pop2)->chrom;
+    PGABinary mask;
+    int j,wi;
 
-    PGADebugEntered("PGABinaryUniformCrossover");
+    PGADebugEntered ("PGABinaryUniformCrossover");
 
-    for(wi=0;wi<ctx->ga.tw;wi++) {
-        if ( parent1[wi] == parent2[wi] ) {
+    for (wi=0; wi<ctx->ga.tw; wi++) {
+        if (parent1[wi] == parent2[wi]) {
             child1[wi] = parent1[wi];
             child2[wi] = parent2[wi];
-        }
-        else {
+        } else {
             mask = 0;
-            for (j=0;j<WL;j++)
-                if(PGARandomFlip(ctx, ctx->ga.UniformCrossProb))
+            for (j=0; j<(int)WL; j++) {
+                if (PGARandomFlip (ctx, ctx->ga.UniformCrossProb)) {
                     SET(j,mask);
-            child1[wi] = (mask & parent1[wi])|(~mask & parent2[wi]);
-            child2[wi] = (mask & parent2[wi])|(~mask & parent1[wi]);
+                }
+            }
+            child1[wi] = (mask & parent1 [wi]) | (~mask & parent2 [wi]);
+            child2[wi] = (mask & parent2 [wi]) | (~mask & parent1 [wi]);
         }
     }
 
-    PGADebugExited("PGABinaryUniformCrossover");
+    PGADebugExited ("PGABinaryUniformCrossover");
 }
 
 /*I****************************************************************************
@@ -587,7 +590,7 @@ void PGABinaryUniformCrossover(PGAContext *ctx, int p1, int p2, int pop1,
       PGAContext *ctx;
       int s;
       :
-      PGABinaryPrintString( ctx, stdout, s, PGA_NEWPOP );
+      PGABinaryPrintString (ctx, stdout, s, PGA_NEWPOP);
 
 ****************************************************************************I*/
 void PGABinaryPrintString (PGAContext *ctx, FILE *fp, int p, int pop)
@@ -676,8 +679,9 @@ void PGABinaryCopyString (PGAContext *ctx, int p1, int pop1, int p2, int pop2)
       PGAContext *ctx;
       int x, y;
       :
-      if ( PGABinaryDuplicate( ctx, x, PGA_NEWPOP, y, PGA_NEWPOP ) )
-          printf("strings are duplicates\n");
+      if (PGABinaryDuplicate (ctx, x, PGA_NEWPOP, y, PGA_NEWPOP)) {
+          printf ("strings are duplicates\n");
+      }
 
 ****************************************************************************I*/
 int PGABinaryDuplicate( PGAContext *ctx, int p1, int pop1, int p2, int pop2)
@@ -699,6 +703,26 @@ int PGABinaryDuplicate( PGAContext *ctx, int p1, int pop1, int p2, int pop2)
      return wi == ctx->ga.tw ? PGA_TRUE : PGA_FALSE;
 }
 
+/*I****************************************************************************
+   PGABinaryHash - Returns hash value of given gene
+
+   Inputs:
+      ctx  - context variable
+      p    - string index of the string to hash
+      pop  - symbolic constant of the population string p is in
+
+   Outputs:
+      Hash value for string
+
+****************************************************************************I*/
+PGAHash PGABinaryHash (PGAContext *ctx, int p, int pop)
+{
+     void *a = PGAGetIndividual (ctx, p, pop)->chrom;
+     PGAHash hash = PGAUtilHash
+        (a, sizeof (PGABinary) * ctx->ga.tw, PGA_INITIAL_HASH);
+     return hash;
+}
+
 /*I****************************************************************************
    PGABinaryInitString - randomly initialize a string of type PGABinary
 
@@ -754,58 +778,34 @@ void PGABinaryInitString(PGAContext *ctx, int p, int pop)
       Called only by MPI routines.  Not for user consumption.
 
 ****************************************************************************I*/
-MPI_Datatype PGABinaryBuildDatatype(PGAContext *ctx, int p, int pop)
+MPI_Datatype PGABinaryBuildDatatype (PGAContext *ctx, int p, int pop)
 {
-    int            n = 6;
-    int            counts[7];      /* Number of elements in each
-                                      block (array of integer) */
-    MPI_Aint       displs[7];      /* byte displacement of each
-                                      block (array of integer) */
-    MPI_Datatype   types[7];       /* type of elements in each block (array
-                                      of handles to datatype objects) */
+    int            idx = 0;
+    /* Number of elements in each block (array of integer) */
+    int            counts [PGA_MPI_HEADER_ELEMENTS + 1];
+    /* byte displacement of each block (array of integer) */
+    MPI_Aint       displs [PGA_MPI_HEADER_ELEMENTS + 1];
+    /* type of elements in each block (array of handles to datatype objects) */
+    MPI_Datatype   types  [PGA_MPI_HEADER_ELEMENTS + 1];
     MPI_Datatype   individualtype; /* new datatype (handle) */
     PGAIndividual *traveller;      /* address of individual in question */
 
-    PGADebugEntered("PGABinaryBuildDatatype");
+    PGADebugEntered ("PGABinaryBuildDatatype");
 
-    traveller = PGAGetIndividual(ctx, p, pop);
-    MPI_Get_address(&traveller->evalue, &displs[0]);
-    counts[0] = 1;
-    types[0]  = MPI_DOUBLE;
+    traveller = PGAGetIndividual (ctx, p, pop);
+    idx = PGABuildDatatypeHeader (ctx, p, pop, counts, displs, types);
 
-    MPI_Get_address(&traveller->fitness, &displs[1]);
-    counts[1] = 1;
-    types[1]  = MPI_DOUBLE;
+    MPI_Get_address (traveller->chrom, &displs [idx]);
+    counts [idx] = ctx->ga.tw;
+    types  [idx] = MPI_UNSIGNED_LONG;
+    idx++;
 
-    MPI_Get_address(&traveller->evaluptodate, &displs[2]);
-    counts[2] = 1;
-    types[2]  = MPI_INT;
+    MPI_Type_create_struct (idx, counts, displs, types, &individualtype);
+    MPI_Type_commit (&individualtype);
 
-    MPI_Get_address(traveller->chrom, &displs[3]);
-    counts[3] = ctx->ga.tw;
-    types[3]  = MPI_UNSIGNED_LONG;
+    PGADebugExited ("PGABinaryBuildDatatype");
 
-    MPI_Get_address(&traveller->auxtotal, &displs[4]);
-    counts[4] = 1;
-    types[4]  = MPI_DOUBLE;
-
-    MPI_Get_address(&traveller->auxtotalok, &displs[5]);
-    counts[5] = 1;
-    types[5]  = MPI_INT;
-
-    if (ctx->ga.NumAuxEval) {
-        MPI_Get_address(traveller->auxeval, &displs[6]);
-        counts[6] = ctx->ga.NumAuxEval;
-        types[6]  = MPI_DOUBLE;
-        n += 1;
-    }
-
-    MPI_Type_create_struct(n, counts, displs, types, &individualtype);
-    MPI_Type_commit(&individualtype);
-
-    PGADebugExited("PGABinaryBuildDatatype");
-
-    return (individualtype);
+    return individualtype;
 }
 
 
@@ -835,28 +835,28 @@ int PGABinaryHammingDistance ( PGAContext *ctx, PGABinary *s1, PGABinary *s2 )
     int        j, wi, distance;
     PGABinary  t1, t2, mask;
 
-    PGADebugEntered("PGABinaryHammingDistance");
+    PGADebugEntered ("PGABinaryHammingDistance");
 
     distance = 0;
-    for(wi=0; wi<ctx->ga.tw; wi++)  /* step through each word in the string */
-        if ( s1[wi] != s2[wi] ) {   /* if equal, no bits are different      */
-            /*fprintf(stdout,"s1[wi] = %x, s2[wi] = %x\n",s1[wi],s2[wi]);*/
+    for (wi=0; wi<ctx->ga.tw; wi++) { /* step through each word in string   */
+        if (s1[wi] != s2[wi]) {     /* if equal, no bits are different      */
             mask = 1;
-            for(j=0;j<WL;++j) {     /* not equal, compare all bits          */
+            for (j=0; j<(int)WL; ++j) {  /* not equal, compare all bits     */
                 /* Build bit mask in position j. Mask bit from each         */
                 /* string into t1 and t2 and test if bits are the same      */
-                t1 = s1[wi] & mask;
-                t2 = s2[wi] & mask;
-                /*fprintf(stdout,"mask = %u, t1 = %u, t2 = %u, j = %d, wi = %d\n",mask,t1,t2,j,wi);*/
-                if ( t1 != t2 )
+                t1 = s1 [wi] & mask;
+                t2 = s2 [wi] & mask;
+                if (t1 != t2) {
                     distance++;
+                }
                 mask <<= 1;          /* shift mask 1 position */
             }
         }
+    }
 
-    PGADebugExited("PGABinaryHammingDistance");
+    PGADebugExited ("PGABinaryHammingDistance");
 
-    return(distance);
+    return (distance);
 }
 
 /*I****************************************************************************
