@@ -45,8 +45,15 @@ privately owned rights.
 *              Brian P. Walenz
 *****************************************************************************/
 
+#include <stdint.h>
 #include "pgapack.h"
 
+/* Helper function to free builtin datatype */
+static void ChromFree (PGAIndividual *ind)
+{
+    free (ind->chrom);
+}
+
 /*U****************************************************************************
   PGACreate - creates an uninitialized context variable.  The Fortran version
   of this function call contains only the last three arguments
@@ -219,12 +226,23 @@ PGAContext *PGACreate
     ctx->ga.NumAuxEval         = PGA_UNINITIALIZED_INT;
     ctx->ga.NumConstraint      = PGA_UNINITIALIZED_INT;
     ctx->ga.SumConstraints     = PGA_UNINITIALIZED_INT;
+    ctx->ga.Epsilon            = 0.0;
+    ctx->ga.Epsilon_0          = 0.0;
+    ctx->ga.EpsilonGeneration  = PGA_UNINITIALIZED_INT;
+    ctx->ga.EpsilonExponent    = PGA_UNINITIALIZED_DOUBLE;
+    ctx->ga.EffEpsExponent     = PGA_UNINITIALIZED_DOUBLE;
+    ctx->ga.EpsTLambda         = PGA_UNINITIALIZED_INT;
+    ctx->ga.EpsilonTheta       = PGA_UNINITIALIZED_INT;
     ctx->ga.StoppingRule       = PGA_STOP_MAXITER;
     ctx->ga.MaxIter            = PGA_UNINITIALIZED_INT;
     ctx->ga.MaxNoChange        = PGA_UNINITIALIZED_INT;
     ctx->ga.MaxSimilarity      = PGA_UNINITIALIZED_INT;
     ctx->ga.NumReplace         = PGA_UNINITIALIZED_INT;
     ctx->ga.CrossoverType      = PGA_UNINITIALIZED_INT;
+    ctx->ga.CrossBoundedFlag   = PGA_UNINITIALIZED_INT;
+    ctx->ga.CrossBounceFlag    = PGA_UNINITIALIZED_INT;
+    ctx->ga.CrossSBXEta        = PGA_UNINITIALIZED_DOUBLE;
+    ctx->ga.CrossSBXOnce       = PGA_UNINITIALIZED_INT;
     ctx->ga.SelectType         = PGA_UNINITIALIZED_INT;
     ctx->ga.TournamentSize     = PGA_UNINITIALIZED_INT;
     ctx->ga.TournamentWithRepl = PGA_UNINITIALIZED_INT;
@@ -234,12 +252,13 @@ PGAContext *PGACreate
     ctx->ga.FitnessType        = PGA_UNINITIALIZED_INT;
     ctx->ga.FitnessMinType     = PGA_UNINITIALIZED_INT;
     ctx->ga.MutationType       = PGA_UNINITIALIZED_INT;
-    ctx->ga.MutateOnly         = PGA_UNINITIALIZED_INT;
-    ctx->ga.MutateOnlyNoCross  = PGA_UNINITIALIZED_INT;
+    ctx->ga.MixingType         = PGA_UNINITIALIZED_INT;
     ctx->ga.MutateRealValue    = PGA_UNINITIALIZED_DOUBLE;
     ctx->ga.MutateIntegerValue = PGA_UNINITIALIZED_INT;
     ctx->ga.MutateBoundedFlag  = PGA_UNINITIALIZED_INT;
     ctx->ga.MutateBounceFlag   = PGA_UNINITIALIZED_INT;
+    ctx->ga.MutatePolyEta      = PGA_UNINITIALIZED_DOUBLE;
+    ctx->ga.MutatePolyValue    = PGA_UNINITIALIZED_DOUBLE;
     ctx->ga.DEVariant          = PGA_UNINITIALIZED_INT;
     ctx->ga.DENumDiffs         = PGA_UNINITIALIZED_INT;
     ctx->ga.DECrossoverType    = PGA_UNINITIALIZED_INT;
@@ -259,6 +278,8 @@ PGAContext *PGACreate
     ctx->ga.FitnessCmaxValue   = PGA_UNINITIALIZED_DOUBLE;
     ctx->ga.PopReplace         = PGA_UNINITIALIZED_INT;
     ctx->ga.iter               = 0;
+    ctx->ga.last_iter          = -1;
+    ctx->ga.perm_idx           = 0;
     ctx->ga.ItersOfSame        = 0;
     ctx->ga.PercentSame        = 0;
     ctx->ga.selected           = NULL;
@@ -266,6 +287,32 @@ PGAContext *PGACreate
     ctx->ga.restart            = PGA_UNINITIALIZED_INT;
     ctx->ga.restartFreq        = PGA_UNINITIALIZED_INT;
     ctx->ga.restartAlleleProb  = PGA_UNINITIALIZED_DOUBLE;
+    ctx->ga.OutputFile         = stdout;
+    ctx->ga.OutFileName        = NULL;
+    ctx->ga.CustomData         = NULL;
+
+    /* Fixed edges for Edge Crossover */
+    ctx->ga.n_edges            = 0;
+    ctx->ga.edges              = NULL;
+    ctx->ga.r_edge             = NULL;
+    ctx->ga.symmetric          = PGA_FALSE;
+
+    /* NSGA-III */
+    ctx->ga.nrefdirs       = 0;
+    ctx->ga.nrefpoints     = 0;
+    ctx->ga.refdirs        = NULL;
+    ctx->ga.refpoints      = NULL;
+    ctx->ga.ndir_npart     = 0;
+    ctx->ga.dirscale       = 0;
+    ctx->ga.extreme        = NULL;
+    ctx->ga.extreme_valid  = PGA_FALSE;
+    ctx->ga.utopian        = NULL;
+    ctx->ga.utopian_valid  = PGA_FALSE;
+    ctx->ga.nadir          = NULL;
+    ctx->ga.worst          = NULL;
+    ctx->ga.worst_valid    = PGA_FALSE;
+    ctx->ga.normdirs       = NULL;
+    ctx->ga.ndpoints       = 0;
 
     /* Operations */
     ctx->cops.CreateString      = NULL;
@@ -280,6 +327,11 @@ PGAContext *PGACreate
     ctx->cops.EndOfGen          = NULL;
     ctx->cops.GeneDistance      = NULL;
     ctx->cops.PreEval           = NULL;
+    ctx->cops.Hash              = NULL;
+    ctx->cops.Serialize         = NULL;
+    ctx->cops.Deserialize       = NULL;
+    ctx->cops.SerializeFree     = NULL;
+    ctx->cops.ChromFree         = NULL;
 
     ctx->fops.Mutation          = NULL;
     ctx->fops.Crossover         = NULL;
@@ -291,6 +343,7 @@ PGAContext *PGACreate
     ctx->fops.EndOfGen          = NULL;
     ctx->fops.GeneDistance      = NULL;
     ctx->fops.PreEval           = NULL;
+    ctx->fops.Hash              = NULL;
 
     /* Parallel */
     ctx->par.NumIslands        = PGA_UNINITIALIZED_INT;
@@ -305,6 +358,7 @@ PGAContext *PGACreate
     /* Reporting */
     ctx->rep.PrintFreq         = PGA_UNINITIALIZED_INT;
     ctx->rep.PrintOptions      = 0;
+    ctx->rep.MOPrecision       = 14;
     ctx->rep.Offline           = NULL;
     ctx->rep.Online            = NULL;
     ctx->rep.Average           = NULL;
@@ -427,23 +481,43 @@ PGAContext *PGACreate
 void PGASetUp ( PGAContext *ctx )
 {
     /*  These are for temporary storage of datatype specific functions.
-     *  They allow some (understatement of the yesr!!) cleaning of the
+     *  They allow some (understatement of the year!!) cleaning of the
      *  code below.
      */
-    void   (*CreateString)(PGAContext *, int, int, int) = NULL;
-    int    (*Mutation)(PGAContext *, int, int, double) = NULL;
-    void   (*Crossover)(PGAContext *, int, int, int, int, int, int) = NULL;
-    void   (*PrintString)(PGAContext *, FILE *, int, int) = NULL;
-    void   (*CopyString)(PGAContext *, int, int, int, int) = NULL;
-    int    (*Duplicate)(PGAContext *, int, int, int, int) = NULL;
-    void   (*InitString)(PGAContext *, int, int) = NULL;
-    double (*GeneDist)(PGAContext *, int, int, int, int) = NULL;
+    void    (*CreateString)(PGAContext *, int, int, int) = NULL;
+    int     (*Mutation)(PGAContext *, int, int, double) = NULL;
+    void    (*Crossover)(PGAContext *, int, int, int, int, int, int) = NULL;
+    void    (*PrintString)(PGAContext *, FILE *, int, int) = NULL;
+    void    (*CopyString)(PGAContext *, int, int, int, int) = NULL;
+    int     (*Duplicate)(PGAContext *, int, int, int, int) = NULL;
+    PGAHash (*Hash)(PGAContext *, int, int) = NULL;
+    void    (*InitString)(PGAContext *, int, int) = NULL;
+    double  (*GeneDist)(PGAContext *, int, int, int, int) = NULL;
     MPI_Datatype (*BuildDatatype)(PGAContext *, int, int) = NULL;
     int err=0, i;
 
     PGADebugEntered("PGASetUp");
     PGAFailIfSetUp("PGASetUp");
 
+    if (ctx->ga.datatype != PGA_DATATYPE_USER
+       && (  ctx->cops.Serialize
+          || ctx->cops.Deserialize
+          || ctx->cops.SerializeFree
+          )
+       )
+    {
+        PGAErrorPrintf
+            ( ctx, PGA_FATAL
+            , "PGASetUp: Serialize/Deserialize only with user defined datatype"
+            );
+    }
+    if (ctx->ga.datatype != PGA_DATATYPE_USER && ctx->cops.ChromFree) {
+        PGAErrorPrintf
+            ( ctx, PGA_FATAL
+            , "PGASetUp: ChromFree only with user defined datatype"
+            );
+    }
+
     if (  ctx->ga.datatype == PGA_DATATYPE_BINARY
        && ctx->ga.tw       == PGA_UNINITIALIZED_INT
        )
@@ -474,10 +548,6 @@ void PGASetUp ( PGAContext *ctx )
                  );
     }
 
-    if (ctx->ga.PopSize == PGA_UNINITIALIZED_INT) {
-        ctx->ga.PopSize = 100;
-    }
-
     if (ctx->ga.NumAuxEval == PGA_UNINITIALIZED_INT) {
         ctx->ga.NumAuxEval = 0;
     }
@@ -486,10 +556,6 @@ void PGASetUp ( PGAContext *ctx )
         ctx->ga.NumConstraint = ctx->ga.NumAuxEval;
     }
 
-    if (ctx->ga.SumConstraints == PGA_UNINITIALIZED_INT) {
-        ctx->ga.SumConstraints = PGA_FALSE;
-    }
-
     if (  ctx->ga.NumConstraint > ctx->ga.NumAuxEval
        || ctx->ga.NumConstraint < 0
        )
@@ -499,6 +565,133 @@ void PGASetUp ( PGAContext *ctx )
                   );
     }
 
+    if (ctx->ga.PopReplace        == PGA_UNINITIALIZED_INT) {
+        if (ctx->ga.NumAuxEval - ctx->ga.NumConstraint > 0) {
+            ctx->ga.PopReplace    = PGA_POPREPL_NSGA_II;
+        } else {
+            ctx->ga.PopReplace    = PGA_POPREPL_BEST;
+        }
+    }
+
+    if (ctx->ga.nrefpoints > 0 && ctx->ga.PopReplace != PGA_POPREPL_NSGA_III) {
+        PGAErrorPrintf
+            (ctx, PGA_FATAL, "PGASetUp: Reference points only for NSGA-III");
+    }
+    if (ctx->ga.nrefdirs > 0 && ctx->ga.PopReplace != PGA_POPREPL_NSGA_III) {
+        PGAErrorPrintf
+            ( ctx, PGA_FATAL
+            , "PGASetUp: Reference directions only for NSGA-III"
+            );
+    }
+
+    if (  ctx->ga.NumAuxEval - ctx->ga.NumConstraint > 0
+       && ctx->ga.PopReplace != PGA_POPREPL_NSGA_II
+       && ctx->ga.PopReplace != PGA_POPREPL_NSGA_III
+       )
+    {
+        PGAErrorPrintf
+            ( ctx, PGA_FATAL
+            , "PGASetUp: NumAuxEval=%d, NumConstraint=%d: Population "
+              "replacement with multi-objective optimization must be NSGA-II"
+              " or NSGA-III"
+            , ctx->ga.NumAuxEval, ctx->ga.NumConstraint
+            );
+    }
+
+    ctx->ga.extreme_valid = PGA_FALSE;
+    ctx->ga.utopian_valid = PGA_FALSE;
+    ctx->ga.worst_valid   = PGA_FALSE;
+    if (ctx->ga.PopReplace == PGA_POPREPL_NSGA_III) {
+        int dim = ctx->ga.NumAuxEval - ctx->ga.NumConstraint + 1;
+        ctx->ga.extreme = malloc (sizeof (double) * dim * dim);
+        if (ctx->ga.extreme == NULL) {
+            PGAErrorPrintf (ctx, PGA_FATAL, "Cannot allocate extreme point");
+        }
+        memset (ctx->ga.extreme, 0, sizeof (double) * dim * dim);
+        ctx->ga.utopian = malloc (sizeof (double) * dim);
+        if (ctx->ga.utopian == NULL) {
+            PGAErrorPrintf (ctx, PGA_FATAL, "Cannot allocate utopian point");
+        }
+        memset (ctx->ga.utopian, 0, sizeof (double) * dim);
+        ctx->ga.nadir = malloc (sizeof (double) * dim);
+        if (ctx->ga.nadir == NULL) {
+            PGAErrorPrintf (ctx, PGA_FATAL, "Cannot allocate nadir point");
+        }
+        memset (ctx->ga.nadir, 0, sizeof (double) * dim);
+        ctx->ga.worst = malloc (sizeof (double) * dim);
+        if (ctx->ga.worst == NULL) {
+            PGAErrorPrintf (ctx, PGA_FATAL, "Cannot allocate worst point");
+        }
+        memset (ctx->ga.worst, 0, sizeof (double) * dim);
+        if (ctx->ga.nrefdirs == 0) {
+            assert (ctx->ga.refdirs == NULL);
+            if (ctx->ga.nrefpoints == 0) {
+                assert (ctx->ga.refpoints == NULL);
+                (void)LIN_dasdennis (dim, 2, &ctx->ga.refpoints, 0, 1, NULL);
+                ctx->ga.nrefpoints = LIN_binom (dim + 2 - 1, 2);
+                if (ctx->ga.refpoints == NULL) {
+                    PGAErrorPrintf
+                        (ctx, PGA_FATAL, "Cannot allocate ref points");
+                }
+            }
+        } else {
+            /* Allocate space for normalized reference directions */
+            int npart = ctx->ga.ndir_npart;
+            size_t lb = LIN_binom (dim + npart - 1, npart);
+            size_t n = lb * ctx->ga.nrefdirs;
+            /* Check that there is/was no overflow */
+            assert (lb <= n);
+            assert (n < SIZE_MAX / (sizeof (double) * dim));
+            ctx->ga.normdirs = malloc (sizeof (double) * dim * n);
+            if (ctx->ga.normdirs == NULL) {
+                PGAErrorPrintf (ctx, PGA_FATAL, "Cannot allocate normdirs");
+            }
+            ctx->ga.ndpoints = lb;
+            if (ctx->ga.nrefpoints == 0) {
+                assert (ctx->ga.refpoints == NULL);
+                ctx->ga.nrefpoints = LIN_dasdennis
+                    (dim, 1, &ctx->ga.refpoints, 0, 1, NULL);
+                if (ctx->ga.refpoints == NULL) {
+                    PGAErrorPrintf
+                        (ctx, PGA_FATAL, "Cannot allocate ref points");
+                }
+            }
+        }
+    }
+
+    /* Init PopSize from refdirs/refpoints if applicable, otherwise use
+     * default of 100
+     */
+    if (ctx->ga.PopSize == PGA_UNINITIALIZED_INT) {
+        int dim = ctx->ga.NumAuxEval - ctx->ga.NumConstraint + 1;
+        int mod;
+        size_t ps = 0;
+        if (ctx->ga.nrefpoints > 0) {
+            ps += ctx->ga.nrefpoints;
+        }
+        if (ctx->ga.nrefdirs > 0) {
+            int npart = ctx->ga.ndir_npart;
+            size_t lb = LIN_binom (dim + npart - 1, npart);
+            size_t n = lb * ctx->ga.nrefdirs;
+            ps += n;
+        }
+        /* Next multiple of 4, see NSGA-III papers */
+        mod = ps % 4;
+        if (mod) {
+            ps += 4 - mod;
+        }
+
+        assert (ps < INT_MAX);
+        if (ps == 0) {
+            ps = 100;
+        }
+        ctx->ga.PopSize = ps;
+    }
+
+    if (ctx->ga.SumConstraints == PGA_UNINITIALIZED_INT) {
+        ctx->ga.SumConstraints = PGA_FALSE;
+    }
+
     if (ctx->ga.MaxIter == PGA_UNINITIALIZED_INT) {
         ctx->ga.MaxIter = 1000;
     }
@@ -521,10 +714,72 @@ void PGASetUp ( PGAContext *ctx )
                  );
     }
 
+    if (ctx->ga.EpsilonGeneration == PGA_UNINITIALIZED_INT) {
+        ctx->ga.EpsilonGeneration = 0;
+    }
+    if (ctx->ga.EpsilonGeneration > ctx->ga.MaxIter) {
+        PGAError ( ctx, "PGASetUp: EpsilonGeneration > MaxIter"
+                 , PGA_FATAL, PGA_VOID, NULL
+                 );
+    }
+
+    if (ctx->ga.EpsilonExponent == PGA_UNINITIALIZED_DOUBLE) {
+        /* This is later used to initialize it to a dynamic value */
+        ctx->ga.EpsilonExponent = 0;
+        ctx->ga.EffEpsExponent  = 0;
+        ctx->ga.EpsTLambda      = round (0.95 * ctx->ga.EpsilonGeneration);
+        if (ctx->ga.EpsTLambda < 1) {
+            ctx->ga.EpsTLambda = 1;
+        }
+    } else {
+        ctx->ga.EffEpsExponent  = ctx->ga.EpsilonExponent;
+        ctx->ga.EpsTLambda      = 0;
+    }
+
+    if (ctx->ga.EpsilonTheta == PGA_UNINITIALIZED_INT) {
+        ctx->ga.EpsilonTheta = round (0.2 * ctx->ga.PopSize);
+        if (!ctx->ga.EpsilonGeneration) {
+            ctx->ga.EpsilonTheta = 0;
+        }
+    }
+
+    if (ctx->ga.EpsilonTheta >= ctx->ga.PopSize - 1) {
+        PGAError ( ctx, "PGASetUp: EpsilonTheta >= PopSize - 1"
+                 , PGA_FATAL, PGA_VOID, NULL
+                 );
+    }
+
     if (ctx->ga.CrossoverType == PGA_UNINITIALIZED_INT) {
         ctx->ga.CrossoverType = PGA_CROSSOVER_TWOPT;
     }
 
+    if (  ctx->ga.CrossoverType == PGA_CROSSOVER_SBX
+       && ctx->ga.datatype != PGA_DATATYPE_INTEGER
+       && ctx->ga.datatype != PGA_DATATYPE_REAL
+       )
+    {
+        PGAErrorPrintf
+            ( ctx, PGA_FATAL
+            , "PGASetUp: SBX crossover only for Integer and Real datatypes"
+            );
+    }
+
+    if (ctx->ga.CrossBoundedFlag == PGA_UNINITIALIZED_INT) {
+        ctx->ga.CrossBoundedFlag = PGA_FALSE;
+    }
+
+    if (ctx->ga.CrossBounceFlag == PGA_UNINITIALIZED_INT) {
+        ctx->ga.CrossBounceFlag = PGA_FALSE;
+    }
+
+    if (ctx->ga.CrossSBXEta == PGA_UNINITIALIZED_DOUBLE) {
+        ctx->ga.CrossSBXEta = 2;
+    }
+
+    if (ctx->ga.CrossSBXOnce == PGA_UNINITIALIZED_INT) {
+        ctx->ga.CrossSBXOnce = PGA_FALSE;
+    }
+
     if (ctx->ga.SelectType == PGA_UNINITIALIZED_INT) {
         ctx->ga.SelectType = PGA_SELECT_TOURNAMENT;
     }
@@ -554,6 +809,14 @@ void PGASetUp ( PGAContext *ctx )
             v = 1;
         }
         ctx->ga.RTRWindowSize = v;
+    } else if (  ctx->ga.RTRWindowSize > ctx->ga.PopSize
+              || ctx->ga.RTRWindowSize <= 0
+              )
+    {
+        PGAErrorPrintf
+            ( ctx, PGA_FATAL
+            , "PGASetUp: required: 0 < RTR window size <= popsize"
+            );
     }
 
     if ( ctx->ga.NumAuxEval > 0
@@ -587,23 +850,22 @@ void PGASetUp ( PGAContext *ctx )
         ctx->ga.FitnessMinType     = PGA_FITNESSMIN_CMAX;
     }
 
-    if (ctx->ga.MutateOnly        == PGA_UNINITIALIZED_INT) {
-        ctx->ga.MutateOnly         = PGA_FALSE;
+    if (ctx->ga.MixingType        == PGA_UNINITIALIZED_INT) {
+        ctx->ga.MixingType         = PGA_MIX_MUTATE_OR_CROSS;
     }
 
     /* Require a gene length of at least two if we're doing crossover */
-    if (!ctx->ga.MutateOnly && ctx->ga.StringLen <= 1) {
+    if (  (ctx->ga.MixingType != PGA_MIX_MUTATE_ONLY && ctx->ga.StringLen <= 1)
+       || ctx->ga.StringLen <= 0
+       )
+    {
         PGAError ( ctx, "PGACreate: Invalid value of StringLen:"
                  , PGA_FATAL, PGA_INT, (void *) &ctx->ga.StringLen
                  );
     }
 
-    if (ctx->ga.MutateOnlyNoCross == PGA_UNINITIALIZED_INT) {
-        ctx->ga.MutateOnlyNoCross  = PGA_TRUE;
-    }
-
     if (  ctx->ga.CrossoverType == PGA_CROSSOVER_TWOPT
-       && ctx->ga.StringLen == 2 && !ctx->ga.MutateOnlyNoCross
+       && ctx->ga.StringLen == 2 && ctx->ga.MixingType != PGA_MIX_MUTATE_ONLY
        )
     {
         PGAError ( ctx
@@ -673,6 +935,14 @@ void PGASetUp ( PGAContext *ctx )
         ctx->ga.MutateBounceFlag   = PGA_FALSE;
     }
 
+    if (ctx->ga.MutatePolyEta  == PGA_UNINITIALIZED_DOUBLE) {
+        ctx->ga.MutatePolyEta   = 100;
+    }
+
+    if (ctx->ga.MutatePolyValue  == PGA_UNINITIALIZED_DOUBLE) {
+        ctx->ga.MutatePolyValue   = -1.0;
+    }
+
     if (ctx->ga.DEVariant         == PGA_UNINITIALIZED_INT) {
         ctx->ga.DEVariant          = PGA_DE_VARIANT_RAND;
     }
@@ -747,26 +1017,6 @@ void PGASetUp ( PGAContext *ctx )
         ctx->ga.FitnessCmaxValue   = 1.01;
     }
 
-    if (ctx->ga.PopReplace        == PGA_UNINITIALIZED_INT) {
-        if (ctx->ga.NumAuxEval - ctx->ga.NumConstraint > 0) {
-            ctx->ga.PopReplace    = PGA_POPREPL_NSGA_II;
-        } else {
-            ctx->ga.PopReplace    = PGA_POPREPL_BEST;
-        }
-    }
-
-    if (  ctx->ga.NumAuxEval - ctx->ga.NumConstraint > 0
-       && ctx->ga.PopReplace != PGA_POPREPL_NSGA_II
-       )
-    {
-        PGAErrorPrintf
-            ( ctx, PGA_FATAL
-            , "PGASetUp: NumAuxEval=%d, NumConstraint=%d: Population "
-              "replacement with multi-objective optimization must be NSGA-II"
-            , ctx->ga.NumAuxEval, ctx->ga.NumConstraint
-            );
-    }
-
     if (ctx->ga.restart           == PGA_UNINITIALIZED_INT) {
         ctx->ga.restart            = PGA_FALSE;
     }
@@ -818,6 +1068,7 @@ void PGASetUp ( PGAContext *ctx )
 	Duplicate      = PGABinaryDuplicate;
 	InitString     = PGABinaryInitString;
 	GeneDist       = PGABinaryGeneDistance;
+        Hash           = PGABinaryHash;
 	break;
       case PGA_DATATYPE_INTEGER:
         CreateString   = PGAIntegerCreateString;
@@ -833,12 +1084,19 @@ void PGASetUp ( PGAContext *ctx )
 	  case PGA_CROSSOVER_UNIFORM:
 	    Crossover  = PGAIntegerUniformCrossover;
             break;
+	  case PGA_CROSSOVER_SBX:
+	    Crossover  = PGAIntegerSBXCrossover;
+            break;
+	  case PGA_CROSSOVER_EDGE:
+	    Crossover  = PGAIntegerEdgeCrossover;
+            break;
         }
 	PrintString    = PGAIntegerPrintString;
 	CopyString     = PGAIntegerCopyString;
 	Duplicate      = PGAIntegerDuplicate;
 	InitString     = PGAIntegerInitString;
 	GeneDist       = PGAIntegerGeneDistance;
+        Hash           = PGAIntegerHash;
 	break;
       case PGA_DATATYPE_REAL:
 	CreateString   = PGARealCreateString;
@@ -854,12 +1112,16 @@ void PGASetUp ( PGAContext *ctx )
 	  case PGA_CROSSOVER_UNIFORM:
 	    Crossover  = PGARealUniformCrossover;
             break;
+	  case PGA_CROSSOVER_SBX:
+	    Crossover  = PGARealSBXCrossover;
+            break;
         }
 	PrintString   = PGARealPrintString;
 	CopyString    = PGARealCopyString;
 	Duplicate     = PGARealDuplicate;
 	InitString    = PGARealInitString;
 	GeneDist      = PGARealGeneDistance;
+	Hash          = PGARealHash;
 	break;
       case PGA_DATATYPE_CHARACTER:
 	CreateString  = PGACharacterCreateString;
@@ -881,66 +1143,102 @@ void PGASetUp ( PGAContext *ctx )
 	Duplicate   = PGACharacterDuplicate;
 	InitString  = PGACharacterInitString;
 	GeneDist    = PGACharacterGeneDistance;
+	Hash        = PGACharacterHash;
 	break;
       case PGA_DATATYPE_USER:
         if (ctx->cops.CreateString == NULL) {
-            PGAError ( ctx
-                     , "PGASetUp: User datatype needs CreateString function:"
-                     , PGA_WARNING, PGA_INT, (void *) &err
-                     );
+            PGAError
+                ( ctx, "PGASetUp: User datatype needs CreateString function:"
+                , PGA_FATAL, PGA_INT, (void *) &err
+                );
         }
-        if (ctx->cops.Mutation     == NULL) {
-            PGAError ( ctx
-                     , "PGASetUp: User datatype needs Mutation function:"
-                     , PGA_WARNING, PGA_INT, (void *) &err
-                     );
+        if (ctx->cops.Mutation == NULL) {
+            PGAError
+                ( ctx, "PGASetUp: User datatype needs Mutation function:"
+                , PGA_WARNING, PGA_INT, (void *) &err
+                );
         }
-        if (ctx->cops.Crossover    == NULL) {
-            PGAError ( ctx
-                     , "PGASetUp: User datatype needs Crossover function:"
-                     , PGA_WARNING, PGA_INT, (void *) &err
-                     );
+        if (ctx->cops.Crossover == NULL) {
+            PGAError
+                ( ctx, "PGASetUp: User datatype needs Crossover function:"
+                , PGA_WARNING, PGA_INT, (void *) &err
+                );
         }
-        if (ctx->cops.PrintString  == NULL) {
-            PGAError ( ctx
-                     , "PGASetUp: User datatype needs PrintString function:"
-                     , PGA_WARNING, PGA_INT, (void *) &err
-                     );
+        if (ctx->cops.PrintString == NULL) {
+            PGAError
+                ( ctx, "PGASetUp: User datatype needs PrintString function:"
+                , PGA_WARNING, PGA_INT, (void *) &err
+                );
         }
-	if (ctx->cops.Duplicate    == NULL) {
-            PGAError ( ctx
-                     , "PGASetUp: User datatype needs Duplicate function:"
-                     , PGA_WARNING, PGA_INT, (void *) &err
-                     );
+	if (ctx->cops.Duplicate == NULL && ctx->ga.NoDuplicates) {
+            PGAError
+                ( ctx, "PGASetUp: User datatype needs Duplicate function:"
+                , PGA_FATAL, PGA_INT, (void *) &err
+                );
         }
-	if (ctx->cops.CopyString    == NULL) {
-            PGAError ( ctx
-                     , "PGASetUp: User datatype needs CopyString function:"
-                     , PGA_WARNING, PGA_INT, (void *) &err
-                     );
+	if (ctx->cops.CopyString == NULL) {
+            PGAError
+                ( ctx, "PGASetUp: User datatype needs CopyString function:"
+                , PGA_FATAL, PGA_INT, (void *) &err
+                );
+        }
+        if (  ( ctx->cops.Serialize && !ctx->cops.Deserialize)
+           || (!ctx->cops.Serialize &&  ctx->cops.Deserialize)
+           )
+        {
+            PGAErrorPrintf
+                ( ctx, PGA_FATAL
+                , "PGASetUp: Serialize/Deserialize must be specified together"
+                );
+        }
+        if (ctx->cops.Serialize && ctx->cops.BuildDatatype) {
+            PGAErrorPrintf
+                ( ctx, PGA_FATAL
+                , "PGASetUp: Serialize/Deserialize must not "
+                  "specify BuildDatatype"
+                );
+        }
+        if (ctx->cops.Serialize) {
+            ctx->cops.BuildDatatype = PGASerializedBuildDatatype;
+            /* If not set we asume it's allocated by malloc and friends */
+            if (ctx->cops.SerializeFree == NULL) {
+                ctx->cops.SerializeFree = &free;
+            }
         }
         if (ctx->cops.BuildDatatype == NULL) {
-             PGAError ( ctx
-                      , "PGASetUp: User datatype needs BuildDatatype function:"
-                      , PGA_FATAL, PGA_INT, (void *) &err
-                      );
+             PGAError
+                ( ctx, "PGASetUp: User datatype needs BuildDatatype function:"
+                , PGA_FATAL, PGA_INT, (void *) &err
+                );
         }
         if (  ctx->cops.GeneDistance == NULL
            && ctx->ga.PopReplace == PGA_POPREPL_RTR
            )
         {
-             PGAError ( ctx
-                      , "PGASetUp: User datatype needs GeneDistance function:"
-                      , PGA_FATAL, PGA_INT, (void *) &err
-                      );
+             PGAError
+                ( ctx, "PGASetUp: User datatype needs GeneDistance function:"
+                , PGA_FATAL, PGA_INT, (void *) &err
+                );
+        }
+	if (ctx->cops.Hash == NULL && ctx->ga.NoDuplicates) {
+            PGAError
+                ( ctx, "PGASetUp: User datatype needs Hash function:"
+                , PGA_FATAL, PGA_INT, (void *) &err
+                );
         }
         break;
     }
+    if (ctx->cops.ChromFree == NULL) {
+        ctx->cops.ChromFree = &ChromFree;
+    }
     if ((ctx->cops.Mutation     == NULL) && (ctx->fops.Mutation    == NULL)) {
 	ctx->cops.Mutation      = Mutation;
     }
     if ((ctx->cops.Crossover    == NULL) && (ctx->fops.Crossover   == NULL)) {
 	ctx->cops.Crossover     = Crossover;
+        if (Crossover == NULL) {
+            PGAErrorPrintf (ctx, PGA_FATAL, "PGASetUp: No crossover specified");
+        }
     }
     if ((ctx->cops.PrintString  == NULL) && (ctx->fops.PrintString == NULL)) {
 	ctx->cops.PrintString   = PrintString;
@@ -963,6 +1261,9 @@ void PGASetUp ( PGAContext *ctx )
     if (ctx->cops.BuildDatatype == NULL) {
 	ctx->cops.BuildDatatype = BuildDatatype;
     }
+    if ((ctx->cops.Hash == NULL) && (ctx->fops.Hash == NULL)) {
+	ctx->cops.Hash = Hash;
+    }
 
 /* par */
     if (ctx->par.NumIslands == PGA_UNINITIALIZED_INT) {
@@ -1053,32 +1354,94 @@ void PGASetUp ( PGAContext *ctx )
                  );
     }
 
-    ctx->scratch.intscratch = (int *)malloc( sizeof(int) * ctx->ga.PopSize );
+    ctx->scratch.intscratch = malloc( sizeof(int) * ctx->ga.PopSize );
     if (ctx->scratch.intscratch == NULL) {
-        PGAError( ctx, "PGASetUp: No room to allocate ctx->scratch.intscratch"
+        PGAError
+            ( ctx, "PGASetUp: No room to allocate ctx->scratch.intscratch"
+            , PGA_FATAL, PGA_VOID, NULL
+            );
+    }
+    ctx->scratch.permute = NULL;
+    ctx->ga.perm_idx = 0;
+    if (  ctx->ga.SelectType == PGA_SELECT_TOURNAMENT
+       || ctx->ga.SelectType == PGA_SELECT_TRUNCATION
+       )
+    {
+        ctx->scratch.permute = malloc (sizeof (int) * ctx->ga.PopSize);
+        if (ctx->scratch.permute == NULL) {
+            PGAError
+                ( ctx, "PGASetUp: No room to allocate ctx->scratch.permute"
                 , PGA_FATAL, PGA_VOID, NULL
                 );
+        }
+        /* This forces a first shuffle */
+        ctx->ga.perm_idx = ctx->ga.PopSize;
     }
 
-    ctx->scratch.dblscratch =
-        (double *)malloc(sizeof(double) * ctx->ga.PopSize);
+    ctx->scratch.dblscratch = malloc (sizeof (double) * ctx->ga.PopSize);
 
     if (ctx->scratch.dblscratch == NULL) {
-        PGAError ( ctx, "PGASetUp: No room to allocate ctx->scratch.dblscratch"
-                 , PGA_FATAL, PGA_VOID, NULL
-                 );
+        PGAError
+            ( ctx, "PGASetUp: No room to allocate ctx->scratch.dblscratch"
+            , PGA_FATAL, PGA_VOID, NULL
+            );
     }
 
     /* If we're doing non-dominated sorting */
     if (ctx->ga.NumAuxEval - ctx->ga.NumConstraint >= 1) {
-        int nintbits = sizeof (int) * 8;
-        int intsfor2pop = (ctx->ga.PopSize * 2 + nintbits - 1) / nintbits;
+        int intsfor2pop = (ctx->ga.PopSize * 2 + WL - 1) / WL;
         ctx->scratch.dominance = malloc
-            (sizeof (unsigned int) * intsfor2pop * 2 * ctx->ga.PopSize);
+            (sizeof (PGABinary) * intsfor2pop * 2 * ctx->ga.PopSize);
+        if (ctx->scratch.dominance == NULL) {
+            PGAErrorPrintf
+                ( ctx, PGA_FATAL
+                , "PGASetUp: No room to allocate ctx->scratch.dominance"
+                );
+        }
     } else {
         ctx->scratch.dominance = NULL;
     }
 
+    if (ctx->ga.NoDuplicates) {
+        size_t hashsize = sizeof (PGAIndividual *) * ctx->ga.PopSize;
+        ctx->scratch.hashed = malloc (hashsize);
+        if (ctx->scratch.hashed == NULL) {
+            PGAErrorPrintf
+                ( ctx, PGA_FATAL
+                , "PGASetUp: No room to allocate ctx->scratch.hashed"
+                );
+        }
+        memset (ctx->scratch.hashed, 0, hashsize);
+    } else {
+        ctx->scratch.hashed = NULL;
+    }
+
+    /* If the crossover type is Edge crossover */
+    if (ctx->ga.CrossoverType == PGA_CROSSOVER_EDGE) {
+        ctx->scratch.edgemap = malloc
+            (sizeof (PGAInteger) * 4 * ctx->ga.StringLen);
+        if (ctx->scratch.edgemap == NULL) {
+            PGAErrorPrintf
+                ( ctx, PGA_FATAL
+                , "PGASetUp: No room to allocate ctx->scratch.edgemap"
+                );
+        }
+    } else {
+        ctx->scratch.edgemap = NULL;
+        if (ctx->ga.n_edges) {
+            PGAErrorPrintf
+                ( ctx, PGA_FATAL
+                , "PGASetUp: Fixed edges only for edge crossover"
+                );
+        }
+    }
+    if (ctx->ga.n_edges && ctx->init.IntegerMin [0] != 0) {
+        PGAErrorPrintf
+            (ctx, PGA_FATAL , "PGASetUp: Fixed edges only with IntegerMin=0");
+    }
+    ctx->scratch.serialization_size = 0;
+    ctx->scratch.serialized = NULL;
+
     PGACreatePop (ctx, PGA_OLDPOP);
     PGACreatePop (ctx, PGA_NEWPOP);
 
@@ -1125,6 +1488,16 @@ void PGASetUp ( PGAContext *ctx )
     }
     memset (ctx->rep.BestIdx, 0, sizeof (int) * (1 + ctx->ga.NumAuxEval));
 
+    if (ctx->ga.OutFileName != NULL && PGAGetRank (ctx, MPI_COMM_WORLD) == 0) {
+        ctx->ga.OutputFile = fopen (ctx->ga.OutFileName, "w");
+        if (ctx->ga.OutputFile == NULL) {
+            PGAErrorPrintf
+                ( ctx, PGA_FATAL
+                , "Cannot open output file: %s", strerror (errno)
+                );
+        }
+    }
+
     ctx->rep.starttime = time (NULL);
 
     /* This is done at the end to avoid trying to free unallocated memory */
@@ -1193,10 +1566,10 @@ void PGASetRandomInitFlag(PGAContext *ctx, int RandomBoolean)
       raninit = PGAGetRandomInitFlag(ctx);
       switch (raninit) {
       case PGA_TRUE:
-          printf("Population is randomly initialized\n");
+          printf ("Population is randomly initialized\n");
           break;
       case PGA_FALSE:
-          printf("Population initialized to zero\n");
+          printf ("Population initialized to zero\n");
           break;
       }
 
@@ -1311,6 +1684,7 @@ void PGACreateIndividual (PGAContext *ctx, int p, int pop, int initflag)
     PGADebugEntered("PGACreateIndividual");
 
     ind->ctx              = ctx;
+    ind->index            = p;
     ind->pop              = pop == PGA_OLDPOP ? ctx->ga.oldpop : ctx->ga.newpop;
     ind->evalue           = 0.0;
     ind->fitness          = 0.0;
@@ -1320,6 +1694,9 @@ void PGACreateIndividual (PGAContext *ctx, int p, int pop, int initflag)
     ind->rank             = UINT_MAX;
     ind->crowding         = 0;
     ind->funcidx          = 0;
+    ind->distance         = 0;
+    ind->point_idx        = 0;
+    ind->next_hash        = NULL;
     if (ctx->ga.NumAuxEval) {
         ind->auxeval = malloc (sizeof (double) * ctx->ga.NumAuxEval);
         if (ind->auxeval == NULL) {
@@ -1329,6 +1706,16 @@ void PGACreateIndividual (PGAContext *ctx, int p, int pop, int initflag)
     } else {
         ind->auxeval = NULL;
     }
+    if (ctx->ga.PopReplace == PGA_POPREPL_NSGA_III) {
+        int dim = ctx->ga.NumAuxEval - ctx->ga.NumConstraint + 1;
+        ind->normalized = malloc (sizeof (double) * dim);
+        if (ind->normalized == NULL) {
+            PGAErrorPrintf (ctx, PGA_FATAL, "Cannot allocate normalized point");
+        }
+        memset (ind->normalized, 0, sizeof (double) * dim);
+    } else {
+        ind->normalized = NULL;
+    }
 
     (*ctx->cops.CreateString)(ctx, p, pop, initflag);
 
@@ -1497,3 +1884,185 @@ int PGAGetSumConstraintsFlag (PGAContext *ctx)
 {
     return ctx->ga.SumConstraints;
 }
+
+/*I****************************************************************************
+  PGASetEpsilonGeneration - Configure the generation until which
+  constraints are relaxed via the Epsilon Contraint method. The default
+  is 0 (no Epsilon Contraint method is used).
+
+  Inputs:
+     ctx      - context variable
+     gen      - Epsilon contraint generation, must be below the value
+                set with PGASetMaxGAIterValue
+
+  Outputs:
+     None
+
+  Example:
+     PGAContext *ctx;
+     :
+     PGASetEpsilonGeneration (ctx, 50);
+
+****************************************************************************I*/
+void PGASetEpsilonGeneration (PGAContext *ctx, int gen)
+{
+    ctx->ga.EpsilonGeneration = gen;
+}
+
+/*I****************************************************************************
+  PGAGetEpsilonGeneration - Get value of the generation until which
+  constraints are relaxed via the Epsilon Contraint method.
+
+  Inputs:
+     ctx      - context variable
+
+  Outputs:
+     PGAGetEpsilonGeneration
+
+  Example:
+     PGAContext *ctx;
+     int n;
+     :
+     n = PGAGetEpsilonGeneration (ctx);
+
+****************************************************************************I*/
+int PGAGetEpsilonGeneration (PGAContext *ctx)
+{
+    return ctx->ga.EpsilonGeneration;
+}
+
+/*I****************************************************************************
+  PGASetEpsilonExponent - Configure the exponent of the term computing
+  the expsilon value in each generation.
+
+  Inputs:
+     ctx      - context variable
+     e        - Exponent
+
+  Outputs:
+     None
+
+  Example:
+     PGAContext *ctx;
+     :
+     PGASetEpsilonExponent (ctx, 5);
+
+****************************************************************************I*/
+void PGASetEpsilonExponent (PGAContext *ctx, double e)
+{
+    /* Note that in some papers the recommended minimum is 3 while in
+     * others it is 2. We allow 2.
+     */
+    if (e < 2 || e > PGA_EPSILON_EXPONENT_MAX) {
+        PGAError
+            ( ctx, "PGASetEpsilonExponent: 2 <= e <= 10 required"
+            , PGA_FATAL, PGA_VOID, NULL
+            );
+    }
+    ctx->ga.EpsilonExponent = e;
+}
+
+/*I****************************************************************************
+  PGAGetEpsilonExponent - Get the exponent used for epsilon constraints
+
+  Inputs:
+     ctx      - context variable
+
+  Outputs:
+     PGAGetEpsilonExponent
+
+  Example:
+     PGAContext *ctx;
+     double e;
+     :
+     e = PGAGetEpsilonExponent (ctx);
+
+****************************************************************************I*/
+double PGAGetEpsilonExponent (PGAContext *ctx)
+{
+    return ctx->ga.EpsilonExponent;
+}
+
+/*I****************************************************************************
+  PGASetEpsilonTheta - Set the theta generation value that is used for
+  initializing the epsilon constraint. The initial population is sorted
+  by constraint violation and the individual with index theta is used
+  for initializing the epsilon_0 for the epsilon constraint method.
+
+  Inputs:
+     ctx      - context variable
+     n        - population index theta
+
+  Outputs:
+     None
+
+  Example:
+     PGAContext *ctx;
+     :
+     PGASetEpsilonTheta (ctx, n);
+
+****************************************************************************I*/
+void PGASetEpsilonTheta (PGAContext *ctx, int n)
+{
+    if (n < 1) {
+        PGAError ( ctx, "PGASetUp: EpsilonTheta must be >= 1"
+                 , PGA_FATAL, PGA_VOID, NULL
+                 );
+    }
+    ctx->ga.EpsilonTheta = n;
+}
+
+/*I****************************************************************************
+  PGAGetEpsilonTheta - Query the population index for initializing
+  epsilon for the epsilon constraint method.
+
+  Inputs:
+     ctx      - context variable
+
+  Outputs:
+     PGAGetEpsilonTheta
+
+  Example:
+     PGAContext *ctx;
+     int n;
+     :
+     n = PGAGetEpsilonTheta (ctx);
+
+****************************************************************************I*/
+int PGAGetEpsilonTheta (PGAContext *ctx)
+{
+    return ctx->ga.EpsilonTheta;
+}
+
+/*I****************************************************************************
+  PGASetOutputFile - Set output file name for printing statistics etc.
+    Note that the file is not immediately opened, instead it is later
+    opened in the rank 0 individual.
+
+  Inputs:
+     ctx      - context variable
+     name     - output filename
+
+  Outputs:
+     None
+
+  Example:
+     PGAContext *ctx;
+     char *name = "output.file";
+     :
+     PGASetOutputFile (ctx, name);
+
+****************************************************************************I*/
+void PGASetOutputFile (PGAContext *ctx, char *name)
+{
+    char *n = malloc (strlen (name) + 1);
+    if (n == NULL) {
+        PGAErrorPrintf
+            ( ctx, PGA_FATAL
+            , "PGASetOutputFile: Cannot allocate name"
+            );
+    }
+    strcpy (n, name);
+    ctx->ga.OutFileName = n;
+}
+
