@@ -102,8 +102,8 @@ privately owned rights.
 #if OPTIMIZE==0
 typedef struct
 {
-        char    PGAFuncName[36];
-        int     PGAFuncNum;
+    char *PGAFuncName;
+    int   PGAFuncNum;
 } PGAFuncRec;
 
 int        PGANumFcns = 0;
@@ -206,7 +206,7 @@ PGAFuncRec PGAFuncIndex[] =
         { "PGASetUniformCrossoverProb",     316 },
 
         /* pop.c */
-        { "PGA_NSGA_II_Replacement",            317 },
+        { "PGA_NSGA_Replacement",               317 },
         { "PGAPairwiseBestReplacement",         318 },
         { "PGARestrictedTournamentReplacement", 319 },
         { "PGASortPop",                         320 },
@@ -413,9 +413,15 @@ PGAFuncRec PGAFuncIndex[] =
         { "PGASetUserFunction",             830 },
 
         /* END GUARD */
-        { }
+        { NULL }
 };
 
+int func_compare (const void *f1, const void *f2)
+{
+    const PGAFuncRec *pf1 = f1;
+    const PGAFuncRec *pf2 = f2;
+    return strcmp (pf1->PGAFuncName, pf2->PGAFuncName);
+}
 
 /*I****************************************************************************
    PGASortDebugIndex - Sort the index of function names alphabetically.
@@ -429,10 +435,8 @@ PGAFuncRec PGAFuncIndex[] =
 ****************************************************************************I*/
 void PGASortFuncNameIndex(PGAContext *ctx)
 {
-    PGANumFcns = sizeof (PGAFuncIndex) / sizeof (PGAFuncRec);
-
-    qsort(PGAFuncIndex, PGANumFcns, sizeof(PGAFuncRec),
-	  (int (*) (const void *, const void *)) &strcmp);
+    PGANumFcns = sizeof (PGAFuncIndex) / sizeof (PGAFuncRec) - 1;
+    qsort (PGAFuncIndex, PGANumFcns, sizeof(PGAFuncRec), func_compare);
 }
 
 
@@ -456,7 +460,7 @@ void PGASortFuncNameIndex(PGAContext *ctx)
                  datatype parameter (or NULL, if PGA_VOID).
 
   Outputs:
-     The debugging information is printed to stderr.
+     The debugging information is printed to stdout.
 
   Example:
      If the debugging level includes printing variables (level 82), print the
@@ -466,69 +470,81 @@ void PGASortFuncNameIndex(PGAContext *ctx)
      PGAContext *ctx;
      int num;
      :
-     PGADebugPrint(ctx, PGA_DEBUG_PRINTVAR, "Add2Nums", "num = ", PGA_INT,
-                   (void *) &num);
+     PGADebugPrint
+        (ctx, PGA_DEBUG_PRINTVAR, "Add2Nums", "num = ", PGA_INT, (void *) &num);
 
 ****************************************************************************U*/
-void PGADebugPrint( PGAContext *ctx, int level, char *funcname,
-                   char *msg, int datatype, void *data )
+void PGADebugPrint
+    ( PGAContext *ctx, int level
+    , char *funcname, char *msg, int datatype, void *data
+    )
 {
-     int rank;
+    int rank;
+    FILE *fout = stdout;
 
-     /*  Added check if level > 10 so that PGAGetDebugFlag is only called
-      *  if it is _not_ a user debug level.
-      */
+    /*  Added check if level > 10 so that PGAGetDebugFlag is only called
+     *  if it is _not_ a user debug level.
+     */
 
-     if (ctx->debug.PGADebugFlags[0]     ||
-         ctx->debug.PGADebugFlags[level] ||
-         ((level > 10) && PGAGetDebugFlag (ctx, funcname)))
-     {
-          MPI_Comm_rank(MPI_COMM_WORLD, &rank);
-          switch (datatype)
-          {
-          case PGA_VOID:
-               fprintf(stdout, "%4d: %-32s: %s\n", rank, funcname, msg);
-               break;
-          case PGA_INT:
-               switch (*(int *) data)
-               {
-               case PGA_TEMP1:
-                    fprintf(stdout, "%4d: %-32s: %s PGA_TEMP1\n", rank,
-                            funcname, msg);
-                    break;
-               case PGA_TEMP2:
-                    fprintf(stdout, "%4d: %-32s: %s PGA_TEMP2\n", rank,
-                            funcname, msg);
-                    break;
-               case PGA_OLDPOP:
-                    fprintf(stdout, "%4d: %-32s: %s PGA_OLDPOP\n", rank,
-                            funcname, msg);
-                    break;
-               case PGA_NEWPOP:
-                    fprintf(stdout, "%4d: %-32s: %s PGA_NEWPOP\n", rank,
-                            funcname, msg);
-                    break;
-               default:
-                    fprintf(stdout, "%4d: %-32s: %s %d\n", rank, funcname, msg,
-                            *(int *) data);
-                    break;
-               }
-               break;
-          case PGA_DOUBLE:
-               fprintf(stdout, "%4d: %-32s: %s %e\n", rank,
-                       funcname, msg, *(double *) data);
-               break;
-          case PGA_CHAR:
-               fprintf(stdout, "%4d: %-32s: %s %s\n", rank,
-                       funcname, msg,  (char *) data);
-               break;
-          default:
-               fprintf(stderr, "PGADebugPrint: Invalid value of datatype: %d",
-                       datatype);
-               exit(-1);
-               break;
-          }
-     }
+    if (  ctx->debug.PGADebugFlags[0]
+       || ctx->debug.PGADebugFlags[level]
+       || ((level > 10) && PGAGetDebugFlag (ctx, funcname))
+       )
+    {
+        MPI_Comm_rank (MPI_COMM_WORLD, &rank);
+        switch (datatype)
+        {
+        case PGA_VOID:
+            fprintf (fout, "%4d: %-32s: %s\n", rank, funcname, msg);
+            break;
+        case PGA_INT:
+            switch (*(int *) data)
+            {
+            case PGA_TEMP1:
+                fprintf
+                    (fout, "%4d: %-32s: %s PGA_TEMP1\n" , rank, funcname, msg);
+                break;
+            case PGA_TEMP2:
+                fprintf
+                    (fout, "%4d: %-32s: %s PGA_TEMP2\n", rank, funcname, msg);
+                break;
+            case PGA_OLDPOP:
+                fprintf
+                    (fout, "%4d: %-32s: %s PGA_OLDPOP\n", rank, funcname, msg);
+                break;
+            case PGA_NEWPOP:
+                fprintf
+                    (fout, "%4d: %-32s: %s PGA_NEWPOP\n" , rank, funcname, msg);
+                break;
+            default:
+                fprintf
+                    ( fout, "%4d: %-32s: %s %d\n"
+                    , rank, funcname, msg, *(int *) data
+                    );
+                break;
+            }
+            break;
+        case PGA_DOUBLE:
+            fprintf
+                ( fout, "%4d: %-32s: %s %e\n"
+                , rank, funcname, msg, *(double *) data
+                );
+            break;
+        case PGA_CHAR:
+            fprintf
+                ( fout, "%4d: %-32s: %s %s\n"
+                , rank, funcname, msg,  (char *) data
+                );
+            break;
+        default:
+            fprintf
+                ( stderr, "PGADebugPrint: Invalid value of datatype: %d"
+                , datatype
+                );
+            exit (-1);
+            break;
+        }
+    }
 }
 
 /*U****************************************************************************
@@ -701,37 +717,19 @@ void PGAClearDebugLevelByName(PGAContext *ctx, char *funcname)
 
 
 ****************************************************************************I*/
-int PGAGetDebugLevelOfName(PGAContext *ctx, char *funcname)
+int PGAGetDebugLevelOfName (PGAContext *ctx, char *funcname)
 {
-    int     l, h;
-    int     m, missing;
-
-    /*  Binary Search  */
-    l = 0;
-    h = PGANumFcns-1;
-    while (l <= h) {
-	m = (l+h)/2 + (l+h)%2;  /*  Actually, floor((l+h)/2)  */
-	missing = strcmp(funcname, PGAFuncIndex[m].PGAFuncName);
-
-        if (missing == 0) {
-            break;
-        } else {
-            if (missing < 0) {
-                h = m - 1;
-            } else {
-                l = m + 1;
-            }
-        }
-    }
-
-    if (missing) {
-	fprintf(stderr, "PGAGetDebugFlag: Function missing from "
-		"PGAFuncIndex: '%s'\n", funcname);
-	PGADestroy(ctx);
-	exit(-1);
+    PGAFuncRec x = { funcname };
+    PGAFuncRec *rec = bsearch
+        (&x, PGAFuncIndex, PGANumFcns, sizeof (PGAFuncRec), func_compare);
+    if (rec == NULL) {
+        PGAErrorPrintf
+            ( ctx, PGA_FATAL
+            , "PGAGetDebugFlag: Function missing from PGAFuncIndex: '%s'\n"
+            , funcname
+            );
     } 
-
-    return(PGAFuncIndex[m].PGAFuncNum);
+    return rec->PGAFuncNum;
 }
 
 /*I****************************************************************************
