@@ -85,6 +85,13 @@ privately owned rights.
 #define pgagetnumconstraint_             PGAGETNUMCONSTRAINT
 #define pgasetsumconstraintsflag_        PGASETSUMCONSTRAINTSFLAG
 #define pgagetsumconstraintsflag_        PGAGETSUMCONSTRAINTSFLAG
+#define pgasetepsilongeneration_         PGASETEPSILONGENERATION
+#define pgagetepsilongeneration_         PGAGETEPSILONGENERATION
+#define pgasetepsilonexponent_           PGASETEPSILONEXPONENT
+#define pgagetepsilonexponent_           PGAGETEPSILONEXPONENT
+#define pgasetepsilontheta_              PGASETEPSILONTHETA
+#define pgagetepsilontheta_              PGAGETEPSILONTHETA
+#define pgasetoutputfile_                PGASETOUTPUTFILE
 
 /* cross.c */
 #define pgacrossover_                    PGACROSSOVER
@@ -94,6 +101,14 @@ privately owned rights.
 #define pgasetcrossovertype_             PGASETCROSSOVERTYPE
 #define pgasetcrossoverprob_             PGASETCROSSOVERPROB
 #define pgasetuniformcrossoverprob_      PGASETUNIFORMCROSSOVERPROB
+#define pgasetcrossoverboundedflag_      PGASETCROSSOVERBOUNDEDFLAG
+#define pgagetcrossoverboundedflag_      PGAGETCROSSOVERBOUNDEDFLAG
+#define pgasetcrossoverbouncebackflag_   PGASETCROSSOVERBOUNCEBACKFLAG
+#define pgagetcrossoverbouncebackflag_   PGAGETCROSSOVERBOUNCEBACKFLAG
+#define pgasetcrossoversbxeta_           PGASETCROSSOVERSBXETA
+#define pgagetcrossoversbxeta_           PGAGETCROSSOVERSBXETA
+#define pgasetcrossoversbxonceperstring_ PGASETCROSSOVERSBXONCEPERSTRING
+#define pgagetcrossoversbxonceperstring_ PGAGETCROSSOVERSBXONCEPERSTRING
 /* debug.c */
 #define pgadebugprint_                   PGADEBUGPRINT
 #define pgasetdebuglevel_                PGASETDEBUGLEVEL
@@ -174,6 +189,10 @@ privately owned rights.
 #define pgagetdedither_                  PGAGETDEDITHER
 #define pgasetdeditherperindividual_     PGASETDEDITHERPERINDIVIDUAL
 #define pgagetdeditherperindividual_     PGAGETDEDITHERPERINDIVIDUAL
+#define pgasetmutationpolyeta_           PGASETMUTATIONPOLYETA
+#define pgagetmutationpolyeta_           PGAGETMUTATIONPOLYETA
+#define pgasetmutationpolyvalue_         PGASETMUTATIONPOLYVALUE
+#define pgagetmutationpolyvalue_         PGAGETMUTATIONPOLYVALUE
 /* parallel.c */
 #define pgarungm_                        PGARUNGM
 #define pgaevaluate_                     PGAEVALUATE
@@ -199,9 +218,11 @@ privately owned rights.
 #define pgasetmutationorcrossoverflag_   PGASETMUTATIONORCROSSOVERFLAG
 #define pgasetmutationandcrossoverflag_  PGASETMUTATIONANDCROSSOVERFLAG
 #define pgasetmutationonlyflag_          PGASETMUTATIONONLYFLAG
+#define pgasetmixingtype_                PGASETMIXINGTYPE
 #define pgagetmutationorcrossoverflag_   PGAGETMUTATIONORCROSSOVERFLAG
 #define pgagetmutationandcrossoverflag_  PGAGETMUTATIONANDCROSSOVERFLAG
 #define pgagetmutationonlyflag_          PGAGETMUTATIONONLYFLAG
+#define pgagetmixingtype_                PGAGETMIXINGTYPE
 /* pop.c */
 #define pgasortpop_                      PGASORTPOP
 #define pgagetpopsize_                   PGAGETPOPSIZE
@@ -234,6 +255,8 @@ privately owned rights.
 #define pgasetprintoptions_              PGASETPRINTOPTIONS
 #define pgasetprintfrequencyvalue_       PGASETPRINTFREQUENCYVALUE
 #define pgagetprintfrequencyvalue_       PGAGETPRINTFREQUENCYVALUE
+#define pgasetmultiobjprecision_         PGASETMULTIOBJPRECISION
+#define pgagetmultiobjprecision_         PGAGETMULTIOBJPRECISION
 #define pgaprintpopulation_              PGAPRINTPOPULATION
 #define pgaprintindividual_              PGAPRINTINDIVIDUAL
 #define pgaprintstring_                  PGAPRINTSTRING
@@ -270,6 +293,7 @@ privately owned rights.
 #define pgagetmaxgaitervalue_            PGAGETMAXGAITERVALUE
 #define pgasetmaxnochangevalue_          PGASETMAXNOCHANGEVALUE
 #define pgasetmaxsimilarityvalue_        PGASETMAXSIMILARITYVALUE
+#define pgagetmaxsimilarityvalue_        PGAGETMAXSIMILARITYVALUE
 /* system.c */
 #define pgaerror_                        PGAERROR
 #define pgadestroy_                      PGADESTROY
@@ -291,6 +315,7 @@ privately owned rights.
 #define pgagetbestindex_                 PGAGETBESTINDEX
 #define pgagetbestreport_                PGAGETBESTREPORT
 #define pgagetbestreportindex_           PGAGETBESTREPORTINDEX
+#define pgautilhash_                     PGAUTILHASH
 
 #elif defined(FORTRANTWOUNDERSCORE)
 /* binary.c */
@@ -313,6 +338,13 @@ privately owned rights.
 #define pgagetnumconstraint_             _pgagetnumconstraint_
 #define pgasetsumconstraintsflag_        _pgasetsumconstraintsflag_
 #define pgagetsumconstraintsflag_        _pgagetsumconstraintsflag_
+#define pgasetepsilongeneration_         _pgasetepsilongeneration_
+#define pgagetepsilongeneration_         _pgagetepsilongeneration_
+#define pgasetepsilonexponent_           _pgasetepsilonexponent_
+#define pgagetepsilonexponent_           _pgagetepsilonexponent_
+#define pgasetepsilontheta_              _pgasetepsilontheta_
+#define pgagetepsilontheta_              _pgagetepsilontheta_
+#define pgasetoutputfile_                _pgasetoutputfile_
 /* cross.c */
 #define pgacrossover_                    _pgacrossover_
 #define pgagetcrossovertype_             _pgagetcrossovertype_
@@ -321,6 +353,14 @@ privately owned rights.
 #define pgasetcrossovertype_             _pgasetcrossovertype_
 #define pgasetcrossoverprob_             _pgasetcrossoverprob_
 #define pgasetuniformcrossoverprob_      _pgasetuniformcrossoverprob_
+#define pgasetcrossoverboundedflag_      _pgasetcrossoverboundedflag_
+#define pgagetcrossoverboundedflag_      _pgagetcrossoverboundedflag_
+#define pgasetcrossoverbouncebackflag_   _pgasetcrossoverbouncebackflag_
+#define pgagetcrossoverbouncebackflag_   _pgagetcrossoverbouncebackflag_
+#define pgasetcrossoversbxeta_           _pgasetcrossoversbxeta_
+#define pgagetcrossoversbxeta_           _pgagetcrossoversbxeta_
+#define pgasetcrossoversbxonceperstring_ _pgasetcrossoversbxonceperstring_
+#define pgagetcrossoversbxonceperstring_ _pgagetcrossoversbxonceperstring_
 /* debug.c */
 #define pgadebugprint_                   _pgadebugprint_
 #define pgasetdebuglevel_                _pgasetdebuglevel_
@@ -401,6 +441,10 @@ privately owned rights.
 #define pgagetdedither_                  _pgagetdedither_
 #define pgasetdeditherperindividual_     _pgasetdeditherperindividual_
 #define pgagetdeditherperindividual_     _pgagetdeditherperindividual_
+#define pgasetmutationpolyeta_           _pgasetmutationpolyeta_
+#define pgagetmutationpolyeta_           _pgagetmutationpolyeta_
+#define pgasetmutationpolyvalue_         _pgasetmutationpolyvalue_
+#define pgagetmutationpolyvalue_         _pgagetmutationpolyvalue_
 /* parallel.c */
 #define pgarungm_                        _pgarungm_
 #define pgaevaluate_                     _pgaevaluate_
@@ -426,9 +470,11 @@ privately owned rights.
 #define pgasetmutationorcrossoverflag_   _pgasetmutationorcrossoverflag_
 #define pgasetmutationandcrossoverflag_  _pgasetmutationandcrossoverflag_
 #define pgasetmutationonlyflag_          _pgasetmutationonlyflag_
+#define pgasetmixingtype_                _pgasetmixingtype_
 #define pgagetmutationorcrossoverflag_   _pgagetmutationorcrossoverflag_
 #define pgagetmutationandcrossoverflag_  _pgagetmutationandcrossoverflag_
 #define pgagetmutationonlyflag_          _pgagetmutationonlyflag_
+#define pgagetmixingtype_                _pgagetmixingtype_
 /* pop.c */
 #define pgasortpop_                      _pgasortpop_
 #define pgagetpopsize_                   _pgagetpopsize_
@@ -461,6 +507,8 @@ privately owned rights.
 #define pgasetprintoptions_              _pgasetprintoptions_
 #define pgasetprintfrequencyvalue_       _pgasetprintfrequencyvalue_
 #define pgagetprintfrequencyvalue_       _pgagetprintfrequencyvalue_
+#define pgasetmultiobjprecision_         _pgasetmultiobjprecision_
+#define pgagetmultiobjprecision_         _pgagetmultiobjprecision_
 #define pgaprintpopulation_              _pgaprintpopulation_
 #define pgaprintindividual_              _pgaprintindividual_
 #define pgaprintstring_                  _pgaprintstring_
@@ -497,6 +545,7 @@ privately owned rights.
 #define pgagetmaxgaitervalue_            _pgagetmaxgaitervalue_
 #define pgasetmaxnochangevalue_          _pgasetmaxnochangevalue_
 #define pgasetmaxsimilarityvalue_        _pgasetmaxsimilarityvalue_
+#define pgagetmaxsimilarityvalue_        _pgagetmaxsimilarityvalue_
 /* system.c */
 #define pgaerror_                        _pgaerror_
 #define pgadestroy_                      _pgadestroy_
@@ -518,6 +567,7 @@ privately owned rights.
 #define pgagetbestindex_                 _pgagetbestindex_
 #define pgagetbestreport_                _pgagetbestreport_
 #define pgagetbestreportindex_           _pgagetbestreportindex_
+#define pgautilhash_                     _pgautilhash_
 
 #elif !defined(FORTRANUNDERSCORE)
 /* binary.c */
@@ -540,6 +590,13 @@ privately owned rights.
 #define pgagetnumconstraint_             pgagetnumconstraint
 #define pgasetsumconstraintsflag_        pgasetsumconstraintsflag
 #define pgagetsumconstraintsflag_        pgagetsumconstraintsflag
+#define pgasetepsilongeneration_         pgasetepsilongeneration
+#define pgagetepsilongeneration_         pgagetepsilongeneration
+#define pgasetepsilonexponent_           pgasetepsilonexponent
+#define pgagetepsilonexponent_           pgagetepsilonexponent
+#define pgasetepsilontheta_              pgasetepsilontheta
+#define pgagetepsilontheta_              pgagetepsilontheta
+#define pgasetoutputfile_                pgasetoutputfile
 /* cross.c */
 #define pgacrossover_                    pgacrossover
 #define pgagetcrossovertype_             pgagetcrossovertype
@@ -548,6 +605,14 @@ privately owned rights.
 #define pgasetcrossovertype_             pgasetcrossovertype
 #define pgasetcrossoverprob_             pgasetcrossoverprob
 #define pgasetuniformcrossoverprob_      pgasetuniformcrossoverprob
+#define pgasetcrossoverboundedflag_      pgasetcrossoverboundedflag
+#define pgagetcrossoverboundedflag_      pgagetcrossoverboundedflag
+#define pgasetcrossoverbouncebackflag_   pgasetcrossoverbouncebackflag
+#define pgagetcrossoverbouncebackflag_   pgagetcrossoverbouncebackflag
+#define pgasetcrossoversbxeta_           pgasetcrossoversbxeta
+#define pgagetcrossoversbxeta_           pgagetcrossoversbxeta
+#define pgasetcrossoversbxonceperstring_ pgasetcrossoversbxonceperstring
+#define pgagetcrossoversbxonceperstring_ pgagetcrossoversbxonceperstring
 /* debug.c */
 #define pgadebugprint_                   pgadebugprint
 #define pgasetdebuglevel_                pgasetdebuglevel
@@ -628,6 +693,10 @@ privately owned rights.
 #define pgagetdedither_                  pgagetdedither
 #define pgasetdeditherperindividual_     pgasetdeditherperindividual
 #define pgagetdeditherperindividual_     pgagetdeditherperindividual
+#define pgasetmutationpolyeta_           pgasetmutationpolyeta
+#define pgagetmutationpolyeta_           pgagetmutationpolyeta
+#define pgasetmutationpolyvalue_         pgasetmutationpolyvalue
+#define pgagetmutationpolyvalue_         pgagetmutationpolyvalue
 /* parallel.c */
 #define pgarungm_                        pgarungm
 #define pgaevaluate_                     pgaevaluate
@@ -653,9 +722,11 @@ privately owned rights.
 #define pgasetmutationorcrossoverflag_   pgasetmutationorcrossoverflag
 #define pgasetmutationandcrossoverflag_  pgasetmutationandcrossoverflag
 #define pgasetmutationonlyflag_          pgasetmutationonlyflag
+#define pgasetmixingtype_                pgasetmixingtype
 #define pgagetmutationorcrossoverflag_   pgagetmutationorcrossoverflag
 #define pgagetmutationandcrossoverflag_  pgagetmutationandcrossoverflag
 #define pgagetmutationonlyflag_          pgagetmutationonlyflag
+#define pgagetmixingtype_                pgagetmixingtype
 /* pop.c */
 #define pgasortpop_                      pgasortpop
 #define pgagetpopsize_                   pgagetpopsize
@@ -688,6 +759,8 @@ privately owned rights.
 #define pgasetprintoptions_              pgasetprintoptions
 #define pgasetprintfrequencyvalue_       pgasetprintfrequencyvalue
 #define pgagetprintfrequencyvalue_       pgagetprintfrequencyvalue
+#define pgasetmultiobjprecision_         pgasetmultiobjprecision
+#define pgagetmultiobjprecision_         pgagetmultiobjprecision
 #define pgaprintpopulation_              pgaprintpopulation
 #define pgaprintindividual_              pgaprintindividual
 #define pgaprintstring_                  pgaprintstring
@@ -724,6 +797,7 @@ privately owned rights.
 #define pgagetmaxgaitervalue_            pgagetmaxgaitervalue
 #define pgasetmaxnochangevalue_          pgasetmaxnochangevalue
 #define pgasetmaxsimilarityvalue_        pgasetmaxsimilarityvalue
+#define pgagetmaxsimilarityvalue_        pgagetmaxsimilarityvalue
 /* system.c */
 #define pgaerror_                        pgaerror
 #define pgadestroy_                      pgadestroy
@@ -745,6 +819,7 @@ privately owned rights.
 #define pgagetbestindex_                 pgagetbestindex
 #define pgagetbestreport_                pgagetbestreport
 #define pgagetbestreportindex_           pgagetbestreportindex
+#define pgautilhash_                     pgautilhash
 #endif
 
 /* binary.c */
@@ -770,6 +845,13 @@ void pgasetnumconstraint_(PGAContext **ftx, int *n);
 int pgagetnumconstraint_(PGAContext **ftx);
 void pgasetsumconstraintsflag_(PGAContext **ftx, int *n);
 int pgagetsumconstraintsflag_(PGAContext **ftx);
+void pgasetepsilongeneration_ (PGAContext **ftx, int *n);
+int pgagetepsilongeneration_ (PGAContext **ftx);
+void pgasetepsilonexponent_ (PGAContext **ftx, float *n);
+float pgagetepsilonexponent_ (PGAContext **ftx);
+void pgasetepsilontheta_ (PGAContext **ftx, int *n);
+int pgagetepsilontheta_ (PGAContext **ftx);
+void pgasetoutputfile_ (PGAContext **ftx, char *name);
 /* cross.c */
 void pgacrossover_(PGAContext **ftx, int *m1, int *m2, int *oldpop, int *t1,
      int *t2, int *newpop);
@@ -778,8 +860,15 @@ double pgagetcrossoverprob_(PGAContext **ftx);
 double pgagetuniformcrossoverprob_(PGAContext **ftx);
 void pgasetcrossovertype_(PGAContext **ftx, int *crossover_type);
 void pgasetcrossoverprob_(PGAContext **ftx, double *crossover_prob);
-void pgasetuniformcrossoverprob_(PGAContext **ftx,
-     double *uniform_cross_prob);
+void pgasetuniformcrossoverprob_(PGAContext **ftx, double *uniform_cross_prob);
+void pgasetcrossoverboundedflag_ (PGAContext **ftx, int *val);
+int pgagetcrossoverboundedflag_ (PGAContext **ftx);
+void pgasetcrossoverbouncebackflag_ (PGAContext **ftx, int *val);
+int pgagetcrossoverbouncebackflag_ (PGAContext **ftx);
+void pgasetcrossoversbxeta_ (PGAContext **ftx, double *val);
+double pgagetcrossoversbxeta_ (PGAContext **ftx);
+void pgasetcrossoversbxonceperstring_ (PGAContext **ftx, int *val);
+int pgagetcrossoversbxonceperstring_ (PGAContext **ftx);
 /* debug.c */
 void pgadebugprint_(PGAContext **ftx, int *level, char *funcname, char *msg,
      int *datatype, void *data, int len1, int len2);
@@ -788,7 +877,7 @@ void pgacleardebuglevel_(PGAContext **ftx, int *level);
 void pgasetdebuglevelbyname_(PGAContext **ftx, char *name, int len);
 void pgacleardebuglevelbyname_(PGAContext **ftx, char *name, int len);
 /* duplcate.c */
-int pgaduplicate_(PGAContext **ftx, int *j, int *pop1, int *pop2, int *n);
+int pgaduplicate_(PGAContext **ftx, int *j, int *pop1, int *pop2);
 void pgachange_(PGAContext **ftx, int *j, int *popindex);
 void pgasetnoduplicatesflag_(PGAContext **ftx, int *no_dup);
 int pgagetnoduplicatesflag_(PGAContext **ftx);
@@ -872,6 +961,10 @@ void pgasetdedither_ (PGAContext **ftx, double *val);
 double pgagetdedither_ (PGAContext **ftx);
 void pgasetdeditherperindividual_ (PGAContext **ftx, int *val);
 int pgagetdeditherperindividual_ (PGAContext **ftx);
+void pgasetmutationpolyeta_ (PGAContext **ftx, double *val);
+double pgagetmutationpolyeta_ (PGAContext **ftx);
+void pgasetmutationpolyvalue_ (PGAContext **ftx, double *val);
+double pgagetmutationpolyvalue_ (PGAContext **ftx);
 /* parallel.c */
 void pgarungm_(PGAContext **ftx, double (*f)(PGAContext *, int, int, double *),
      MPI_Comm *comm);
@@ -901,9 +994,11 @@ int pgagetevalcount_(PGAContext **ftx);
 void pgasetmutationorcrossoverflag_(PGAContext **ftx, int *flag);
 void pgasetmutationandcrossoverflag_(PGAContext **ftx, int *flag);
 void pgasetmutationonlyflag_(PGAContext **ftx, int *flag);
+void pgasetmixingtype_ (PGAContext **ftx, int *type);
 int pgagetmutationorcrossoverflag_(PGAContext **ftx);
 int pgagetmutationandcrossoverflag_(PGAContext **ftx);
 int pgagetmutationonlyflag_(PGAContext **ftx);
+int pgagetmixingtype_ (PGAContext **ftx);
 /* pop.c */
 void pgasortpop_(PGAContext **ftx, int *pop);
 int pgagetpopsize_(PGAContext **ftx);
@@ -937,6 +1032,8 @@ void pgaprintreport_(PGAContext **ftx, char *name, int *pop, int len);
 void pgasetprintoptions_(PGAContext **ftx, int *option);
 void pgasetprintfrequencyvalue_(PGAContext **ftx, int *print_freq);
 int pgagetprintfrequencyvalue_(PGAContext **ftx);
+void pgasetmultiobjprecision_ (PGAContext **ftx, int *n);
+int pgagetmultiobjprecision_ (PGAContext **ftx);
 void pgaprintpopulation_(PGAContext **ftx, char *name, int *pop, int len);
 void pgaprintindividual_ (PGAContext **ftx, char *name, int *p,
      int *pop, int len);
@@ -958,8 +1055,8 @@ void pgasetselecttype_(PGAContext **ftx, int *select_type);
 int pgagetselecttype_(PGAContext **ftx);
 void pgasetptournamentprob_(PGAContext **ftx, double *ptournament_prob);
 double pgagetptournamentprob_(PGAContext **ftx);
-void pgasettournamentsize_(PGAContext **ftx, int *tournament_size);
-int pgagettournamentsize_(PGAContext **ftx);
+void pgasettournamentsize_(PGAContext **ftx, double *tournament_size);
+double pgagettournamentsize_(PGAContext **ftx);
 void pgasettournamentwithreplacement_(PGAContext **ftx, int *value);
 int pgagettournamentwithreplacement_(PGAContext **ftx);
 void pgasettruncationproportion_(PGAContext **ftx, double *proportion);
@@ -975,6 +1072,7 @@ void pgasetmaxgaitervalue_(PGAContext **ftx, int *maxiter);
 int pgagetmaxgaitervalue_(PGAContext **ftx);
 void pgasetmaxnochangevalue_(PGAContext **ftx, int *max_no_change);
 void pgasetmaxsimilarityvalue_(PGAContext **ftx, int *max_similarity);
+int pgagetmaxsimilarityvalue_(PGAContext **ftx);
 /* system.c */
 void pgaerror_(PGAContext **ftx, char *msg, int *level, int *datatype,
      void **data, int len);
@@ -997,6 +1095,7 @@ int pgagetworstindex_ (PGAContext **ftx, int *pop);
 int pgagetbestindex_ (PGAContext **ftx, int *pop);
 double pgagetbestreport_ (PGAContext **ftx, int *pop, int *idx);
 int pgagetbestreportindex_ (PGAContext **ftx, int *pop, int *idx);
+int pgautilhash_ (void *data, size_t *sz, int *initval);
 
 /* binary.c */
 void pgasetbinaryallele_(PGAContext **ftx, int *p, int *pop, int *i,
@@ -1107,6 +1206,41 @@ int pgagetsumconstraintsflag_(PGAContext **ftx)
      return PGAGetSumConstraintsFlag  (*ftx);
 }
 
+void pgasetepsilongeneration_ (PGAContext **ftx, int *n)
+{
+    PGASetEpsilonGeneration (*ftx, *n);
+}
+
+int pgagetepsilongeneration_ (PGAContext **ftx)
+{
+    return PGAGetEpsilonGeneration (*ftx);
+}
+
+void pgasetepsilonexponent_ (PGAContext **ftx, float *n)
+{
+    PGASetEpsilonExponent (*ftx, *n);
+}
+
+float pgagetepsilonexponent_ (PGAContext **ftx)
+{
+    return PGAGetEpsilonExponent (*ftx);
+}
+
+void pgasetepsilontheta_ (PGAContext **ftx, int *n)
+{
+    PGASetEpsilonTheta (*ftx, *n);
+}
+
+int pgagetepsilontheta_ (PGAContext **ftx)
+{
+    return PGAGetEpsilonTheta (*ftx);
+}
+
+void pgasetoutputfile_ (PGAContext **ftx, char *name)
+{
+    PGASetOutputFile (*ftx, name);
+}
+
 /* cross.c */
 void pgacrossover_(PGAContext **ftx, int *m1, int *m2, int *oldpop, int *t1,
      int *t2, int *newpop)
@@ -1151,84 +1285,150 @@ void pgasetuniformcrossoverprob_(PGAContext **ftx,
      PGASetUniformCrossoverProb  (*ftx, *uniform_cross_prob);
 }
 
+void pgasetcrossoverboundedflag_ (PGAContext **ftx, int *val)
+{
+     PGASetCrossoverBoundedFlag (*ftx, *val);
+}
+
+int pgagetcrossoverboundedflag_ (PGAContext **ftx)
+{
+     return PGAGetCrossoverBoundedFlag (*ftx);
+}
+
+void pgasetcrossoverbouncebackflag_ (PGAContext **ftx, int *val)
+{
+     PGASetCrossoverBounceBackFlag (*ftx, *val);
+}
+
+int pgagetcrossoverbouncebackflag_ (PGAContext **ftx)
+{
+     return PGAGetCrossoverBounceBackFlag (*ftx);
+}
+
+void pgasetcrossoversbxeta_ (PGAContext **ftx, double *val)
+{
+     PGASetCrossoverSBXEta (*ftx, *val);
+}
+
+double pgagetcrossoversbxeta_ (PGAContext **ftx)
+{
+     return PGAGetCrossoverSBXEta (*ftx);
+}
+
+void pgasetcrossoversbxonceperstring_ (PGAContext **ftx, int *val)
+{
+     PGASetCrossoverSBXOncePerString (*ftx, *val);
+}
+
+int pgagetcrossoversbxonceperstring_ (PGAContext **ftx)
+{
+     return PGAGetCrossoverSBXOncePerString (*ftx);
+}
+
 #if OPTIMIZE==0
 /* debug.c */
-void pgadebugprint_(PGAContext **ftx, int *level, char *funcname, char *msg,
-     int *datatype, void *data, int len1, int len2)
+void pgadebugprint_
+    ( PGAContext **ftx, int *level, char *funcname
+    , char *msg, int *datatype, void *data, int len1, int len2
+    )
 /* FORTRAN implicitly passes the lengths of funcname and msg into len1
    and len2, respectively */
 {
-     if (funcname[len1] != 0 || msg[len2] != 0)
-	  funcname[len1] = msg[len2] = 0;
-     PGADebugPrint(*ftx, *level, funcname, msg, *datatype, data );
+     if (funcname [len1] != 0 || msg [len2] != 0)
+	  funcname [len1] = msg [len2] = 0;
+     PGADebugPrint (*ftx, *level, funcname, msg, *datatype, data );
 }
 #else
-void pgadebugprint_(PGAContext **ftx, int *level, char *funcname, char *msg,
-		    int *datatype, void *data, int len1, int len2)
+void pgadebugprint_
+    ( PGAContext **ftx, int *level, char *funcname
+    , char *msg, int *datatype, void *data, int len1, int len2
+    )
 {
-     printf("PGADebugPrint is not supported in the optimized version of PGAPack.\n");
+     fprintf
+        ( stderr
+        , "PGADebugPrint is not supported in the optimized version "
+          "of PGAPack.\n"
+        );
 }
 #endif
 
 #if OPTIMIZE==0
-void pgasetdebuglevel_(PGAContext **ftx, int *level)
+void pgasetdebuglevel_ (PGAContext **ftx, int *level)
 {
-    PGASetDebugLevel(*ftx, *level);
+    PGASetDebugLevel (*ftx, *level);
 }
 #else
-void pgasetdebuglevel_(PGAContext **ftx, int *level)
+void pgasetdebuglevel_ (PGAContext **ftx, int *level)
 {
-    printf("PGASetDebugLevel is not supported in the optimized version of PGAPack.\n");
+    fprintf
+        ( stderr
+        , "PGASetDebugLevel is not supported in the optimized version "
+          "of PGAPack.\n"
+        );
 }
 #endif
 #if OPTIMIZE==0
-void pgacleardebuglevel_(PGAContext **ftx, int *level)
+void pgacleardebuglevel_ (PGAContext **ftx, int *level)
 {
-    PGAClearDebugLevel(*ftx, *level);
+    PGAClearDebugLevel (*ftx, *level);
 }
 #else
-void pgacleardebuglevel_(PGAContext **ftx, int *level)
+void pgacleardebuglevel_ (PGAContext **ftx, int *level)
 {
-    printf("PGAClearDebugLevel is not supported in the optimized version of PGAPack.\n");
+    fprintf
+        ( stderr
+        , "PGAClearDebugLevel is not supported in the optimized version "
+          "of PGAPack.\n"
+        );
 }
 #endif
 #if OPTIMIZE==0
-void pgasetdebuglevelbyname_(PGAContext **ftx, char *name, int len)
+void pgasetdebuglevelbyname_ (PGAContext **ftx, char *name, int len)
 {
-    if (name[len] != 0)  name[len] = 0;
-    PGASetDebugLevelByName(*ftx, name);
+    if (name [len] != 0) {
+        name [len] = 0;
+    }
+    PGASetDebugLevelByName (*ftx, name);
 }
 #else
-void pgasetdebuglevelbyname_(PGAContext **ftx, char *name, int len)
+void pgasetdebuglevelbyname_ (PGAContext **ftx, char *name, int len)
 {
-    printf("PGASetDebugLevelByName is not supported in the optimized version of PGAPack.\n");
+    fprintf
+        ( stderr
+        , "PGASetDebugLevelByName is not supported in the "
+          "optimized version of PGAPack.\n"
+        );
 }
 #endif
 #if OPTIMIZE==0
-void pgacleardebuglevelbyname_(PGAContext **ftx, char *name, int len)
+void pgacleardebuglevelbyname_ (PGAContext **ftx, char *name, int len)
 {
-    if (name[len] != 0)  name[len] = 0;
-    PGAClearDebugLevelByName(*ftx, name);
+    if (name [len] != 0) {
+        name [len] = 0;
+    }
+    PGAClearDebugLevelByName (*ftx, name);
 }
 #else
-void pgacleardebuglevelbyname_(PGAContext **ftx, char *name, int len)
+void pgacleardebuglevelbyname_ (PGAContext **ftx, char *name, int len)
 {
-    printf("PGAClearDebugLevelByName is not supported in the optimized version of PGAPack.\n");
+    fprintf
+        ( stderr
+        , "PGAClearDebugLevelByName is not supported in the "
+          "optimized version of PGAPack.\n"
+        );
 }
 #endif
 /* duplcate.c */
-int pgaduplicate_(PGAContext **ftx, int *j, int *pop1, int *pop2, int *n)
+int pgaduplicate_(PGAContext **ftx, int *j, int *pop1, int *pop2)
 {
-     return PGADuplicate(*ftx,
-		  *j == PGA_TEMP1 || *j == PGA_TEMP2 ? *j : *j-1,
-		  *pop1, *pop2, *n);
+     int jj = (*j == PGA_TEMP1 || *j == PGA_TEMP2) ? *j : *j - 1;
+     return PGADuplicate (*ftx, jj, *pop1, *pop2);
 }
 
 void pgachange_(PGAContext **ftx, int *j, int *popindex)
 {
-     PGAChange(*ftx,
-	   *j == PGA_TEMP1 || *j == PGA_TEMP2 ? *j : *j-1,
-	   *popindex);
+     int jj = (*j == PGA_TEMP1 || *j == PGA_TEMP2) ? *j : *j - 1;
+     PGAChange (*ftx, jj, *popindex);
 }
 
 void pgasetnoduplicatesflag_(PGAContext **ftx, int *no_dup)
@@ -1245,14 +1445,14 @@ int pgagetnoduplicatesflag_(PGAContext **ftx)
 void pgasetevaluation_
     (PGAContext **ftx, int *p, int *pop, double *val)
 {
-     PGASetEvaluation
-        (*ftx, *p == PGA_TEMP1 || *p == PGA_TEMP2 ? *p : *p-1, *pop, *val);
+     _PGASetEvaluation
+        (*ftx, *p == PGA_TEMP1 || *p == PGA_TEMP2 ? *p : *p-1, *pop, *val, NULL);
 }
 
 double pgagetevaluation_ (PGAContext **ftx, int *p, int *pop)
 {
-     return PGAGetEvaluation
-        (*ftx, *p == PGA_TEMP1 || *p == PGA_TEMP2 ? *p : *p-1, *pop);
+     return _PGAGetEvaluation
+        (*ftx, *p == PGA_TEMP1 || *p == PGA_TEMP2 ? *p : *p-1, *pop, NULL);
 }
 
 void pgasetevaluationaux_
@@ -1624,6 +1824,26 @@ int pgagetdeditherperindividual_ (PGAContext **ftx)
     return PGAGetDEDitherPerIndividual (*ftx);
 }
 
+void pgasetmutationpolyeta_ (PGAContext **ftx, double *val)
+{
+    PGASetMutationPolyEta (*ftx, *val);
+}
+
+double pgagetmutationpolyeta_ (PGAContext **ftx)
+{
+    return PGAGetMutationPolyEta (*ftx);
+}
+
+void pgasetmutationpolyvalue_ (PGAContext **ftx, double *val)
+{
+    PGASetMutationPolyValue (*ftx, *val);
+}
+
+double pgagetmutationpolyvalue_ (PGAContext **ftx)
+{
+    return PGAGetMutationPolyValue (*ftx);
+}
+
 
 /* parallel.c */
 void pgarungm_(PGAContext **ftx, double (*f)(PGAContext *, int, int, double *),
@@ -1755,6 +1975,11 @@ void pgasetmutationonlyflag_(PGAContext **ftx, int *flag)
     PGASetMutationOnlyFlag(*ftx, *flag);
 }
 
+void pgasetmixingtype_ (PGAContext **ftx, int *type)
+{
+    PGASetMixingType (*ftx, *type);
+}
+
 int pgagetmutationorcrossoverflag_(PGAContext **ftx)
 {
      return PGAGetMutationOrCrossoverFlag  (*ftx);
@@ -1770,6 +1995,11 @@ int pgagetmutationonlyflag_(PGAContext **ftx)
     return PGAGetMutationOnlyFlag (*ftx);
 }
 
+int pgagetmixingtype_ (PGAContext **ftx)
+{
+    return PGAGetMixingType (*ftx);
+}
+
 /* pop.c */
 void pgasortpop_(PGAContext **ftx, int *pop)
 {
@@ -1944,6 +2174,16 @@ int pgagetprintfrequencyvalue_(PGAContext **ftx)
      return PGAGetPrintFrequencyValue  (*ftx);
 }
 
+void pgasetmultiobjprecision_ (PGAContext **ftx, int *n)
+{
+    PGASetMultiObjPrecision (*ftx, *n);
+}
+
+int pgagetmultiobjprecision_ (PGAContext **ftx)
+{
+    return PGAGetMultiObjPrecision (*ftx);
+}
+
 void pgaprintpopulation_(PGAContext **ftx, char *name, int *pop, int len)
 /* FORTRAN implicitly passes the length of name into len. */
 {
@@ -2139,12 +2379,12 @@ double pgagetptournamentprob_(PGAContext **ftx)
      return PGAGetPTournamentProb  (*ftx);
 }
 
-void pgasettournamentsize_(PGAContext **ftx, int *tournament_size)
+void pgasettournamentsize_(PGAContext **ftx, double *tournament_size)
 {
     PGASetTournamentSize(*ftx, *tournament_size);
 }
 
-int pgagettournamentsize_(PGAContext **ftx)
+double pgagettournamentsize_(PGAContext **ftx)
 {
     return PGAGetTournamentSize(*ftx);
 }
@@ -2220,6 +2460,11 @@ void pgasetmaxsimilarityvalue_(PGAContext **ftx, int *max_similarity)
      PGASetMaxSimilarityValue  (*ftx, *max_similarity);
 }
 
+int pgagetmaxsimilarityvalue_ (PGAContext **ftx)
+{
+     return PGAGetMaxSimilarityValue  (*ftx);
+}
+
 /* system.c */
 void pgaerror_(PGAContext **ftx, char *msg, int *level, int *datatype,
      void **data, int len)
@@ -2320,3 +2565,9 @@ int pgagetbestreportindex_(PGAContext **ftx, int *pop, int *idx)
 {
      return PGAGetBestReportIndex(*ftx, *pop, *idx - 1) + 1;
 }
+
+int pgautilhash_ (void *data, size_t *sz, int *initval)
+{
+     int value = (int)PGAUtilHash (data, *sz, *initval);
+     return value;
+}
