@@ -139,6 +139,7 @@ void PGASetMutationType( PGAContext *ctx, int mutation_type)
      case PGA_MUTATION_GAUSSIAN:
      case PGA_MUTATION_PERMUTE:
      case PGA_MUTATION_DE:
+     case PGA_MUTATION_POLY:
           ctx->ga.MutationType = mutation_type;
           break;
      default:
@@ -170,19 +171,19 @@ void PGASetMutationType( PGAContext *ctx, int mutation_type)
       mutatetype = PGAGetMutationType(ctx);
       switch (mutatetype) {
       case PGA_MUTATION_CONSTANT:
-          printf("Mutation Type = PGA_MUTATION_CONSTANT\n");
+          printf ("Mutation Type = PGA_MUTATION_CONSTANT\n");
           break;
       case PGA_MUTATION_RANGE:
-          printf("Mutation Type = PGA_MUTATION_RANGE\n");
+          printf ("Mutation Type = PGA_MUTATION_RANGE\n");
           break;
       case PGA_MUTATION_UNIFORM:
-          printf("Mutation Type = PGA_MUTATION_UNIFORM\n");
+          printf ("Mutation Type = PGA_MUTATION_UNIFORM\n");
           break;
       case PGA_MUTATION_GAUSSIAN:
-          printf("Mutation Type = PGA_MUTATION_GAUSSIAN\n");
+          printf ("Mutation Type = PGA_MUTATION_GAUSSIAN\n");
           break;
       case PGA_MUTATION_PERMUTE:
-          printf("Mutation Type = PGA_MUTATION_PERMUTE\n");
+          printf ("Mutation Type = PGA_MUTATION_PERMUTE\n");
           break;
       }
 
@@ -528,6 +529,112 @@ double PGAGetMutationProb (PGAContext *ctx)
     return(ctx->ga.MutationProb);
 }
 
+/*U****************************************************************************
+   PGASetMutationEtamu for polynomial mutation
+
+   Category: Operators
+
+   Inputs:
+      ctx - context variable
+      eta - the polynomial mutation eta
+
+   Outputs:
+      None
+
+   Example:
+      PGAContext *ctx;
+      :
+      PGASetMutationPolyEta (ctx, 200);
+
+****************************************************************************U*/
+void PGASetMutationPolyEta (PGAContext *ctx, double eta)
+{
+    if (eta < 0.0) {
+        PGAError
+            ( ctx, "PGASetMutationPolyEta: Invalid value of eta:"
+            , PGA_FATAL, PGA_DOUBLE, (void *) &eta
+            );
+    }
+    ctx->ga.MutatePolyEta = eta;
+}
+
+/*U***************************************************************************
+   PGAGetMutationPolyEta - Returns the eta for polynomial mutation
+
+   Category: Operators
+
+   Inputs:
+      ctx - context variable
+
+   Outputs:
+      The eta for polynomial mutation
+
+   Example:
+      PGAContext *ctx;
+      double eta;
+      :
+      eta = PGAGetMutationPolyEta (ctx);
+
+***************************************************************************U*/
+double PGAGetMutationPolyEta (PGAContext *ctx)
+{
+    PGAFailIfNotSetUp("PGAGetMutationPolyEta");
+    return(ctx->ga.MutatePolyEta);
+}
+
+/*U****************************************************************************
+   PGASetMutationPolyValue - Specifies the constant for polynomial mutation
+
+   Category: Operators
+
+   Inputs:
+      ctx - context variable
+      v   - the polynomial mutation constant
+
+   Outputs:
+      None
+
+   Example:
+      PGAContext *ctx;
+      :
+      PGASetMutationPolyValue (ctx, 2.5);
+
+****************************************************************************U*/
+void PGASetMutationPolyValue (PGAContext *ctx, double v)
+{
+    if (v < 0.0) {
+        PGAError
+            ( ctx, "PGASetMutationPolyConstant: Invalid value:"
+            , PGA_FATAL, PGA_DOUBLE, (void *) &v
+            );
+    }
+    ctx->ga.MutatePolyValue = v;
+}
+
+/*U***************************************************************************
+   PGAGetMutationPolyValue - Returns the value for polynomial mutation
+
+   Category: Operators
+
+   Inputs:
+      ctx - context variable
+
+   Outputs:
+      The value for polynomial mutation
+
+   Example:
+      PGAContext *ctx;
+      double v;
+      :
+      v = PGAGetMutationPolyValue (ctx);
+
+***************************************************************************U*/
+double PGAGetMutationPolyValue (PGAContext *ctx)
+{
+    PGAFailIfNotSetUp("PGAGetMutationPolyValue");
+    return(ctx->ga.MutatePolyValue);
+}
+
 
 /*U****************************************************************************
   PGASetDEVariant - sets the variant used for Differential Evolution.
@@ -587,10 +694,10 @@ void PGASetDEVariant (PGAContext *ctx, int variant)
       variant = PGAGetDEVariant(ctx);
       switch (variant) {
       case PGA_DE_VARIANT_RAND:
-          printf("DE Variant = PGA_DE_VARIANT_RAND\n");
+          printf ("DE Variant = PGA_DE_VARIANT_RAND\n");
           break;
       case PGA_DE_VARIANT_BEST:
-          printf("DE Variant = PGA_DE_VARIANT_BEST\n");
+          printf ("DE Variant = PGA_DE_VARIANT_BEST\n");
           break;
       }
 
