@@ -75,7 +75,7 @@ void PGARunGM(PGAContext *ctx, double (*f)(PGAContext *, int, int, double *),
 	      MPI_Comm comm)
 {
     int       rank, Restarted;
-    void    (*CreateNewGeneration)(PGAContext *, int, int);
+    void    (*CreateNewGeneration)(PGAContext *, int, int) = NULL;
 
     /*  Let this be warned:
      *  The communicator is NOT duplicated.  There might be problems with
@@ -98,18 +98,51 @@ void PGARunGM(PGAContext *ctx, double (*f)(PGAContext *, int, int, double *),
     PGAEvaluate (ctx, PGA_OLDPOP, f, comm);
     if (rank == 0) {
         int st = PGAGetSelectType (ctx);
+        /* If epsilon constraints are used */
+        if (ctx->ga.NumConstraint && ctx->ga.EpsilonGeneration) {
+            int idx;
+            PGAIndividual *ind;
+            /* Sort population by auxiliary eval */
+	    /* No need to init the indeces, filled in by PGAEvalSort */
+	    PGAEvalSort (ctx, PGA_OLDPOP, ctx->scratch.intscratch);
+            idx = ctx->scratch.intscratch [ctx->ga.EpsilonTheta];
+            ind = PGAGetIndividual (ctx, idx, PGA_OLDPOP);
+            assert (ind->auxtotalok);
+            ctx->ga.Epsilon = ctx->ga.Epsilon_0 = ind->auxtotal;
+            if (!ctx->ga.EpsilonExponent) {
+                double l10 = log (10);
+                assert (ctx->ga.EffEpsExponent == 0);
+                ctx->ga.EffEpsExponent =
+                    (-5 - log (ctx->ga.Epsilon) / l10) / (log (0.05) / l10);
+                if (ctx->ga.EffEpsExponent < PGA_EPSILON_EXPONENT_MIN) {
+                    ctx->ga.EffEpsExponent = PGA_EPSILON_EXPONENT_MIN;
+                }
+                if (ctx->ga.EffEpsExponent > PGA_EPSILON_EXPONENT_MAX) {
+                    ctx->ga.EffEpsExponent = PGA_EPSILON_EXPONENT_MAX;
+                }
+            }
+        }
         PGAUpdateBest (ctx, PGA_OLDPOP);
         if (st == PGA_SELECT_SUS || st == PGA_SELECT_PROPORTIONAL) {
             PGAFitness (ctx, PGA_OLDPOP);
         }
     }
 
-    if (PGAGetMutationOnlyFlag(ctx)) {
-	CreateNewGeneration = PGARunMutationOnly;
-    } else if (PGAGetMutationOrCrossoverFlag(ctx)) {
+    switch (PGAGetMixingType (ctx)) {
+    case PGA_MIX_MUTATE_OR_CROSS:
 	CreateNewGeneration = PGARunMutationOrCrossover;
-    } else {
+        break;
+    case PGA_MIX_MUTATE_AND_CROSS:
 	CreateNewGeneration = PGARunMutationAndCrossover;
+        break;
+    case PGA_MIX_MUTATE_ONLY:
+	CreateNewGeneration = PGARunMutationOnly;
+        break;
+    case PGA_MIX_TRADITIONAL:
+	CreateNewGeneration = PGARunMutationAndCrossover;
+        break;
+    default:
+        assert (0);
     }
 
     while (!PGADone(ctx, comm)) {
@@ -140,6 +173,26 @@ void PGARunGM(PGAContext *ctx, double (*f)(PGAContext *, int, int, double *),
             if (st == PGA_SELECT_SUS || st == PGA_SELECT_PROPORTIONAL) {
                 PGAFitness (ctx, PGA_NEWPOP);
             }
+            /* If epsilon constraints are used */
+            if (ctx->ga.NumConstraint && ctx->ga.EpsilonGeneration) {
+                /* Smaller Exponent after generation EpsTLambda */
+                if (ctx->ga.EpsTLambda && ctx->ga.iter == ctx->ga.EpsTLambda) {
+                    assert (ctx->ga.EpsilonExponent == 0);
+                    ctx->ga.EffEpsExponent = 0.3 * ctx->ga.EffEpsExponent
+                                           + 0.7 * PGA_EPSILON_EXPONENT_MIN;
+                }
+                if (ctx->ga.iter >= ctx->ga.EpsilonGeneration) {
+                    ctx->ga.Epsilon = 0;
+                } else {
+                    ctx->ga.Epsilon =
+                        ( ctx->ga.Epsilon_0
+                        * pow ( 1.0
+                              - (double)ctx->ga.iter / ctx->ga.EpsilonGeneration
+                              , ctx->ga.EffEpsExponent
+                              )
+                        );
+                }
+            }
         }
 
 	/*  If the GA wasn't restarted, update the generation and print
@@ -148,8 +201,9 @@ void PGARunGM(PGAContext *ctx, double (*f)(PGAContext *, int, int, double *),
 	 */
 	if (!Restarted) {
 	    PGAUpdateGeneration(ctx, comm);
-	    if (rank == 0)
-		PGAPrintReport(ctx, stdout, PGA_OLDPOP);
+	    if (rank == 0) {
+		PGAPrintReport (ctx, ctx->ga.OutputFile, PGA_OLDPOP);
+            }
 	}
     }
 
@@ -159,35 +213,53 @@ void PGARunGM(PGAContext *ctx, double (*f)(PGAContext *, int, int, double *),
         int numcon = PGAGetNumConstraint (ctx);
         if (numaux == numcon) {
             int best_p = PGAGetBestIndex (ctx, pop);
-            printf
-                ( "The Best Evaluation: %e"
-                , PGAGetEvaluation (ctx, best_p, pop)
+            fprintf
+                ( ctx->ga.OutputFile
+                , "The Best Evaluation: %e"
+                , _PGAGetEvaluation (ctx, best_p, pop, NULL)
                 );
             if (numaux) {
-                printf (" Constraints: %e", PGAGetAuxTotal (ctx, best_p, pop));
+                fprintf
+                    ( ctx->ga.OutputFile
+                    , " Constraints: %e", PGAGetAuxTotal (ctx, best_p, pop)
+                    );
             }
-            printf (".\n");
-            printf ("The Best String:\n");
-            PGAPrintString (ctx, stdout, best_p, pop);
+            fprintf (ctx->ga.OutputFile, ".\n");
+            fprintf (ctx->ga.OutputFile, "The Best String:\n");
+            PGAPrintString (ctx, ctx->ga.OutputFile, best_p, pop);
         } else {
             int i, k;
+            char s [34];
+            int p = ctx->rep.MOPrecision + 6;
+            sprintf (s, "F %%5d %%%d.%de\n", p, ctx->rep.MOPrecision);
             PGAIndividual *ind = PGAGetIndividual (ctx, 0, pop);
             for (k=0; k<numaux+1; k++) {
-                printf ("The Best (%d) evaluation: %e\n", k, ctx->rep.Best [k]);
+                fprintf
+                    ( ctx->ga.OutputFile
+                    , "The Best (%d) evaluation: %e\n", k, ctx->rep.Best [k]
+                    );
             }
-            printf ("The Nondominated Strings:\n");
-            for (i=0; i<ctx->ga.PopSize; i++) {
+            fprintf (ctx->ga.OutputFile, "The Nondominated Strings:\n");
+            for (i=0; i<ctx->ga.PopSize; i++, ind++) {
+                int j;
                 if (ind->rank == 0) {
+                    for (j=0; j<numcon; j++) {
+                        if (ind->auxeval [numaux - numcon + j] > 0) {
+                            break;
+                        }
+                    }
+                    if (j < numcon) {
+                        continue;
+                    }
                     for (k=0; k<numaux+1; k++) {
                         double e = (k==0) ? ind->evalue : ind->auxeval [k-1];
-                        printf ("F %5d %20.14e\n", k, e);
+                        fprintf (ctx->ga.OutputFile, s, k, e);
                     }
-                    PGAPrintString (ctx, stdout, i, pop);
+                    PGAPrintString (ctx, ctx->ga.OutputFile, i, pop);
                 }
-                ind++;
             }
         }
-	fflush (stdout);
+	fflush (ctx->ga.OutputFile);
     }
     PGADebugExited ("PGARunGM");
 }
@@ -414,8 +486,8 @@ void PGAEvaluateCoop(PGAContext *ctx, int pop,
 	    PGASetEvaluation (ctx, p, pop, e, aux);
             ctx->rep.nevals++;
 #if DEBUG_EVAL
-	    printf ("%4d: %10.8e Local\n", p, e);
-            fflush (stdout);
+            fprintf (stdout, "%4d: %10.8e Local\n", p, e);
+            fflush  (stdout);
 #endif
 	}
 	
@@ -425,8 +497,8 @@ void PGAEvaluateCoop(PGAContext *ctx, int pop,
             PGASetEvaluationUpToDateFlag (ctx, q, pop, PGA_TRUE);
             ctx->rep.nevals++;
 #if DEBUG_EVAL
-	    printf ("%4d: %10.8e Slave %d\n", p, e, 1);
-            fflush (stdout);
+            fprintf (stdout, "%4d: %10.8e Slave %d\n", p, e, 1);
+            fflush  (stdout);
 #endif
 	    q = -1;
 	}
@@ -488,7 +560,8 @@ void PGAEvaluateMS(PGAContext *ctx, int pop,
 	    work[s] = k;
 	    PGASendIndividual(ctx, k, pop, s, PGA_COMM_STRINGTOEVAL, comm);
 #if DEBUG_EVAL
-	    printf("%4d: Sent to slave %d.\n", k, s); fflush(stdout);
+            fprintf (stdout, "%4d: Sent to slave %d.\n", k, s);
+            fflush  (stdout);
 #endif
 	    sentout++;
 	    s++;
@@ -509,17 +582,21 @@ void PGAEvaluateMS(PGAContext *ctx, int pop,
      *  will return an evaluation and get a new one immediately.
      */
     while(k<ctx->ga.PopSize) {
-	/*  Receive the next evaluated string.  */
+        /*  Receive the next evaluated string.  */
         PGAReceiveEvaluation
             ( ctx, PGA_TEMP1, pop
             , MPI_ANY_SOURCE, PGA_COMM_EVALOFSTRING, comm, &stat
             );
-	p = work [stat.MPI_SOURCE];
+        p = work [stat.MPI_SOURCE];
         PGASetEvaluation (ctx, p, pop, tmp1->evalue, tmp1->auxeval);
         ctx->rep.nevals++;
 #if DEBUG_EVAL
-	printf("%4d: %10.8e Slave %d  Sent %d\n", work[stat.MPI_SOURCE],
-	       e, stat.MPI_SOURCE, k); fflush(stdout);
+        fprintf
+            ( stdout
+            , "%4d: %10.8e Slave %d  Sent %d\n"
+            , work[stat.MPI_SOURCE], e, stat.MPI_SOURCE, k
+            );
+        fflush (stdout);
 #endif
 	/*  Immediately send another string to be evaluated.  */
 	work [stat.MPI_SOURCE] = k;
@@ -542,8 +619,12 @@ void PGAEvaluateMS(PGAContext *ctx, int pop,
         ctx->rep.nevals++;
 	sentout--;
 #if DEBUG_EVAL
-	printf("%4d: %10.8e Slave %d\n",
-	       work[stat.MPI_SOURCE], e, stat.MPI_SOURCE); fflush(stdout);
+        fprintf
+            ( stdout
+            , "%4d: %10.8e Slave %d\n"
+            , work [stat.MPI_SOURCE], e, stat.MPI_SOURCE
+            );
+        fflush (stdout);
 #endif
     }
     free(work);
@@ -581,11 +662,15 @@ void PGAEvaluateSlave(PGAContext *ctx, int pop,
     PGADebugEntered("PGAEvaluateSlave");
 
     k = PGA_TEMP1;
-    MPI_Probe(0, MPI_ANY_TAG, comm, &stat);
-    while (stat.MPI_TAG == PGA_COMM_STRINGTOEVAL) {
+
+    MPI_Probe (0, MPI_ANY_TAG, comm, &stat);
+    while (  stat.MPI_TAG == PGA_COMM_STRINGTOEVAL
+          || stat.MPI_TAG == PGA_COMM_SERIALIZE_SIZE
+          )
+    {
         double *aux;
-	PGAReceiveIndividual(ctx, PGA_TEMP1, pop, 0, PGA_COMM_STRINGTOEVAL,
-			     comm, &stat);
+	PGAReceiveIndividual
+            (ctx, PGA_TEMP1, pop, 0, PGA_COMM_STRINGTOEVAL, comm, &stat);
 
         aux = PGAGetAuxEvaluation (ctx, PGA_TEMP1, pop);
 	if (ctx->sys.UserFortran == PGA_TRUE)
@@ -695,6 +780,118 @@ MPI_Datatype PGABuildDatatype(PGAContext *ctx, int p, int pop)
 }
 
 
+/*U****************************************************************************
+  PGABuildDatatypeHeader - Common part for building an MPI datatype
+                           for an individual
+
+  Category: Parallel
+
+  Inputs:
+    ctx      - context variable
+    counts   - Number of elements in each block
+    displs   - byte displacement array pointer
+    types    - type elements
+
+
+  Outputs:
+    A partially filled array of counts, displs, types
+    The index of the next-to-be-filled counts, displs, types
+    The returned index will be max PGA_MPI_HEADER_ELEMENTS.
+    This means callers may use a statically allocated buffer.
+
+  Example:
+    PGAContext *ctx;
+    int idx = 0;
+    int counts [7];
+    int displs [7];
+    MPI_Datatype types [7];
+    :
+    idx = PGABuildDatatypeHeader (ctx, counts, displs, types);
+    // Fill rest of counts, displs, types here and build datatype
+    counts [idx] =
+    displs [idx] =
+    types  [idx] =
+    idx++;
+    ...
+
+****************************************************************************U*/
+int PGABuildDatatypeHeader
+    ( PGAContext *ctx, int p, int pop
+    , int *counts, MPI_Aint *displs, MPI_Datatype *types
+    )
+{
+    int n = 6;
+    PGAIndividual *traveller = PGAGetIndividual (ctx, p, pop);
+    MPI_Get_address (&traveller->evalue, &displs [0]);
+    counts [0] = 1;
+    types  [0] = MPI_DOUBLE;
+    MPI_Get_address (&traveller->fitness, &displs [1]);
+    counts [1] = 1;
+    types  [1] = MPI_DOUBLE;
+    MPI_Get_address (&traveller->evaluptodate, &displs [2]);
+    counts [2] = 1;
+    types  [2] = MPI_INT;
+    MPI_Get_address (&traveller->auxtotal, &displs [3]);
+    counts [3] = 1;
+    types  [3] = MPI_DOUBLE;
+    MPI_Get_address (&traveller->auxtotalok, &displs [4]);
+    counts [4] = 1;
+    types  [4] = MPI_INT;
+    MPI_Get_address (&traveller->index, &displs [5]);
+    counts [5] = 1;
+    types  [5] = MPI_INT;
+    if (ctx->ga.NumAuxEval) {
+        MPI_Get_address (traveller->auxeval, &displs [6]);
+        counts [6] = ctx->ga.NumAuxEval;
+        types  [6] = MPI_DOUBLE;
+        n += 1;
+    }
+    assert (n <= PGA_MPI_HEADER_ELEMENTS);
+    return n;
+}
+
+
+/*U****************************************************************************
+  PGASerializedBuildDatatype - Build datatype from serialized data
+
+  Category: Parallel
+
+  Inputs:
+    ctx - context variable
+    p   - index of string
+    pop - symbolic constant of the population string p is in
+
+
+  Outputs:
+
+  Example:
+
+****************************************************************************U*/
+MPI_Datatype PGASerializedBuildDatatype (PGAContext *ctx, int p, int pop)
+{
+    int            idx = 0;
+    /* Number of elements in each block (array of integer) */
+    int            counts [PGA_MPI_HEADER_ELEMENTS + 1];
+    /* byte displacement of each block (array of integer) */
+    MPI_Aint       displs [PGA_MPI_HEADER_ELEMENTS + 1];
+    /* type of elements in each block (array of handles to datatype objects) */
+    MPI_Datatype   types  [PGA_MPI_HEADER_ELEMENTS + 1];
+    MPI_Datatype   individualtype; /* new datatype (handle) */
+
+    idx = PGABuildDatatypeHeader (ctx, p, pop, counts, displs, types);
+
+    MPI_Get_address (ctx->scratch.serialized, &displs [idx]);
+    counts [idx] = ctx->scratch.serialization_size;
+    types  [idx] = MPI_BYTE;
+    idx++;
+
+    MPI_Type_create_struct (idx, counts, displs, types, &individualtype);
+    MPI_Type_commit (&individualtype);
+
+    return individualtype;
+}
+
+
 /*U****************************************************************************
   PGASendIndividual - transmit an individual to another process
 
@@ -723,13 +920,36 @@ void PGASendIndividual(PGAContext *ctx, int p, int pop, int dest, int tag,
 {
     MPI_Datatype individualtype;
 
-    PGADebugEntered("PGASendIndividual");
+    PGADebugEntered ("PGASendIndividual");
 
-    individualtype = PGABuildDatatype(ctx, p, pop);
-    MPI_Send(MPI_BOTTOM, 1, individualtype, dest, tag, comm);
-    MPI_Type_free(&individualtype);
+    /* If we do serialization of user defined type we must first send
+     * the size
+     */
+    if (ctx->cops.Serialize) {
+        unsigned long size = 0;
+        assert (ctx->ga.datatype == PGA_DATATYPE_USER);
+        assert (ctx->scratch.serialized == NULL);
+        ctx->scratch.serialization_size = ctx->cops.Serialize
+            (ctx, p, pop, (const void **)&ctx->scratch.serialized);
+        assert (ctx->scratch.serialization_size != 0);
+        size = (unsigned long)ctx->scratch.serialization_size;
+        MPI_Send
+            ( &size, 1, MPI_UNSIGNED_LONG
+            , dest, PGA_COMM_SERIALIZE_SIZE, comm
+            );
+    }
 
-    PGADebugExited("PGASendIndividual");
+    individualtype = PGABuildDatatype (ctx, p, pop);
+    MPI_Send (MPI_BOTTOM, 1, individualtype, dest, tag, comm);
+    MPI_Type_free (&individualtype);
+
+    if (ctx->cops.Serialize) {
+        ctx->cops.SerializeFree (ctx->scratch.serialized);
+        ctx->scratch.serialized = NULL;
+        ctx->scratch.serialization_size = 0;
+    }
+
+    PGADebugExited ("PGASendIndividual");
 }
 
 /*U****************************************************************************
@@ -764,15 +984,45 @@ void PGASendIndividual(PGAContext *ctx, int p, int pop, int dest, int tag,
 void PGAReceiveIndividual(PGAContext *ctx, int p, int pop, int source, int tag,
                           MPI_Comm comm, MPI_Status *status)
 {
-     MPI_Datatype individualtype;
+    MPI_Datatype individualtype;
 
-    PGADebugEntered("PGAReceiveIndividual");
+    PGADebugEntered ("PGAReceiveIndividual");
 
-     individualtype = PGABuildDatatype(ctx, p, pop);
-     MPI_Recv(MPI_BOTTOM, 1, individualtype, source, tag, comm, status);
-     MPI_Type_free(&individualtype);
+    /* If we do serialization of user defined type we must first receive
+     * the size
+     */
+    if (ctx->cops.Serialize) {
+        unsigned long size = 0;
+        assert (ctx->ga.datatype == PGA_DATATYPE_USER);
+        assert (ctx->scratch.serialized == NULL);
+        MPI_Recv
+            ( &size, 1, MPI_UNSIGNED_LONG, source
+            , PGA_COMM_SERIALIZE_SIZE, comm, status
+            );
+        ctx->scratch.serialization_size = size;
+        assert (ctx->scratch.serialization_size != 0);
+        ctx->scratch.serialized = malloc (ctx->scratch.serialization_size);
+        if (ctx->scratch.serialized == NULL) {
+            PGAErrorPrintf
+                (ctx, PGA_FATAL, "Cannot allocate serialization buffer");
+        }
+    }
 
-    PGADebugExited("PGAReceiveIndividual");
+    individualtype = PGABuildDatatype (ctx, p, pop);
+    MPI_Recv (MPI_BOTTOM, 1, individualtype, source, tag, comm, status);
+    MPI_Type_free (&individualtype);
+
+    if (ctx->cops.Serialize) {
+        ctx->cops.Deserialize
+            ( ctx, p, pop
+            , ctx->scratch.serialized, ctx->scratch.serialization_size
+            );
+        free (ctx->scratch.serialized);
+        ctx->scratch.serialized = NULL;
+        ctx->scratch.serialization_size = 0;
+    }
+
+    PGADebugExited ("PGAReceiveIndividual");
 }
 
 /*U****************************************************************************
@@ -969,9 +1219,9 @@ int PGAGetRank (PGAContext *ctx, MPI_Comm comm)
   Example:
       PGAContext  *ctx;
       :
-      if (PGAGetNumProcs(ctx, MPI_COMM_WORLD) < 4) {
-          printf("Too few processors for decent performance!\n");
-	  exit(-1);
+      if (PGAGetNumProcs (ctx, MPI_COMM_WORLD) < 4) {
+          printf ("Too few processors for decent performance!\n");
+          exit (-1);
       }
 
 ****************************************************************************U*/
