@@ -47,6 +47,20 @@ privately owned rights.
 
 #include "pgapack.h"
 
+/* Utility function to reset the hash for all individuals */
+void reset_hash (PGAContext *ctx, int pop)
+{
+    int i = 0;
+    if (ctx->ga.NoDuplicates) {
+        PGAIndividual *ind = PGAGetIndividual (ctx, 0, pop);
+        size_t hashsize = sizeof (PGAIndividual *) * ctx->ga.PopSize;
+        memset (ctx->scratch.hashed, 0, hashsize);
+        for (i=0; i<ctx->ga.PopSize; i++, ind++) {
+            ind->next_hash = NULL;
+        }
+    }
+}
+
 /*U****************************************************************************
   PGARun - Highest level routine to execute the genetic algorithm.  It
   is called after PGACreate and PGASetup have been called.
@@ -145,7 +159,7 @@ void PGARun (PGAContext *ctx,
      /**********************************************************************/
      PGADebugExited("PGARun");
      return;
- }
+}
 
 
 /*U****************************************************************************
@@ -165,7 +179,7 @@ void PGARun (PGAContext *ctx,
   Example:
      PGAContext *ctx,
     :
-    PGARunMutationAndCrossover(ctx, PGA_OLDPOP, PGA_NEWPOP);
+    PGARunMutationAndCrossover (ctx, PGA_OLDPOP, PGA_NEWPOP);
 
 ****************************************************************************U*/
 void PGARunMutationAndCrossover (PGAContext *ctx, int oldpop, int newpop)
@@ -178,6 +192,7 @@ void PGARunMutationAndCrossover (PGAContext *ctx, int oldpop, int newpop)
 
     popsize = PGAGetPopSize (ctx);
     numreplace = PGAGetNumReplaceValue (ctx);
+    reset_hash (ctx, newpop);
     /*** first, copy n best strings to new pop ***/
     /*** Note that we do not need to do this for PGA_POPREPL_RTR   ***/
     /*** And neither for PGA_POPREPL_PAIRWISE_BEST                 ***/
@@ -192,6 +207,7 @@ void PGARunMutationAndCrossover (PGAContext *ctx, int oldpop, int newpop)
         for (i=0; i < n; i++) {
             j = PGAGetSortedPopIndex (ctx, i);
             PGACopyIndividual (ctx, j, oldpop, i, newpop);
+            PGAHashIndividual (ctx, i, newpop);
         }
     }
     pc = PGAGetCrossoverProb (ctx);
@@ -203,26 +219,43 @@ void PGARunMutationAndCrossover (PGAContext *ctx, int oldpop, int newpop)
             PGACrossover (ctx, m1, m2, oldpop, PGA_TEMP1, PGA_TEMP2, newpop);
 
             /*** mutate and copy first string to new population ***/
-            PGAMutate ( ctx, PGA_TEMP1, newpop);
-            while (PGADuplicate( ctx, PGA_TEMP1, newpop, newpop, n))
-                 PGAChange ( ctx, PGA_TEMP1, newpop );
-            PGACopyIndividual ( ctx, PGA_TEMP1, newpop, n, newpop);
+            PGAMutate (ctx, PGA_TEMP1, newpop);
+            while (PGADuplicate (ctx, PGA_TEMP1, newpop, newpop)) {
+                PGAChange (ctx, PGA_TEMP1, newpop);
+            }
+            PGACopyIndividual (ctx, PGA_TEMP1, newpop, n, newpop);
+            PGAHashIndividual (ctx, n, newpop);
             n++;
 
-            if ( n < popsize ) {
+            if (n < popsize) {
                 /*** mutate and copy second string to new population ***/
                 PGAMutate (ctx, PGA_TEMP2, newpop);
-                while (PGADuplicate (ctx, PGA_TEMP2, newpop, newpop, n))
-                     PGAChange (ctx, PGA_TEMP2, newpop);
+                while (PGADuplicate (ctx, PGA_TEMP2, newpop, newpop)) {
+                    PGAChange (ctx, PGA_TEMP2, newpop);
+                }
                 PGACopyIndividual (ctx, PGA_TEMP2, newpop, n, newpop);
+                PGAHashIndividual (ctx, n, newpop);
                 n++;
             }
-        }
-        else {
+        } else {
             PGACopyIndividual (ctx, m1, oldpop, n, newpop);
+            if (ctx->ga.MixingType == PGA_MIX_TRADITIONAL) {
+                PGAMutate (ctx, n, newpop);
+            }
+            while (PGADuplicate (ctx, n, newpop, newpop)) {
+                PGAChange (ctx, n, newpop);
+            }
+            PGAHashIndividual (ctx, n, newpop);
             n++;
             if (n < ctx->ga.PopSize) {
                 PGACopyIndividual (ctx, m2, oldpop, n, newpop);
+                if (ctx->ga.MixingType == PGA_MIX_TRADITIONAL) {
+                    PGAMutate (ctx, n, newpop);
+                }
+                while (PGADuplicate (ctx, n, newpop, newpop)) {
+                    PGAChange (ctx, n, newpop);
+                }
+                PGAHashIndividual (ctx, n, newpop);
                 n++;
             }
         }
@@ -249,7 +282,7 @@ void PGARunMutationAndCrossover (PGAContext *ctx, int oldpop, int newpop)
   Example:
     PGAContext *ctx,
     :
-    PGARunMutationOrCrossover(ctx, PGA_OLDPOP, PGA_NEWPOP);
+    PGARunMutationOrCrossover (ctx, PGA_OLDPOP, PGA_NEWPOP);
 
 ****************************************************************************U*/
 void PGARunMutationOrCrossover (PGAContext *ctx, int oldpop, int newpop)
@@ -262,6 +295,7 @@ void PGARunMutationOrCrossover (PGAContext *ctx, int oldpop, int newpop)
 
     popsize = PGAGetPopSize (ctx);
     numreplace = PGAGetNumReplaceValue (ctx);
+    reset_hash (ctx, newpop);
     /*** first, copy n best strings to new pop ***/
     /*** Note that we do not need to do this for PGA_POPREPL_RTR   ***/
     /*** And neither for PGA_POPREPL_PAIRWISE_BEST                 ***/
@@ -276,6 +310,7 @@ void PGARunMutationOrCrossover (PGAContext *ctx, int oldpop, int newpop)
         for (i=0; i < n; i++) {
             j = PGAGetSortedPopIndex (ctx, i);
             PGACopyIndividual (ctx, j, oldpop, i, newpop);
+            PGAHashIndividual (ctx, i, newpop);
         }
     }
     pc = PGAGetCrossoverProb (ctx);
@@ -287,32 +322,40 @@ void PGARunMutationOrCrossover (PGAContext *ctx, int oldpop, int newpop)
             PGACrossover (ctx, m1, m2, oldpop, PGA_TEMP1, PGA_TEMP2, newpop);
 
             /*** copy first string to new population ***/
-            while (PGADuplicate(ctx, PGA_TEMP1, newpop,  newpop, n))
+            while (PGADuplicate(ctx, PGA_TEMP1, newpop,  newpop)) {
                 PGAChange (ctx, PGA_TEMP1, newpop);
+            }
             PGACopyIndividual (ctx, PGA_TEMP1, newpop, n, newpop);
+            PGAHashIndividual (ctx, n, newpop);
             n++;
 
             if (n < popsize) {
                  /*** copy second string to new population ***/
-                 while (PGADuplicate(ctx, PGA_TEMP2, newpop,  newpop, n))
-                      PGAChange (ctx, PGA_TEMP2, newpop);
+                 while (PGADuplicate(ctx, PGA_TEMP2, newpop,  newpop)) {
+                     PGAChange (ctx, PGA_TEMP2, newpop);
+                 }
                  PGACopyIndividual (ctx, PGA_TEMP2, newpop, n, newpop);
+                 PGAHashIndividual (ctx, n, newpop);
                  n++;
             }
         } else {
              PGACopyIndividual (ctx, m1, oldpop, PGA_TEMP1, newpop);
              PGAMutate (ctx, PGA_TEMP1, newpop);
-             while (PGADuplicate (ctx, PGA_TEMP1, newpop, newpop, n))
-                  PGAChange (ctx, PGA_TEMP1, newpop);
+             while (PGADuplicate (ctx, PGA_TEMP1, newpop, newpop)) {
+                 PGAChange (ctx, PGA_TEMP1, newpop);
+             }
              PGACopyIndividual (ctx, PGA_TEMP1, newpop, n, newpop);
+             PGAHashIndividual (ctx, n, newpop);
              n++;
 
              if (n < popsize) {
                  PGACopyIndividual(ctx, m2, oldpop, PGA_TEMP2, newpop);
                  PGAMutate (ctx, PGA_TEMP2, newpop);
-                 while (PGADuplicate(ctx, PGA_TEMP2, newpop, newpop, n))
+                 while (PGADuplicate(ctx, PGA_TEMP2, newpop, newpop)) {
                      PGAChange (ctx, PGA_TEMP2, newpop);
+                 }
                  PGACopyIndividual (ctx, PGA_TEMP2, newpop, n, newpop);
+                 PGAHashIndividual (ctx, n, newpop);
                  n++;
              }
         }
@@ -339,7 +382,7 @@ void PGARunMutationOrCrossover (PGAContext *ctx, int oldpop, int newpop)
   Example:
     PGAContext *ctx,
     :
-    PGARunMutationOnly(ctx, PGA_OLDPOP, PGA_NEWPOP);
+    PGARunMutationOnly (ctx, PGA_OLDPOP, PGA_NEWPOP);
 
 ****************************************************************************U*/
 void PGARunMutationOnly (PGAContext *ctx, int oldpop, int newpop)
@@ -351,6 +394,7 @@ void PGARunMutationOnly (PGAContext *ctx, int oldpop, int newpop)
 
     popsize = PGAGetPopSize (ctx);
     numreplace = PGAGetNumReplaceValue (ctx);
+    reset_hash (ctx, newpop);
     /*** first, copy n best strings to new pop ***/
     /*** Note that we do not need to do this for PGA_POPREPL_RTR   ***/
     /*** And neither for PGA_POPREPL_PAIRWISE_BEST                 ***/
@@ -365,6 +409,7 @@ void PGARunMutationOnly (PGAContext *ctx, int oldpop, int newpop)
         for (i=0; i < n; i++) {
             j = PGAGetSortedPopIndex (ctx, i);
             PGACopyIndividual (ctx, j, oldpop, i, newpop);
+            PGAHashIndividual (ctx, i, newpop);
         }
     }
     /*** reproduce with mutation only to create rest of the new population ***/
@@ -372,8 +417,10 @@ void PGARunMutationOnly (PGAContext *ctx, int oldpop, int newpop)
         m = PGASelectNextIndex (ctx, oldpop);
         PGACopyIndividual (ctx, m, oldpop, n, newpop);
         PGAMutate (ctx, n, newpop);
-        while (PGADuplicate (ctx, n, newpop, newpop, n))
-             PGAChange (ctx, n, newpop);
+        while (PGADuplicate (ctx, n, newpop, newpop)) {
+            PGAChange (ctx, n, newpop);
+        }
+        PGAHashIndividual (ctx, n, newpop);
         n++;
     }
 
@@ -416,7 +463,7 @@ void PGAUpdateGeneration (PGAContext *ctx, MPI_Comm comm)
     ctx->ga.iter++;
 
     if (rank == 0) {
-        double oldbest [ctx->ga.NumAuxEval + 1];
+        DECLARE_DYNARRAY (double, oldbest, ctx->ga.NumAuxEval + 1);
         /* The three replacement schemes PGA_POPREPL_PAIRWISE_BEST,
          * PGA_POPREPL_RTR, and PGA_POPREPL_NSGA_II all replace some new
          * individuals from PGA_NEWPOP into PGA_OLDPOP. Then
@@ -446,6 +493,12 @@ void PGAUpdateGeneration (PGAContext *ctx, MPI_Comm comm)
              * Then OLDPOP/NEWPOP are exchanged
              */
             PGA_NSGA_II_Replacement (ctx);
+        } else if (ctx->ga.PopReplace == PGA_POPREPL_NSGA_III) {
+            /* This performs nondominated sorting and replaces the best
+             * individuals over both populations into OLDPOP.
+             * Then OLDPOP/NEWPOP are exchanged
+             */
+            PGA_NSGA_III_Replacement (ctx);
         }
 
 	if (ctx->rep.PrintOptions & PGA_REPORT_AVERAGE) {
@@ -526,19 +579,19 @@ void PGAUpdateGeneration (PGAContext *ctx, MPI_Comm comm)
       datatype = PGAGetDataType(ctx);
       switch (datatype) {
       case PGA_DATATYPE_BINARY:
-          printf("Data Type = PGA_DATATYPE_BINARY\n");
+          printf ("Data Type = PGA_DATATYPE_BINARY\n");
           break;
       case PGA_DATATYPE_CHARACTER:
-          printf("Data Type = PGA_DATATYPE_CHARACTER\n");
+          printf ("Data Type = PGA_DATATYPE_CHARACTER\n");
           break;
       case PGA_DATATYPE_INTEGER:
-          printf("Data Type = PGA_DATATYPE_INTEGER\n");
+          printf ("Data Type = PGA_DATATYPE_INTEGER\n");
           break;
       case PGA_DATATYPE_REAL:
-          printf("Data Type = PGA_DATATYPE_REAL\n");
+          printf ("Data Type = PGA_DATATYPE_REAL\n");
           break;
       case PGA_DATATYPE_USER:
-          printf("Data Type = PGA_DATATYPE_USER\n");
+          printf ("Data Type = PGA_DATATYPE_USER\n");
           break;
       }
 
@@ -572,10 +625,10 @@ int PGAGetDataType (PGAContext *ctx)
       optdir = PGAGetOptDirFlag(ctx);
       switch (optdir) {
       case PGA_MAXIMIZE:
-          printf("Optimization direction = PGA_MAXIMIZE\n");
+          printf ("Optimization direction = PGA_MAXIMIZE\n");
           break;
       case PGA_MINIMIZE:
-          printf("Optimization direction = PGA_MINIMIZE\n");
+          printf ("Optimization direction = PGA_MINIMIZE\n");
           break;
       }
 
@@ -710,6 +763,7 @@ int PGAGetEvalCount (PGAContext *ctx)
 /*U****************************************************************************
   PGASetMutationOrCrossoverFlag - A boolean flag to indicate if recombination
   uses exactly one of crossover or mutation on selected strings.
+  Note: This is a legacy interface, use PGASetMixingType instead.
 
    Category: Generation
 
@@ -721,29 +775,18 @@ int PGAGetEvalCount (PGAContext *ctx)
       None
 
    Example:
-      Set the genetic algorithm to use mutation only when crossover is
-      not used.
-
-      PGAContext *ctx;
-      :
-      PGASetMutationOrCrossoverFlag(ctx,PGA_FALSE);
+      Do not use this for new code.
 
 ****************************************************************************U*/
 void PGASetMutationOrCrossoverFlag( PGAContext *ctx, int flag)
 {
     PGADebugEntered("PGASetMutationOrCrossoverFlag");
 
-     switch (flag)
-     {
-     case PGA_TRUE:
-     case PGA_FALSE:
-          ctx->ga.MutateOnlyNoCross = flag;
-          break;
-     default:
-          PGAError (ctx, "PGASetMutationOrCrossoverFlag: Invalid value of "
-                    "flag:", PGA_FATAL, PGA_INT, (void *) &flag);
-          break;
-     }
+    if (flag) {
+        ctx->ga.MixingType = PGA_MIX_MUTATE_OR_CROSS;
+    } else {
+        ctx->ga.MixingType = PGA_MIX_MUTATE_AND_CROSS;
+    }
 
     PGADebugExited("PGASetMutationOrCrossoverFlag");
 }
@@ -751,6 +794,7 @@ void PGASetMutationOrCrossoverFlag( PGAContext *ctx, int flag)
 /*U****************************************************************************
   PGASetMutationAndCrossoverFlag - A boolean flag to indicate if
   recombination uses both crossover and mutation on selected strings
+  Note: This is a legacy interface, use PGASetMixingType instead.
 
    Category: Generation
 
@@ -762,35 +806,27 @@ void PGASetMutationOrCrossoverFlag( PGAContext *ctx, int flag)
       None
 
    Example:
-      Set the genetic algorithm to use both crossover and mutation when
-      reproducing strings.
-
-      PGAContext *ctx;
-      :
-      PGASetMutationAndCrossoverFlag(ctx,PGA_FALSE);
+      Do not use this for new code.
 
 ****************************************************************************U*/
 void PGASetMutationAndCrossoverFlag( PGAContext *ctx, int flag)
 {
-    PGADebugEntered("PGASetMutationAndCrossoverFlag");
+    PGADebugEntered ("PGASetMutationAndCrossoverFlag");
 
-     switch (flag)
-     {
-     case PGA_TRUE:
-     case PGA_FALSE:
-          ctx->ga.MutateOnlyNoCross = !flag;
-          break;
-     default:
-          PGAError (ctx, "PGASetMutationAndCrossoverFlag: Invalid value of "
-                    "flag:", PGA_FATAL, PGA_INT, (void *) &flag);
-          break;
-     }
+    if (flag) {
+        ctx->ga.MixingType = PGA_MIX_MUTATE_AND_CROSS;
+    } else {
+        ctx->ga.MixingType = PGA_MIX_MUTATE_OR_CROSS;
+    }
 
-    PGADebugExited("PGASetMutationAndCrossoverFlag");
+    PGADebugExited ("PGASetMutationAndCrossoverFlag");
 }
 /*U***************************************************************************
    PGAGetMutationOrCrossoverFlag - Returns true if mutation only occurs when
    crossover does not.
+   Note: This is a legacy interface. If mixing types other than
+   PGA_MIX_MUTATE_OR_CROSS and PGA_MIX_MUTATE_AND_CROSS have been set
+   this might return wrong values, use PGAGetMixingType instead.
 
    Category: Generation
 
@@ -802,33 +838,27 @@ void PGASetMutationAndCrossoverFlag( PGAContext *ctx, int flag)
       otherwise, returns PGA_FALSE.
 
    Example:
-      PGAContext *ctx;
-      int mutatetype;
-      :
-      mutatetype = PGAGetMutationOrCrossoverFlag(ctx);
-      switch (mutatetype) {
-      case PGA_TRUE:
-          printf("Only mutating strings not undergoing crossover\n");
-          break;
-      case PGA_FALSE:
-          printf("Mutating strings only after crossover\n");
-          break;
-      }
+      Do not use this for new code.
 
 ***************************************************************************U*/
 int PGAGetMutationOrCrossoverFlag (PGAContext *ctx)
 {
-    PGADebugEntered("PGAGetMutationOrCrossoverFlag");
-    PGAFailIfNotSetUp("PGAGetMutationOrCrossoverFlag");
+    PGADebugEntered ("PGAGetMutationOrCrossoverFlag");
+    PGAFailIfNotSetUp ("PGAGetMutationOrCrossoverFlag");
 
-    PGADebugExited("PGAGetMutationOrCrossoverFlag");
+    PGADebugExited ("PGAGetMutationOrCrossoverFlag");
 
-    return(ctx->ga.MutateOnlyNoCross);
+    return (ctx->ga.MixingType == PGA_MIX_MUTATE_OR_CROSS
+           ? PGA_TRUE : PGA_FALSE
+           );
 }
 
 /*U***************************************************************************
    PGAGetMutationAndCrossoverFlag - Returns true if mutation occurs only
    when crossover does.
+   Note: This is a legacy interface. If mixing types other than
+   PGA_MIX_MUTATE_OR_CROSS and PGA_MIX_MUTATE_AND_CROSS have been set
+   this might return wrong values, use PGAGetMixingType instead.
 
    Category: Generation
 
@@ -840,33 +870,28 @@ int PGAGetMutationOrCrossoverFlag (PGAContext *ctx)
       Otherwise, returns PGA_FALSE
 
    Example:
-      PGAContext *ctx;
-      int mutatetype;
-      :
-      mutatetype = PGAGetMutationAndCrossoverFlag(ctx);
-      switch (mutatetype) {
-      case PGA_TRUE:
-          printf("Mutating strings only after crossover\n");
-          break;
-      case PGA_FALSE:
-          printf("Only mutating strings not undergoing crossover\n");
-          break;
-      }
+      Do not use this for new code.
 
 ***************************************************************************U*/
 int PGAGetMutationAndCrossoverFlag (PGAContext *ctx)
 {
-    PGADebugEntered("PGAGetMutationAndCrossoverFlag");
-    PGAFailIfNotSetUp("PGAGetMutationAndCrossoverFlag");
+    PGADebugEntered ("PGAGetMutationAndCrossoverFlag");
+    PGAFailIfNotSetUp ("PGAGetMutationAndCrossoverFlag");
 
-    PGADebugExited("PGAGetMutationAndCrossoverFlag");
+    PGADebugExited ("PGAGetMutationAndCrossoverFlag");
 
-    return(!ctx->ga.MutateOnlyNoCross);
+    return (ctx->ga.MixingType == PGA_MIX_MUTATE_AND_CROSS
+           ? PGA_TRUE : PGA_FALSE
+           );
 }
 
 /*U****************************************************************************
   PGASetMutationOnlyFlag - A boolean flag to indicate that recombination
   uses mutation only.
+  Note: This is a legacy interface, use PGASetMixingType instead.
+  Note: This will override settings of PGASetMutationOrCrossoverFlag and
+  PGASetMutationAndCrossoverFlag and will set the default
+  (PGASetMutationAndCrossoverFlag) when using PGA_FALSE as the flag.
 
    Category: Generation
 
@@ -878,30 +903,21 @@ int PGAGetMutationAndCrossoverFlag (PGAContext *ctx)
       None
 
    Example:
-      Set the genetic algorithm to use mutation only.
-
-      PGAContext *ctx;
-      :
-      PGASetMutationOnlyFlag(ctx,PGA_TRUE);
+      Do not use this for new code.
 
 ****************************************************************************U*/
 void PGASetMutationOnlyFlag (PGAContext *ctx, int flag)
 {
-     switch (flag)
-     {
-     case PGA_TRUE:
-     case PGA_FALSE:
-          ctx->ga.MutateOnly = flag;
-          break;
-     default:
-          PGAError (ctx, "PGASetMutationOrCrossoverFlag: Invalid value of "
-                    "flag:", PGA_FATAL, PGA_INT, (void *) &flag);
-          break;
-     }
+    if (flag) {
+        ctx->ga.MixingType = PGA_MIX_MUTATE_ONLY;
+    } else {
+        ctx->ga.MixingType = PGA_MIX_MUTATE_OR_CROSS;
+    }
 }
 
 /*U***************************************************************************
    PGAGetMutationOnlyFlag - Returns true if only mutation is used
+   Note: This is a legacy interface, use PGAGetMixingType instead.
 
    Category: Generation
 
@@ -913,23 +929,76 @@ void PGASetMutationOnlyFlag (PGAContext *ctx, int flag)
       Otherwise, returns PGA_FALSE
 
    Example:
-      PGAContext *ctx;
-      int mutateonly;
-      :
-      mutateonly = PGAGetMutationOnlyFlag(ctx);
-      switch (mutatetype) {
-      case PGA_TRUE:
-          printf("Mutating only\n");
-          break;
-      case PGA_FALSE:
-          printf("Mutating and/or crossover\n");
-          break;
-      }
+      Do not use this for new code.
 
 ***************************************************************************U*/
 int PGAGetMutationOnlyFlag (PGAContext *ctx)
 {
-    PGAFailIfNotSetUp("PGAGetMutationOnlyFlag");
-    return(ctx->ga.MutateOnly);
+    PGAFailIfNotSetUp ("PGAGetMutationOnlyFlag");
+    return (ctx->ga.MixingType == PGA_MIX_MUTATE_ONLY ? PGA_TRUE : PGA_FALSE);
+}
+
+/*U****************************************************************************
+  PGASetMixingType - Strategy for combining Mutation and Crossover
+
+   Category: Generation
+
+   Inputs:
+      ctx  - context variable
+      type - Type of Mutation/Crossover combination
+
+   Outputs:
+      None
+
+   Example:
+      Set the genetic algorithm to use mutation only.
+
+      PGAContext *ctx;
+      :
+      PGASetMixingType (ctx, PGA_MIX_MUTATE_ONLY);
+
+****************************************************************************U*/
+void PGASetMixingType (PGAContext *ctx, int type)
+{
+    switch (type) {
+    case PGA_MIX_MUTATE_OR_CROSS:
+    case PGA_MIX_MUTATE_AND_CROSS:
+    case PGA_MIX_MUTATE_ONLY:
+    case PGA_MIX_TRADITIONAL:
+        ctx->ga.MixingType = type;
+        break;
+    default:
+        PGAErrorPrintf
+            ( ctx, PGA_FATAL
+            , "PGASetMixingType: Invalid value of type: %d"
+            , type
+            );
+        break;
+    }
+}
+
+/*U***************************************************************************
+   PGAGetMixingType - Returns the strategy setting for combination of
+   mutation and crossover
+
+   Category: Generation
+
+   Inputs:
+      ctx - context variable
+
+   Outputs:
+      Returns the mixing type.
+
+   Example:
+      PGAContext *ctx;
+      int mixtype;
+      :
+      mixtype = PGAGetMixingType (ctx);
+
+***************************************************************************U*/
+int PGAGetMixingType (PGAContext *ctx)
+{
+    PGAFailIfNotSetUp ("PGAGetMixingType");
+    return (ctx->ga.MixingType);
 }
 
