@@ -45,9 +45,15 @@ privately owned rights.
 *              Brian P. Walenz
 *****************************************************************************/
 
-#include <assert.h>
 #include "pgapack.h"
 
+#define OPT_DIR_CMP(ctx, e1, e2) \
+    (ctx->ga.optdir == PGA_MAXIMIZE ? CMP ((e1), (e2)) : CMP ((e2), (e1)))
+#define NORMALIZE(ctx, e, u) \
+    (ctx->ga.optdir == PGA_MAXIMIZE ? ((u) - (e)) : ((e) - (u)))
+#define DENORMALIZE(ctx, e, u) \
+    (ctx->ga.optdir == PGA_MAXIMIZE ? ((u) - (e)) : ((e) + (u)))
+
 /*U****************************************************************************
    PGASortPop - Creates an (internal) array of indices according to one of
    three criteria.  If PGA_POPREPL_BEST is used (the default) the array is
@@ -206,13 +212,13 @@ int PGAGetNumReplaceValue (PGAContext *ctx)
       popreplace = PGAGetPopReplaceType(ctx);
       switch (popreplace) {
       case PGA_POPREPL_BEST:
-          printf("Replacement Strategy = PGA_POPREPL_BEST\n");
+          printf ("Replacement Strategy = PGA_POPREPL_BEST\n");
           break;
       case PGA_POPREPL_RANDOM_REP:
-          printf("Replacement Strategy = PGA_POPREPL_RANDOM_REP\n");
+          printf ("Replacement Strategy = PGA_POPREPL_RANDOM_REP\n");
           break;
       case PGA_POPREPL_RANDOM_NOREP:
-          printf("Replacement Strategy = PGA_POPREPL_RANDOM_NOREP\n");
+          printf ("Replacement Strategy = PGA_POPREPL_RANDOM_NOREP\n");
           break;
       }
 
@@ -253,12 +259,6 @@ void PGASetRTRWindowSize( PGAContext *ctx, int windowsize)
 {
     PGADebugEntered("PGASetRTRWindowSize");
 
-    if (windowsize > ctx->ga.PopSize) {
-        PGAError ( ctx,
-                  "PGASetRTRWindowSize: Invalid value of windowsize:",
-                   PGA_FATAL, PGA_INT, (void *) &windowsize);
-    }
-
     ctx->ga.RTRWindowSize = windowsize;
 
     PGADebugExited("PGASetRTRWindowSize");
@@ -448,6 +448,7 @@ void PGASetPopReplaceType( PGAContext *ctx, int pop_replace)
     case PGA_POPREPL_RTR:
     case PGA_POPREPL_PAIRWISE_BEST:
     case PGA_POPREPL_NSGA_II:
+    case PGA_POPREPL_NSGA_III:
         ctx->ga.PopReplace = pop_replace;
         break;
     default:
@@ -460,6 +461,90 @@ void PGASetPopReplaceType( PGAContext *ctx, int pop_replace)
     PGADebugExited("PGASetPopReplaceType");
 }
 
+/*U****************************************************************************
+   PGASetReferencePoints - Set reference points on reference hyperplane
+   for NSGA-III
+
+   Category: Generation
+
+   Inputs:
+      ctx     - context variable
+      npoints - Number of points
+      points  - Pointer to points
+
+   Outputs:
+      None
+
+   Example:
+      PGAContext *ctx;
+      int dim = PGAGetNumAuxEval (ctx) - PGAGetNumConstraint (ctx) + 1;
+      void *p = NULL;
+      int np  = 0;
+      :
+      np = LIN_dasdennis (dim, 3, &p, 0, 1, NULL);
+      PGASetReferencePoints (ctx, np, p);
+
+
+****************************************************************************U*/
+void PGASetReferencePoints (PGAContext *ctx, size_t npoints, void *points)
+{
+    if (ctx->ga.nrefpoints) {
+        PGAErrorPrintf (ctx, PGA_FATAL, "Can't set reference points twice");
+    }
+    if (points == NULL) {
+        PGAErrorPrintf (ctx, PGA_FATAL, "Need non-NULL points");
+    }
+    ctx->ga.nrefpoints = npoints;
+    ctx->ga.refpoints  = points;
+}
+
+/*U****************************************************************************
+   PGASetReferenceDirections - Set reference directions for NSGA-III
+   A direction is a point in objective space and can be seen as a vector
+   from the origin to that point. During optimization the reference
+   directions are mapped to the reference hyperplane and a scaled
+   Das/Dennis hyperplane is constructed around that point.
+   Each direction consists of dimension double variables.
+
+   Category: Generation
+
+   Inputs:
+      ctx   - context variable
+      ndirs - Number of directions
+      dirs  - Pointer to directions
+      npart - Number of Das / Dennis partitions
+      scale - Scale factor for constructed Das / Dennis points
+              Must be 0 < scale <= 1 but will typically be < 0.5
+
+   Outputs:
+      None
+
+   Example:
+      Asume 3 dimensions, i.e. 3 evaluation functions
+      dim = PGAGetNumAuxEval (ctx) - PGAGetNumConstraint (ctx) + 1;
+
+      PGAContext *ctx;
+      double dirs [][3] = {{1, 2, 3}, {4, 5, 6}};
+      :
+      PGASetReferenceDirections (ctx, 2, dirs, 5, 0.1);
+
+
+****************************************************************************U*/
+void PGASetReferenceDirections
+    (PGAContext *ctx, size_t ndirs, void *dirs, int npart, double scale)
+{
+    if (ctx->ga.nrefdirs) {
+        PGAErrorPrintf (ctx, PGA_FATAL, "Can't set reference directions twice");
+    }
+    if (dirs == NULL) {
+        PGAErrorPrintf (ctx, PGA_FATAL, "Need non-NULL directions");
+    }
+    ctx->ga.nrefdirs   = ndirs;
+    ctx->ga.refdirs    = dirs;
+    ctx->ga.ndir_npart = npart;
+    ctx->ga.dirscale   = scale;
+}
+
 /*U****************************************************************************
    PGARestrictedTournamentReplacement - Perform restricted tournament
    replacement: for each individual in PGA_NEWPOP we select a window of
@@ -585,34 +670,6 @@ void PGAPairwiseBestReplacement (PGAContext *ctx)
     PGADebugExited("PGAPairwiseBestReplacement");
 }
 
-/*U****************************************************************************
-
-   PGA_NSGA_II_Replacement - Perform NSGA-II Replacement
-   First compute a dominance matrix of N x N bits. The rows are the
-   dominated-by relation. We loop over all n^1 pairs of individuals and
-   fill the matrix. Initit all ranks with -1.
-   Then starting with rank0:
-   - Get all rows of the matrix which are 0 and where the individual has
-     no rank yet: These are the currently non-dominated individuals,
-     assign the current rank
-   - Loop over all individuals with the current rank and remove their
-     bits from the dominance matrix
-   - Increment the rank counter
-
-   Category: Generation
-
-   Inputs:
-      ctx         - context variable
-
-   Outputs:
-      None
-
-   Example:
-      PGAContext *ctx;
-      :
-      PGA_NSGA_II_Replacement(ctx);
-
-****************************************************************************U*/
 /* Helper functions for PGA_NSGA_II_Replacement */
 
 #define NONNEGEVAL(v, is_ev) ((is_ev) ? (v) : (((v) < 0) ? 0 : (v)))
@@ -645,21 +702,32 @@ static int nondom_cmp (const void *a1, const void *a2)
     return CMP((*i2)->crowding, (*i1)->crowding);
 }
 
-/* Compute crowding distance over the given individuals */
-static void crowding (PGAContext *ctx, PGAIndividual **start, int n, int rank)
+/* typedef to make it easier to pass crowding functions as parameter */
+typedef void (* crowding_t)
+    (PGAContext *, PGAIndividual **, size_t, unsigned int);
+
+/* Compute crowding distance over the given individuals
+ * This is specific to NSGA-II.
+ * For explanation of is_ev see ranking
+ */
+STATIC void crowding
+    (PGAContext *ctx, PGAIndividual **start, size_t n, unsigned int rank)
 {
-    int i, k;
-    int nrank = 0;
+    size_t i;
+    int k;
+    size_t nrank = 0;
     int is_ev = INDGetAuxTotal (*start) ? 0 : 1;
-    PGAIndividual *crowd [n];
+    DECLARE_DYNARRAY (PGAIndividual *, crowd, n);
     int nc = ctx->ga.NumConstraint;
     int na = ctx->ga.NumAuxEval;
     int base = is_ev ? 0 : (na - nc);
     int nfun = is_ev ? (na - nc + 1) : nc;
-    double f_min [nfun], f_max [nfun];
+    DECLARE_DYNARRAY (double, f_min, nfun);
+    DECLARE_DYNARRAY (double, f_max, nfun);
 
     for (i=0; i<n; i++) {
         PGAIndividual *ind = start [i];
+        ind->crowding = 0;
         if (ind->rank == rank) {
             for (k=0; k<nfun; k++) {
                 double e = GETEVAL (ind, k + base, is_ev);
@@ -670,7 +738,6 @@ static void crowding (PGAContext *ctx, PGAIndividual **start, int n, int rank)
                     f_max [k] = e;
                 }
             }
-            ind->crowding = 0;
             crowd [nrank++] = ind;
         }
     }
@@ -694,36 +761,376 @@ static void crowding (PGAContext *ctx, PGAIndividual **start, int n, int rank)
     }
 }
 
+/* Compute utopian point as the minimum over all solutions */
+STATIC void compute_utopian (PGAContext *ctx, PGAIndividual **start, int n)
+{
+    int i, j;
+    int dim = ctx->ga.NumAuxEval - ctx->ga.NumConstraint + 1;
+
+    if (!ctx->ga.utopian_valid) {
+        for (j=0; j<dim; j++) {
+            ctx->ga.utopian [j] = GETEVAL (start [0], j, 1);
+        }
+        ctx->ga.utopian_valid = PGA_TRUE;
+    }
+    for (i=0; i<n; i++) {
+        for (j=0; j<dim; j++) {
+            double e = GETEVAL (start [i], j, 1);
+            if (OPT_DIR_CMP (ctx, e, ctx->ga.utopian [j]) > 0) {
+                ctx->ga.utopian [j] = e;
+            }
+        }
+    }
+}
+
+/* When computing ASF set values in vector < EPS_VAL to 0
+ * Taken from pymoo code, not documented in any paper.
+ */
+#define EPS_VAL 1e-3
+#define EPS_ASF 1e-6
+#define EPS_NAD 1e-6
+#define WEIGHT(a, j) ((a) == (j) ? 1 : EPS_ASF)
+
+/* Compute ASF (see nsga-iii paper), axis is <= dim - 1 */
+STATIC double compute_asf (PGAContext *ctx, double *point, int axis)
+{
+    int j;
+    int dim = ctx->ga.NumAuxEval - ctx->ga.NumConstraint + 1;
+    double *utop = ctx->ga.utopian;
+    double asf = NORMALIZE (ctx, point [0], utop [0]) / WEIGHT (axis, 0);
+
+    for (j=1; j<dim; j++) {
+        double a = NORMALIZE (ctx, point [j], utop [j]);
+        if (a < EPS_VAL) {
+            a = 0;
+        }
+        a /= WEIGHT (axis, j);
+        if (a > asf) {
+            asf = a;
+        }
+    }
+    return asf;
+}
+
+/* Compute extreme points */
+STATIC void compute_extreme (PGAContext *ctx, PGAIndividual **start, int n)
+{
+    int i, j;
+    int dim = ctx->ga.NumAuxEval - ctx->ga.NumConstraint + 1;
+    DECLARE_DYNPTR (double, extreme, dim) = ctx->ga.extreme;
+    DECLARE_DYNARRAY (double, asfmin, dim);
+
+    if (!ctx->ga.extreme_valid) {
+        for (i=0; i<dim; i++) {
+            for (j=0; j<dim; j++) {
+                DEREF2_DYNPTR (extreme, dim, i, j) = GETEVAL (start [0], j, 1);
+            }
+        }
+    }
+    ctx->ga.extreme_valid = PGA_TRUE;
+    for (j=0; j<dim; j++) {
+        asfmin [j] = compute_asf (ctx, DEREF1_DYNPTR (extreme, dim, j), j);
+    }
+
+    for (i=0; i<n; i++) {
+        DECLARE_DYNARRAY (double, e, dim);
+        for (j=0; j<dim; j++) {
+            e [j] = GETEVAL (start [i], j, 1);
+        }
+        for (j=0; j<dim; j++) {
+            double asf = compute_asf (ctx, e, j);
+            if (asf < asfmin [j]) {
+                asfmin [j] = asf;
+                memcpy
+                    (DEREF1_DYNPTR (extreme, dim, j), e, sizeof (double) * dim);
+            }
+        }
+    }
+}
+
+/* Preferred nadir estimate via extreme points and axes intersect */
+STATIC int compute_intersect (PGAContext *ctx, PGAIndividual **start, int n)
+{
+    int i, j, d;
+    int dim = ctx->ga.NumAuxEval - ctx->ga.NumConstraint + 1;
+    DECLARE_DYNPTR (double, extreme, dim) = ctx->ga.extreme;
+    DECLARE_DYNARRAY2 (double, m, dim, dim);
+    DECLARE_DYNARRAY (double, x, dim);
+    DECLARE_DYNARRAY (double, v, dim);
+
+    for (d=0; d<dim; d++) {
+        int result;
+        for (i=0; i<dim-1; i++) {
+            for (j=0; j<dim; j++) {
+                /* No need to normalize with utopian here, cancels */
+                DEREF2_DYNPTR (m, dim, j, i)
+                    = DEREF2_DYNPTR (extreme, dim, i+1, j)
+                    - DEREF2_DYNPTR (extreme, dim, 0, j);
+            }
+        }
+        for (j=0; j<dim; j++) {
+            DEREF2_DYNPTR (m, dim, j, dim-1) = j == d ? -1 : 0;
+            v [j] = -NORMALIZE
+                (ctx, DEREF2_DYNPTR (extreme, dim, 0, j), ctx->ga.utopian [j]);
+        }
+        result = LIN_solve (dim, m, v);
+        /* Matrix singular? */
+        if (result != 0) {
+            break;
+        }
+        x [d] = v [dim - 1];
+        /* Intercept too small or negative? */
+        if (x [d] <= EPS_NAD) {
+            break;
+        }
+    }
+    if (d >= dim) {
+        /* Success: Use nadir estimate from hyper-plane */
+        for (j=0; j<dim; j++) {
+            ctx->ga.nadir [j] = DENORMALIZE (ctx, x [j], ctx->ga.utopian [j]);
+        }
+        return 0;
+    }
+    /* Fail: No nadir estimate via extreme points, fall back to wof0 */
+    PGAErrorPrintf
+        ( ctx, PGA_WARNING
+        , "Intercept computation failed in Generation %d\n", ctx->ga.iter
+        );
+    return 1;
+}
+
+/* Compute worst of population and nadir estimate (worst of front 0) */
+STATIC void compute_worst
+    (PGAContext *ctx, PGAIndividual **start, int n, double *wpop, double *wof0)
+{
+    int i, j;
+    int dim = ctx->ga.NumAuxEval - ctx->ga.NumConstraint + 1;
+    int wv = 0;
+    for (i=0; i<n; i++) {
+        for (j=0; j<dim; j++) {
+            double e = GETEVAL (start [i], j, 1);
+            if (  !ctx->ga.worst_valid
+               || OPT_DIR_CMP (ctx, e, ctx->ga.worst [j]) < 0
+               )
+            {
+                ctx->ga.worst [j] = e;
+            }
+            if (i==0 || OPT_DIR_CMP (ctx, e, wpop [j]) < 0) {
+                wpop [j] = e;
+            }
+            if (  start [i]->rank == 0
+               && (wv <= j || OPT_DIR_CMP (ctx, e, wof0 [j]) < 0)
+               )
+            {
+                wof0 [j] = e;
+                if (wv <= j) {
+                    wv++;
+                }
+            }
+        }
+        ctx->ga.worst_valid = PGA_TRUE;
+    }
+    assert (wv == dim);
+}
+
+STATIC void compute_nadir (PGAContext *ctx, PGAIndividual **start, int n)
+{
+    int j;
+    int dim = ctx->ga.NumAuxEval - ctx->ga.NumConstraint + 1;
+    DECLARE_DYNARRAY (double, wof0, dim); /* Worst of front 0 */
+    DECLARE_DYNARRAY (double, wpop, dim); /* Worst of population */
+    int ret;
+
+    ret = compute_intersect (ctx, start, n);
+    compute_worst (ctx, start, n, wpop, wof0);
+    /* If a component in estimated nadir is *worse* than corresponding
+     * component of the worst point ever, replace this component.
+     * Taken from pymoo, not documented in any paper.
+     */
+    if (ret == 0) {
+        for (j=0; j<dim; j++) {
+            if (OPT_DIR_CMP (ctx, ctx->ga.nadir [j], ctx->ga.worst [j]) < 0) {
+                ctx->ga.nadir [j] = ctx->ga.worst [j];
+            }
+        }
+    } else {
+        /* Matrix inversion failed, need to fall back on wof0/wpop */
+        for (j=0; j<dim; j++) {
+            if (fabs (wof0 [j] - ctx->ga.utopian [j]) < EPS_NAD) {
+                ctx->ga.nadir [j] = wpop [j];
+            } else {
+                ctx->ga.nadir [j] = wof0 [j];
+            }
+        }
+    }
+}
+
+/* Get points in refpoints and refdir-cloud */
+static double *get_point (PGAContext *ctx, size_t idx)
+{
+    int dim = ctx->ga.NumAuxEval - ctx->ga.NumConstraint + 1;
+    DECLARE_DYNPTR (double, normdirs,  dim) = ctx->ga.normdirs;
+    DECLARE_DYNPTR (double, refpoints, dim) = ctx->ga.refpoints;
+    assert (idx < ctx->ga.ndpoints * ctx->ga.nrefdirs + ctx->ga.nrefpoints);
+    if (idx >= ctx->ga.nrefpoints) {
+        return DEREF1_DYNPTR (normdirs, dim, idx - ctx->ga.nrefpoints);
+    }
+    return DEREF1_DYNPTR (refpoints, dim, idx);
+}
+
+static int assoc_cmp (const void *a1, const void *a2)
+{
+    const PGAIndividual * const *i1 = a1;
+    const PGAIndividual * const *i2 = a2;
+    if ((*i1)->point_idx < (*i2)->point_idx) {
+        return -1;
+    }
+    if ((*i1)->point_idx > (*i2)->point_idx) {
+        return 1;
+    }
+    if ((*i1)->rank < (*i2)->rank) {
+        return -1;
+    }
+    if ((*i1)->rank > (*i2)->rank) {
+        return 1;
+    }
+    return CMP((*i1)->distance, (*i2)->distance);
+}
+
+/* Compute niche preservation algorithm over the given individuals
+ * This is specific to NSGA-III.
+ */
+static void niching
+    (PGAContext *ctx, PGAIndividual **start, size_t n, unsigned int rank)
+{
+    size_t i, j;
+    size_t dim = ctx->ga.NumAuxEval - ctx->ga.NumConstraint + 1;
+    size_t npoints = ctx->ga.ndpoints * ctx->ga.nrefdirs + ctx->ga.nrefpoints;
+    DECLARE_DYNARRAY (double, point, dim);
+
+    compute_utopian (ctx, start, n);
+    compute_extreme (ctx, start, n);
+    compute_nadir   (ctx, start, n);
+    /* Normalize points to hyperplane */
+    for (i=0; i<n; i++) {
+        PGAIndividual *ind = start [i];
+        /* Init crowding metric for all individuals */
+        ind->crowding = 0;
+        for (j=0; j<dim; j++) {
+            double e = GETEVAL (start [i], j, 1);
+            e  = NORMALIZE (ctx, e, ctx->ga.utopian [j]);
+            e /= NORMALIZE (ctx, ctx->ga.nadir [j], ctx->ga.utopian [j]);
+            ind->normalized [j] = e;
+        }
+        LIN_normalize_to_refplane (dim, ind->normalized);
+    }
+    /* map reference directions to hyperplane
+     * and compute the resulting points
+     */
+    for (i=0; i<ctx->ga.nrefdirs; i++) {
+        size_t sz = ctx->ga.ndpoints * sizeof (double) * dim;
+        DECLARE_DYNPTR (double, refdirs, dim) = ctx->ga.refdirs;
+        memcpy (point, DEREF1_DYNPTR (refdirs, dim, i), sizeof (double) * dim);
+        for (j=0; j<dim; j++) {
+            point [j]  = NORMALIZE (ctx, point [j], ctx->ga.utopian [j]);
+            point [j] /= NORMALIZE
+                (ctx, ctx->ga.nadir [j], ctx->ga.utopian [j]);
+        }
+        LIN_dasdennis_allocated
+            ( dim, ctx->ga.ndir_npart
+            , ctx->ga.dirscale, point
+            , ctx->ga.ndpoints, ((char *)ctx->ga.normdirs) + i * sz
+            );
+    }
+    for (j=0; j<n; j++) {
+        PGAIndividual *ind = start [j];
+        double mindist = LIN_euclidian_distance
+            (dim, ind->normalized, get_point (ctx, 0));
+        int minidx = 0;
+        for (i=1; i<npoints; i++) {
+            double *point = get_point (ctx, i);
+            double d = LIN_euclidian_distance
+                (dim, ind->normalized, point);
+            if (d < mindist) {
+                mindist = d;
+                minidx  = i;
+            }
+        }
+        ind->distance = mindist;
+        ind->point_idx = minidx;
+    }
+    /* Sort individuals by associated point index, rank, distance */
+    qsort (start, n, sizeof (PGAIndividual *), assoc_cmp);
+    /* Iterate over individuals and set crowding metric to
+     * the negative of the count of the current point
+     * We don't care that points with lower rank have now a crowding
+     * metric, too.
+     */
+    {
+        int last_pointidx = -1;
+        int pointcount = 0;
+        for (j=0; j<n; j++) {
+            PGAIndividual *ind = start [j];
+            if (last_pointidx != ind->point_idx) {
+                last_pointidx = ind->point_idx;
+                pointcount = 0;
+            } else {
+                pointcount++;
+            }
+            ind->crowding = -pointcount;
+        }
+    }
+}
+
 /* Dominance computation, return the maximum rank given or UINT_MAX if
  * goal was reached exactly (in which case no crowding is necessary)
+ * First compute a dominance matrix of N x N bits. The rows are the
+ * dominated-by relation. We loop over all n^2 pairs of individuals and
+ * fill the matrix. Initit all ranks with -1.
+ * Then starting with rank0:
+ * - Get all rows of the matrix which are 0 and where the individual has
+ *   no rank yet: These are the currently non-dominated individuals,
+ *   assign the current rank
+ * - Loop over all individuals with the current rank and remove their
+ *   bits from the dominance matrix
+ * - Increment the rank counter
+ * The is_ev flag decides if we're ranking the evaluation functions or
+ * if we're ranking constraint violations, it is 0 for eval functions.
+ * The base specifies which is the first aux evaluation. This is 0 if
+ * ranking eval function (not constraint violations) and is the index of
+ * the first constraint otherwise. The nfun variable is the number of
+ * functions to test in each case.
  */
-static int ranking (PGAContext *ctx, PGAIndividual **start, int n, int goal)
+STATIC unsigned int ranking
+    (PGAContext *ctx, PGAIndividual **start, size_t n, int goal)
 {
-    int i, j, k;
-    int is_ev = INDGetAuxTotal (*start) ? 0 : 1;
-    int rank;
+    size_t i, j;
+    int k;
+    int is_ev = INDGetAuxTotal (*start) - ctx->ga.Epsilon <= 0;
+    unsigned int rank;
     int nranked = 0;
     int nc = ctx->ga.NumConstraint;
     int na = ctx->ga.NumAuxEval;
     int base = is_ev ? 0 : (na - nc);
     int nfun = is_ev ? (na - nc + 1) : nc;
-    int nintbits = sizeof (int) * 8;
-    int intsforn = (n + nintbits - 1) / nintbits;
-    int (*dominance)[n][intsforn] =
-        (int (*)[n][intsforn])(ctx->scratch.dominance);
+    size_t intsforn = (n + WL - 1) / WL;
+    DECLARE_DYNPTR (PGABinary, dominance, intsforn) =
+        (void *)(ctx->scratch.dominance);
 
+    /* Initialize rank, dominance, crowding
+     * We initialize crowding here, too because later we compute
+     * crowding metric only for the last dominance rank but we sort
+     * *all* individuals by crowding.
+     */
     for (i=0; i<n; i++) {
-        (start [i])->rank = UINT_MAX;
+        (start [i])->rank     = UINT_MAX;
+        (start [i])->crowding = 0;
         for (j=0; j<intsforn; j++) {
-            (*dominance) [i][j] = 0;
+            DEREF2_DYNPTR (dominance, intsforn, i, j) = 0;
         }
     }
     for (i=0; i<n; i++) {
-        int iidx   = i / nintbits;
-        int ishift = 1 << (i % nintbits);
         for (j=i+1; j<n; j++) {
-            int jidx   = j / nintbits;
-            int jshift = 1 << (j % nintbits);
             int cmp = 0;
             for (k=0; k<nfun; k++) {
                 double e1, e2;
@@ -738,7 +1145,10 @@ static int ranking (PGAContext *ctx, PGAIndividual **start, int n, int goal)
                 if (cmp && ncmp && ncmp != cmp) {
                     break;
                 }
-                cmp = ncmp;
+                /* Don't allow cmp to get zero again */
+                if (ncmp) {
+                    cmp = ncmp;
+                }
             }
             /* Non-dominated? */
             if (!cmp || k<nfun) {
@@ -746,10 +1156,10 @@ static int ranking (PGAContext *ctx, PGAIndividual **start, int n, int goal)
             }
             /* j dominated by i */
             if (cmp < 0) {
-                (*dominance) [j][iidx] |= ishift;
+                SET_BIT (DEREF1_DYNPTR (dominance, intsforn, j), i);
             /* i dominated by j */
             } else {
-                (*dominance) [i][jidx] |= jshift;
+                SET_BIT (DEREF1_DYNPTR (dominance, intsforn, i), j);
             }
         }
     }
@@ -761,12 +1171,12 @@ static int ranking (PGAContext *ctx, PGAIndividual **start, int n, int goal)
                 continue;
             }
             for (j=0; j<intsforn; j++) {
-                if ((*dominance) [i][j]) {
+                if (DEREF2_DYNPTR (dominance, intsforn, i, j)) {
                     break;
                 }
             }
             /* Non-dominated in this rank */
-            if (j==intsforn) {
+            if (j == intsforn) {
                 (*(start+i))->rank = rank;
                 nranked ++;
             }
@@ -777,13 +1187,11 @@ static int ranking (PGAContext *ctx, PGAIndividual **start, int n, int goal)
         }
         /* Remove dominance bits for this rank */
         for (i=0; i<n; i++) {
-            int iidx   = i / nintbits;
-            int ishift = 1 << (i % nintbits);
             if ((*(start+i))->rank != rank) {
                 continue;
             }
             for (j=0; j<n; j++) {
-                (*dominance) [j][iidx] &= ~ishift;
+                CLEAR_BIT (DEREF1_DYNPTR (dominance, intsforn, j), i);
             }
         }
     }
@@ -794,31 +1202,76 @@ static int ranking (PGAContext *ctx, PGAIndividual **start, int n, int goal)
     return rank;
 }
 
-void PGA_NSGA_II_Replacement (PGAContext *ctx)
+/*
+ * PGA_NSGA_Replacement - Perform NSGA Replacement
+ * - Perform dominance computation (ranking)
+ * - Perform crowding computation specific to the NSGA-Variant given as
+ *   the parameter crowding_method
+ * - Sort individuals and replace into next generation
+ */
+static void PGA_NSGA_Replacement (PGAContext *ctx, crowding_t crowding_method)
 {
     int i;
     int n_unc_ind, n_con_ind;
     int popsize = PGAGetPopSize (ctx);
     int numreplace = PGAGetNumReplaceValue (ctx);
-    PGAIndividual *all_individuals [popsize + numreplace];
+    DECLARE_DYNARRAY (PGAIndividual *, all_individuals, popsize + numreplace);
     PGAIndividual **constrained = all_individuals + popsize + numreplace;
     PGAIndividual **unconstrained = all_individuals;
     PGAIndividual *oldpop = ctx->ga.oldpop;
     PGAIndividual *newpop = ctx->ga.newpop;
     PGAIndividual *temp;
+    size_t         n_dupes = 0;
 
-    PGADebugEntered("PGA_NSGA_II_Replacement");
+    PGADebugEntered("PGA_NSGA_Replacement");
 
     /* We keep two pointers into the all_individuals array. One with
      * constrained individuals starts from the end. The other with
      * unconstrained individuals starts from the start.
+     * Note that with the NSGA algorithms it does not make much sense to
+     * have numreplace < popsize: The algorithm is elitist and will keep
+     * the good individuals anyway. If numreplace < popsize we do not
+     * use the individuals that were copied to the new population,
+     * otherwise we would produce duplicates.
      */
 
+    /* Before we put constrained and unconstrained individuals into the
+     * start array, if the NoDuplicates flag is set, we sort all the
+     * duplicates to the end of the array.
+     * A precondition is that neither the individuals in oldpop nor the
+     * individuals in newpop contain duplicates. And all the individuals
+     * in newpop are already in the hash. So we just loop over oldpop
+     * and put all duplicates at the end of the array.
+     * if numreplace < popsize we rely on the fact that the first
+     * popsize-numreplace individuals are the same in oldpop and newpop.
+     * The individuals from newpop are not used in that case, so we do
+     * *not* mark the first popsize-numreplace individuals as
+     * duplicates.
+     */
+    if (ctx->ga.NoDuplicates) {
+        for (i=popsize - numreplace; i<popsize; i++) {
+            if (PGADuplicate (ctx, i, PGA_OLDPOP, PGA_NEWPOP)) {
+                constrained--;
+                *constrained = oldpop + i;
+                n_dupes++;
+            }
+        }
+    }
+
     /* First loop over all old individuals and put them into the
-     * all_individuals array.
+     * all_individuals array. Note that we compare using the current
+     * Epsilon, so this uses Epsilon-Constrained optimization if
+     * enabled.
+     * Dupes are already handled above, skip those.
      */
     for (i=0; i<popsize; i++) {
-        if (INDGetAuxTotal (oldpop + i) > 0) {
+        /* Dupes are handled above */
+        if (ctx->ga.NoDuplicates && i >= popsize - numreplace) {
+            if (PGADuplicate (ctx, i, PGA_OLDPOP, PGA_NEWPOP)) {
+                continue;
+            }
+        }
+        if (INDGetAuxTotal (oldpop + i) - ctx->ga.Epsilon > 0) {
             constrained--;
             *constrained = oldpop + i;
         } else {
@@ -828,7 +1281,7 @@ void PGA_NSGA_II_Replacement (PGAContext *ctx)
     }
     /* Now put all the new individuals into the same array */
     for (i=popsize - numreplace; i<popsize; i++) {
-        if (INDGetAuxTotal (newpop + i) > 0) {
+        if (INDGetAuxTotal (newpop + i) - ctx->ga.Epsilon > 0) {
             constrained--;
             *constrained = newpop + i;
         } else {
@@ -841,10 +1294,11 @@ void PGA_NSGA_II_Replacement (PGAContext *ctx)
      */
     assert (constrained == unconstrained);
     n_unc_ind = unconstrained - all_individuals;
-    n_con_ind = popsize + numreplace - n_unc_ind;
+    n_con_ind = popsize + numreplace - n_unc_ind - n_dupes;
 
     /* First perform non-dominated sorting on unconstrained individuals
-     * Normal sorting if only one eval function */
+     * Normal sorting if only one eval function
+     */
     if (ctx->ga.NumConstraint == ctx->ga.NumAuxEval) {
         qsort
             ( all_individuals
@@ -853,10 +1307,10 @@ void PGA_NSGA_II_Replacement (PGAContext *ctx)
             , PGAEvalSortHelper
             );
     } else {
-        int rank;
+        unsigned int rank;
         rank = ranking (ctx, all_individuals, n_unc_ind, popsize);
         if (n_unc_ind >= popsize && rank != UINT_MAX) {
-            crowding (ctx, all_individuals, n_unc_ind, rank);
+            crowding_method (ctx, all_individuals, n_unc_ind, rank);
         }
         qsort \
             ( all_individuals
@@ -883,7 +1337,7 @@ void PGA_NSGA_II_Replacement (PGAContext *ctx)
                 , PGAEvalSortHelper
                 );
         } else {
-            int rank;
+            unsigned int rank;
             rank = ranking
                 ( ctx
                 , all_individuals + n_unc_ind
@@ -932,6 +1386,57 @@ void PGA_NSGA_II_Replacement (PGAContext *ctx)
     temp           = ctx->ga.oldpop;
     ctx->ga.oldpop = ctx->ga.newpop;
     ctx->ga.newpop = temp;
-    PGADebugExited("PGA_NSGA_II_Replacement");
+    PGADebugExited ("PGA_NSGA_Replacement");
 }
 
+/*U****************************************************************************
+
+   PGA_NSGA_II_Replacement - Perform NSGA-II Replacement
+   - Perform dominance computation (ranking)
+   - Perform crowding computation specific to NSGA-II
+   - Sort individuals and replace into next generation
+
+   Category: Generation
+
+   Inputs:
+      ctx         - context variable
+
+   Outputs:
+      None
+
+   Example:
+      PGAContext *ctx;
+      :
+      PGA_NSGA_II_Replacement(ctx);
+****************************************************************************U*/
+
+void PGA_NSGA_II_Replacement (PGAContext *ctx)
+{
+    PGA_NSGA_Replacement (ctx, crowding);
+}
+
+/*U****************************************************************************
+
+   PGA_NSGA_III_Replacement - Perform NSGA-III Replacement
+   - Perform dominance computation (ranking)
+   - Perform crowding computation specific to NSGA-III
+   - Sort individuals and replace into next generation
+
+   Category: Generation
+
+   Inputs:
+      ctx         - context variable
+
+   Outputs:
+      None
+
+   Example:
+      PGAContext *ctx;
+      :
+      PGA_NSGA_II_Replacement(ctx);
+****************************************************************************U*/
+
+void PGA_NSGA_III_Replacement (PGAContext *ctx)
+{
+    PGA_NSGA_Replacement (ctx, niching);
+}
