@@ -46,6 +46,31 @@ privately owned rights.
 *****************************************************************************/
 
 #include <pgapack.h>
+/* Helper for bounds/bounce check */
+static void bouncheck
+    ( PGAContext *ctx, int idx, int boundflag, int bounceflag
+    , PGAReal *child, PGAReal minp, PGAReal maxp
+    )
+{
+    if (boundflag || bounceflag) {
+        if (child [idx] < ctx->init.RealMin [idx]) {
+            if (bounceflag) {
+                child [idx] = PGARandomUniform
+                    (ctx, ctx->init.RealMin [idx], minp);
+            } else {
+                child [idx] = ctx->init.RealMin [idx];
+            }
+        }
+        if (child [idx] > ctx->init.RealMax [idx]) {
+            if (bounceflag) {
+                child [idx] = PGARandomUniform
+                    (ctx, maxp, ctx->init.RealMax [idx]);
+            } else {
+                child [idx] = ctx->init.RealMax [idx];
+            }
+        }
+    }
+}
 
 /*U****************************************************************************
    PGASetRealAllele - sets the value of real-valued allele i in string p
@@ -337,10 +362,10 @@ double PGAGetMaxRealInitValue (PGAContext *ctx, int i)
       inittype = PGAGetRealInitType(ctx);
       switch (inittype) {
       case PGA_RINIT_PERCENT:
-          printf("Data Type = PGA_RINIT_PERCENT\n");
+          printf ("Data Type = PGA_RINIT_PERCENT\n");
           break;
       case PGA_RINIT_RANGE:
-          printf("Data Type = PGA_RINIT_RANGE\n");
+          printf ("Data Type = PGA_RINIT_RANGE\n");
           break;
       }
 
@@ -450,7 +475,7 @@ int PGARealMutation (PGAContext *ctx, int p, int pop, double mr)
     int do_best = (ctx->ga.DEVariant == PGA_DE_VARIANT_BEST);
     int nrand  = 2 * ctx->ga.DENumDiffs + (!do_best);
     int maxidx = 2 * ctx->ga.DENumDiffs + 1;
-    PGAReal *indivs [maxidx];
+    DECLARE_DYNARRAY (PGAReal *, indivs, maxidx);
     int do_crossover = 1;
     static double de_dither = 0.0;
     static int last_iter = -1;
@@ -458,10 +483,13 @@ int PGARealMutation (PGAContext *ctx, int p, int pop, double mr)
 
     PGADebugEntered("PGARealMutation");
     if (ctx->ga.MutationType == PGA_MUTATION_DE) {
-        int idx [maxidx];
+        DECLARE_DYNARRAY (int, idx, maxidx);
         PGASampleState sstate;
-        int best = PGAGetBestIndex (ctx, PGA_OLDPOP);
+        int best = 0;
         int avoid = 1;
+        if (do_best) {
+            best = PGAGetBestIndex (ctx, PGA_OLDPOP);
+        }
 
         if (ctx->ga.DEDither > 0) {
             if (ctx->ga.DEDitherPerIndividual || last_iter != ctx->ga.iter) {
@@ -530,13 +558,7 @@ int PGARealMutation (PGAContext *ctx, int p, int pop, double mr)
         /* Since indices from PGARandomNextSample are
          * returned in order we need to shuffle
          */
-        for (i=0; i<nrand-1; i++) {
-            int tmp;
-            j = PGARandomInterval (ctx, i, nrand-1);
-            tmp = idx [j];
-            idx [j] = idx [i];
-            idx [i] = tmp;
-        }
+        PGAShuffle (ctx, idx, nrand);
         /* Now we have a list of shuffled indexes that do not
          * collide with one-another or with p or best
          * Add best index as last to the list if do_best
@@ -562,25 +584,48 @@ int PGARealMutation (PGAContext *ctx, int p, int pop, double mr)
             case PGA_MUTATION_CONSTANT:
             case PGA_MUTATION_UNIFORM:
             case PGA_MUTATION_GAUSSIAN:
+            case PGA_MUTATION_POLY:
                 /* randomly choose an allele   */
                 if ( PGARandomFlip(ctx, mr) ) {
                     /* generate on range, or calculate multiplier */
                     switch (ctx->ga.MutationType) {
-                    case PGA_MUTATION_RANGE:
-                        c [i] = PGARandomUniform(ctx, ctx->init.RealMin [i],
-                                                      ctx->init.RealMax [i]);
+                      case PGA_MUTATION_RANGE:
+                        c [i] = PGARandomUniform
+                            (ctx, ctx->init.RealMin [i], ctx->init.RealMax [i]);
                         break;
-                    case PGA_MUTATION_CONSTANT:
+                      case PGA_MUTATION_CONSTANT:
                         val = ctx->ga.MutateRealValue;
                         break;
-                    case PGA_MUTATION_UNIFORM:
+                      case PGA_MUTATION_UNIFORM:
                         val = PGARandomUniform
                             (ctx, 0.0, ctx->ga.MutateRealValue);
                         break;
-                    case PGA_MUTATION_GAUSSIAN:
+                      case PGA_MUTATION_GAUSSIAN:
                         val = PGARandomGaussian
                             (ctx, 0.0, ctx->ga.MutateRealValue);
                         break;
+                      case PGA_MUTATION_POLY:
+                      {
+                        double u = PGARandom01 (ctx, 0);
+                        double eta = PGAGetMutationPolyEta (ctx) + 1;
+                        double delta;
+                        if (u < 0.5) {
+                            delta = pow (2 * u, 1.0 / eta) - 1.0;
+                        } else {
+                            delta = 1.0 - pow (2 * (1 - u), 1.0 / eta);
+                        }
+                        if (ctx->ga.MutatePolyValue >= 0) {
+                            c [i] += delta * ctx->ga.MutatePolyValue;
+                        } else {
+                            if (delta < 0) {
+                                val = fabs (c [i] - ctx->init.RealMin [i]);
+                            } else {
+                                val = fabs (ctx->init.RealMax [i] - c [i]);
+                            }
+                            c [i] += delta * val;
+                        }
+                        break;
+                      }
                     }
                     /* apply multiplier calculated in switch above */
                     if ( (ctx->ga.MutationType == PGA_MUTATION_CONSTANT) ||
@@ -674,24 +719,10 @@ int PGARealMutation (PGAContext *ctx, int p, int pop, double mr)
         }
 
         /* reset to min/max or bounce if outside range */
-        if (ctx->ga.MutateBoundedFlag || ctx->ga.MutateBounceFlag) {
-            if( c [idx] < ctx->init.RealMin [idx]) {
-                if (ctx->ga.MutateBounceFlag) {
-                    c [idx] = PGARandomUniform
-                        (ctx, ctx->init.RealMin [idx], old_value);
-                } else {
-                    c [idx] = ctx->init.RealMin [idx];
-                }
-            }
-            if( c [idx] > ctx->init.RealMax [idx]) {
-                if (ctx->ga.MutateBounceFlag) {
-                    c [idx] = PGARandomUniform
-                        (ctx, old_value, ctx->init.RealMax [idx]);
-                } else {
-                    c [idx] = ctx->init.RealMax [idx];
-                }
-            }
-        }
+        bouncheck
+            ( ctx, idx, ctx->ga.MutateBoundedFlag, ctx->ga.MutateBounceFlag
+            , c, old_value, old_value
+            );
     }
 
     PGADebugExited("PGARealMutation");
@@ -889,6 +920,74 @@ void PGARealUniformCrossover( PGAContext *ctx, int p1, int p2, int pop1,
     PGADebugExited("PGARealUniformCrossover");
 }
 
+/*I****************************************************************************
+   PGARealSBXCrossover - performs simulated binary crossover (SBX)
+   on two parent strings producing two children via side-effect
+
+   Inputs:
+      ctx  - context variable
+      p1   - the first parent string
+      p2   - the second parent string
+      pop1 - symbolic constant of the population containing string p1 and p2
+      c1   - the first child string
+      c2   - the second child string
+      pop2 - symbolic constant of the population to contain string c1 and c2
+
+   Outputs:
+      c1 and c2 in population pop2 are modified by side-effect.
+
+   Example:
+      Performs crossover on the two parent strings m and d, producing
+      children s and b.
+
+      PGAContext *ctx;
+      int m, d, s, b;
+      :
+      PGARealSBXCrossover (ctx, m, d, PGA_OLDPOP, s, b, PGA_NEWPOP);
+
+****************************************************************************I*/
+void PGARealSBXCrossover
+    (PGAContext *ctx, int p1, int p2, int pop1, int c1, int c2, int pop2)
+{
+    PGAReal *parent1 = (PGAReal *)PGAGetIndividual (ctx, p1, pop1)->chrom;
+    PGAReal *parent2 = (PGAReal *)PGAGetIndividual (ctx, p2, pop1)->chrom;
+    PGAReal *child1  = (PGAReal *)PGAGetIndividual (ctx, c1, pop2)->chrom;
+    PGAReal *child2  = (PGAReal *)PGAGetIndividual (ctx, c2, pop2)->chrom;
+    int i;
+    double u = 0;
+
+    if (ctx->ga.CrossSBXOnce) {
+        u = PGARandom01 (ctx, 0);
+    }
+
+    for (i=0; i<ctx->ga.StringLen; i++) {
+        if (  parent1 [i] == parent2 [i]
+           || !PGARandomFlip (ctx, ctx->ga.UniformCrossProb)
+           )
+        {
+            child1 [i] = parent1 [i];
+            child2 [i] = parent2 [i];
+        } else {
+            int j;
+            PGAReal minp =
+                parent1 [i] < parent2 [i] ? parent1 [i] : parent2 [i];
+            PGAReal maxp =
+                parent1 [i] > parent2 [i] ? parent1 [i] : parent2 [i];
+            if (!ctx->ga.CrossSBXOnce) {
+                u = PGARandom01 (ctx, 0);
+            }
+            PGACrossoverSBX
+                (ctx, parent1 [i], parent2 [i], u, child1 + i, child2 + i);
+            for (j=0; j<2; j++) {
+                bouncheck
+                    ( ctx, i, ctx->ga.CrossBoundedFlag, ctx->ga.CrossBounceFlag
+                    , j ? child2 : child1, minp, maxp
+                    );
+            }
+        }
+    }
+}
+
 /*I****************************************************************************
    PGARealPrintString - writes a real-valued string to a file.  This routine
    casts the void string pointer it is passed as the second argument.
@@ -907,7 +1006,7 @@ void PGARealUniformCrossover( PGAContext *ctx, int p1, int p2, int pop1,
       PGAContext *ctx;
       int s;
       :
-      PGARealPrintString( ctx, stdout, s, PGA_NEWPOP );
+      PGARealPrintString (ctx, stdout, s, PGA_NEWPOP);
 
 ****************************************************************************I*/
 void PGARealPrintString (PGAContext *ctx, FILE *fp, int p, int pop)
@@ -1000,8 +1099,9 @@ void PGARealCopyString ( PGAContext *ctx, int p1, int pop1, int p2, int pop2)
       PGAContext *ctx;
       int x, y;
       :
-      if ( PGARealDuplicate( ctx, x, PGA_OLDPOP, y, PGA_OLDPOP ) )
-          printf("strings are duplicates\n");
+      if (PGARealDuplicate (ctx, x, PGA_OLDPOP, y, PGA_OLDPOP)) {
+          printf ("strings are duplicates\n");
+      }
 
 ****************************************************************************I*/
 int PGARealDuplicate( PGAContext *ctx, int p1, int pop1, int p2, int pop2)
@@ -1023,6 +1123,26 @@ int PGARealDuplicate( PGAContext *ctx, int p1, int pop1, int p2, int pop2)
     return count == ctx->ga.StringLen ? PGA_TRUE : PGA_FALSE;
 }
 
+/*I****************************************************************************
+   PGARealHash - Returns hash value of given gene
+
+   Inputs:
+      ctx - context variable
+      p    - string index of the string to hash
+      pop  - symbolic constant of the population string p is in
+
+   Outputs:
+      Hash value for string
+
+****************************************************************************I*/
+PGAHash PGARealHash (PGAContext *ctx, int p, int pop)
+{
+    void *a = PGAGetIndividual (ctx, p, pop)->chrom;
+    PGAHash hash = PGAUtilHash
+        (a, sizeof (PGAReal) * ctx->ga.StringLen, PGA_INITIAL_HASH);
+    return hash;
+}
+
 /*I****************************************************************************
    PGARealInitString - randomly initialize a string of type PGAReal
 
@@ -1074,63 +1194,40 @@ void PGARealInitString ( PGAContext *ctx, int p, int pop)
      dt = PGARealBuildDatatype(ctx, p, pop);
 
 ****************************************************************************I*/
-MPI_Datatype PGARealBuildDatatype(PGAContext *ctx, int p, int pop)
+MPI_Datatype PGARealBuildDatatype (PGAContext *ctx, int p, int pop)
 {
-    int            n = 6;
-    int            counts[7];      /* Number of elements in each
-                                      block (array of integer) */
-    MPI_Aint       displs[7];      /* byte displacement of each
-                                      block (array of integer) */
-    MPI_Datatype   types[7];       /* type of elements in each block (array
-                                      of handles to datatype objects) */
+    int            idx = 0;
+    /* Number of elements in each block (array of integer) */
+    int            counts [PGA_MPI_HEADER_ELEMENTS + 1];
+    /* byte displacement of each block (array of integer) */
+    MPI_Aint       displs [PGA_MPI_HEADER_ELEMENTS + 1];
+    /* type of elements in each block (array of handles to datatype objects) */
+    MPI_Datatype   types  [PGA_MPI_HEADER_ELEMENTS + 1];
     MPI_Datatype   individualtype; /* new datatype (handle) */
     PGAIndividual *traveller;      /* address of individual in question */
 
-    PGADebugEntered("PGARealBuildDatatype");
+    PGADebugEntered  ("PGARealBuildDatatype");
 
-    traveller = PGAGetIndividual(ctx, p, pop);
-    MPI_Get_address(&traveller->evalue, &displs[0]);
-    counts[0] = 1;
-    types[0]  = MPI_DOUBLE;
+    traveller = PGAGetIndividual  (ctx, p, pop);
+    idx = PGABuildDatatypeHeader (ctx, p, pop, counts, displs, types);
 
-    MPI_Get_address(&traveller->fitness, &displs[1]);
-    counts[1] = 1;
-    types[1]  = MPI_DOUBLE;
+    MPI_Get_address (traveller->chrom, &displs [idx]);
+    counts [idx] = ctx->ga.StringLen;
+    types  [idx] = MPI_DOUBLE;
+    idx++;
 
-    MPI_Get_address(&traveller->evaluptodate, &displs[2]);
-    counts[2] = 1;
-    types[2]  = MPI_INT;
+    MPI_Type_create_struct (idx, counts, displs, types, &individualtype);
+    MPI_Type_commit (&individualtype);
 
-    MPI_Get_address(traveller->chrom, &displs[3]);
-    counts[3] = ctx->ga.StringLen;
-    types[3]  = MPI_DOUBLE;
+    PGADebugExited ("PGARealBuildDatatype");
 
-    MPI_Get_address(&traveller->auxtotal, &displs[4]);
-    counts[4] = 1;
-    types[4]  = MPI_DOUBLE;
-
-    MPI_Get_address(&traveller->auxtotalok, &displs[5]);
-    counts[5] = 1;
-    types[5]  = MPI_INT;
-
-    if (ctx->ga.NumAuxEval) {
-        MPI_Get_address(traveller->auxeval, &displs[6]);
-        counts[6] = ctx->ga.NumAuxEval;
-        types[6]  = MPI_DOUBLE;
-        n += 1;
-    }
-
-    MPI_Type_create_struct(n, counts, displs, types, &individualtype);
-    MPI_Type_commit(&individualtype);
-
-    PGADebugExited("PGARealBuildDatatype");
-
-    return (individualtype);
+    return individualtype;
 }
 
 /*I****************************************************************************
    PGARealGeneDistance - Compute genetic difference of two strings.
    Sum of the absolute values of the differences of each allele.
+   So this is a Manhattan distance (mainly for performance reasons).
 
    Inputs:
       ctx   - context variable
@@ -1155,7 +1252,7 @@ double PGARealGeneDistance (PGAContext *ctx, int p1, int pop1, int p2, int pop2)
 
     PGADebugEntered("PGARealGeneDistance");
     for (i=0; i<ctx->ga.StringLen; i++) {
-        ret += abs (c1 [i] - c2 [i]);
+        ret += fabs (c1 [i] - c2 [i]);
     }
     PGADebugExited("PGARealGeneDistance");
     return ret;
@@ -1184,13 +1281,8 @@ double PGARealEuclidianDistance (PGAContext *ctx, int p1, int pop1, int p2, int
 {
     PGAReal *c1 = (PGAReal *)PGAGetIndividual (ctx, p1, pop1)->chrom;
     PGAReal *c2 = (PGAReal *)PGAGetIndividual (ctx, p2, pop2)->chrom;
-    double ret = 0.0;
-    int i;
 
     PGADebugEntered("PGARealGeneDistance");
-    for (i=0; i<ctx->ga.StringLen; i++) {
-        ret += pow (c1 [i] - c2 [i], 2);
-    }
     PGADebugExited("PGARealGeneDistance");
-    return sqrt (ret);
+    return LIN_euclidian_distance (ctx->ga.StringLen, c1, c2);
 }
