@@ -65,7 +65,7 @@ privately owned rights.
       PGAContext *ctx;
       int p;
       :
-      PGAPrintReport(ctx, stdout, PGA_NEWPOP);
+      PGAPrintReport (ctx, stdout, PGA_NEWPOP);
 
 ****************************************************************************U*/
 void PGAPrintReport(PGAContext *ctx, FILE *fp, int pop)
@@ -88,7 +88,7 @@ void PGAPrintReport(PGAContext *ctx, FILE *fp, int pop)
     /* No multi objective eval */
     if (numaux == numcon) {
         best_p = PGAGetBestIndex (ctx, pop);
-        best_e = PGAGetEvaluation (ctx, best_p, pop);
+        best_e = _PGAGetEvaluation (ctx, best_p, pop, NULL);
 
         fprintf (fp, "Iter #     Field      Value");
         if (numcon) {
@@ -103,13 +103,20 @@ void PGAPrintReport(PGAContext *ctx, FILE *fp, int pop)
         fprintf (fp, "\n");
         if (ctx->rep.PrintOptions & PGA_REPORT_WORST) {
             p = PGAGetWorstIndex (ctx, pop);
-            e = PGAGetEvaluation (ctx, p, pop);
+            e = _PGAGetEvaluation (ctx, p, pop, NULL);
             fprintf (fp, "           Worst     %13.6e", e);
             if (numaux) {
                 fprintf (fp, "   %13.6e", PGAGetAuxTotal (ctx, p, pop));
             }
             fprintf (fp, "\n");
         }
+        if (numcon && ctx->ga.EpsilonGeneration) {
+            fprintf
+                ( fp
+                , "           Epsilon                   %13.6e\n"
+                , ctx->ga.Epsilon
+                );
+        }
         if (ctx->rep.PrintOptions & PGA_REPORT_AVERAGE) {
             fprintf
                 (fp, "           Average   %13.6e\n", ctx->rep.Average [0]);
@@ -130,6 +137,18 @@ void PGAPrintReport(PGAContext *ctx, FILE *fp, int pop)
                 , ctx->ga.iter, k, ctx->rep.Best [k]
                 );
         }
+        if (numcon) {
+            fprintf
+                ( fp, "%-11dMinConstr    %13.6e\n"
+                , ctx->ga.iter, ctx->rep.MinSumConstr
+                );
+        }
+        if (numcon && ctx->ga.EpsilonGeneration) {
+            fprintf
+                ( fp, "%-11dEpsilon      %13.6e\n"
+                , ctx->ga.iter, ctx->ga.Epsilon
+                );
+        }
         if (ctx->rep.PrintOptions & PGA_REPORT_AVERAGE) {
             for (k=0; k<numaux+1; k++) {
                 fprintf
@@ -165,19 +184,34 @@ void PGAPrintReport(PGAContext *ctx, FILE *fp, int pop)
         } else {
             PGAIndividual *ind = PGAGetIndividual (ctx, 0, pop);
             int found = 0;
-            for (k=0; k<ctx->ga.PopSize; k++) {
+            for (k=0; k<ctx->ga.PopSize; k++, ind++) {
+                int j = 0;
                 if (ind->rank == 0) {
+                    for (j=0; j<numcon; j++) {
+                        if (ind->auxeval [numaux - numcon + j] > 0) {
+                            break;
+                        }
+                    }
+                    if (j < numcon) {
+                        continue;
+                    }
                     if (!found) {
-                        printf ("Non-dominated individuals:\n");
+                        fprintf (fp, "Non-dominated individuals:\n");
                     }
                     PGAPrintString (ctx, fp, k, pop);
                     found++;
                 }
-                ind++;
             }
-            /* Should never happen */
+            /* Can happen if none of the individuals fulfills constraints */
             if (!found) {
-                PGAErrorPrintf (ctx, PGA_FATAL, "No undominated individuals");
+                fprintf (fp, "No individuals fulfilling constraints\n");
+                best_p = PGAGetBestIndex (ctx, pop);
+                fprintf (fp, "Best individual\nConstraints: ");
+                fprintf (fp, "%13.6e\n", PGAGetAuxTotal (ctx, best_p, pop));
+                if (numcon && ctx->ga.EpsilonGeneration) {
+                    fprintf (fp, "Epsilon:     %13.6e\n", ctx->ga.Epsilon);
+                }
+                PGAPrintString (ctx, fp, best_p, pop);
             }
         }
     }
@@ -297,6 +331,62 @@ int PGAGetPrintFrequencyValue (PGAContext *ctx)
     return(ctx->rep.PrintFreq);
 }
 
+/*U****************************************************************************
+   PGASetMultiObjPrecision - Specifies the precision in decimal places
+   for printing evaluations of multi objective optimization.
+
+   Category: Reporting
+
+   Inputs:
+      ctx        - context variable
+      prec       - the precision
+
+   Outputs:
+      None
+
+   Example:
+      PGAContext *ctx;
+      :
+      PGASetMultiObjPrecision (ctx, 12);
+
+****************************************************************************U*/
+void PGASetMultiObjPrecision (PGAContext *ctx, int prec)
+{
+    if (prec < 1 || prec > 20) {
+        PGAError
+            ( ctx
+            , "PGASetMultiObjPrecision: 1 <= prec <= 20"
+            , PGA_FATAL, PGA_INT, (void *) &prec
+            );
+    }
+    ctx->rep.MOPrecision = prec;
+}
+
+/*U***************************************************************************
+   PGAGetMultiObjPrecision - returns the precision for printing multi
+   objective optimization evaluations.
+
+   Category: Reporting
+
+   Inputs:
+      ctx - context variable
+
+   Outputs:
+      The precision of printing multi objective evaluations
+
+   Example:
+      PGAContext *ctx;
+      int prec;
+      :
+      prec = PGAGetMultiObjPrecision (ctx);
+
+***************************************************************************U*/
+int PGAGetMultiObjPrecision (PGAContext *ctx)
+{
+    PGAFailIfNotSetUp ("PGAGetMultiObjPrecision");
+    return (ctx->rep.MOPrecision);
+}
+
 /*U****************************************************************************
    PGAPrintPopulation - Calls PGAPrintIndividual to print each member of a
    population
@@ -315,7 +405,7 @@ int PGAGetPrintFrequencyValue (PGAContext *ctx)
    Example:
       PGAContext *ctx;
       :
-      PGAPrintPopulation(ctx, stdout, PGA_NEWPOP);
+      PGAPrintPopulation (ctx, stdout, PGA_NEWPOP);
 
 ****************************************************************************U*/
 void PGAPrintPopulation ( PGAContext *ctx, FILE *fp, int pop )
@@ -351,7 +441,7 @@ void PGAPrintPopulation ( PGAContext *ctx, FILE *fp, int pop )
       PGAContext *ctx;
       int p;
       :
-      PGAPrintIndividual(ctx, stdout, p, PGA_NEWPOP);
+      PGAPrintIndividual (ctx, stdout, p, PGA_NEWPOP);
 
 ****************************************************************************U*/
 void PGAPrintIndividual ( PGAContext *ctx, FILE *fp, int p, int pop )
@@ -390,7 +480,7 @@ void PGAPrintIndividual ( PGAContext *ctx, FILE *fp, int p, int pop )
       PGAContext *ctx;
       int p;
       :
-      PGAPrintString(ctx, stdout, p, PGA_OLDPOP);
+      PGAPrintString (ctx, stdout, p, PGA_OLDPOP);
 
 ****************************************************************************U*/
 void PGAPrintString ( PGAContext *ctx, FILE *file, int p, int pop )
@@ -430,7 +520,7 @@ void PGAPrintString ( PGAContext *ctx, FILE *file, int p, int pop )
    Example:
       PGAContext *ctx;
       :
-      PGAPrintContextVariable(ctx, stdout);
+      PGAPrintContextVariable (ctx, stdout);
 
 ****************************************************************************U*/
 void PGAPrintContextVariable ( PGAContext *ctx, FILE *fp )
@@ -633,20 +723,26 @@ void PGAPrintContextVariable ( PGAContext *ctx, FILE *fp )
      };
 
 
-     fprintf( fp,"    Mutate [And,Or] Crossover      : ");
-     switch(ctx->ga.MutateOnlyNoCross)
+     fprintf( fp,"    Mixing Type                    : ");
+     switch(ctx->ga.MixingType)
      {
-     case PGA_TRUE:
-          fprintf( fp,"Or\n");
+     case PGA_MIX_MUTATE_OR_CROSS:
+          fprintf (fp, "Mutation or crossover\n");
           break;
-     case PGA_FALSE:
-          fprintf( fp,"And\n");
+     case PGA_MIX_MUTATE_AND_CROSS:
+          fprintf (fp, "Mutation only if crossover\n");
+          break;
+     case PGA_MIX_MUTATE_ONLY:
+          fprintf (fp, "Mutation only\n");
+          break;
+     case PGA_MIX_TRADITIONAL:
+          fprintf (fp, "Traditional: Mutation after crossover\n");
           break;
      case PGA_UNINITIALIZED_INT:
           fprintf( fp,"*UNINITIALIZED*\n");
           break;
      default:
-          fprintf( fp,"!ERROR!  =(%d)?\n", ctx->ga.MutateOnlyNoCross);
+          fprintf( fp,"!ERROR!  =(%d)?\n", ctx->ga.MixingType);
           break;
      };
 
