@@ -311,19 +311,19 @@ void PGASetSelectType( PGAContext *ctx, int select_type)
       selecttype = PGAGetSelectType(ctx);
       switch (selecttype) {
       case PGA_SELECT_PROPORTIONAL:
-          printf("Selection Type = PGA_SELECT_PROPORTIONAL\n");
+          printf ("Selection Type = PGA_SELECT_PROPORTIONAL\n");
           break;
       case PGA_SELECT_SUS:
-          printf("Selection Type = PGA_SELECT_SUS\n");
+          printf ("Selection Type = PGA_SELECT_SUS\n");
           break;
       case PGA_SELECT_TOURNAMENT:
-          printf("Selection Type = PGA_SELECT_TOURNAMENT\n");
+          printf ("Selection Type = PGA_SELECT_TOURNAMENT\n");
           break;
       case PGA_SELECT_PTOURNAMENT:
-          printf("Selection Type = PGA_SELECT_PTOURNAMENT\n");
+          printf ("Selection Type = PGA_SELECT_PTOURNAMENT\n");
           break;
       case PGA_SELECT_TRUNCATION:
-          printf("Selection Type = PGA_SELECT_TRUNCATION\n");
+          printf ("Selection Type = PGA_SELECT_TRUNCATION\n");
           break;
       }
 
@@ -420,13 +420,13 @@ double PGAGetPTournamentProb(PGAContext *ctx)
       PGASetTournamentSize(ctx,3);
 
 ****************************************************************************U*/
-void PGASetTournamentSize(PGAContext *ctx, int tournament_size)
+void PGASetTournamentSize (PGAContext *ctx, double tournament_size)
 {
-    PGADebugEntered("PGASetTournamentSize");
+    PGADebugEntered ("PGASetTournamentSize");
 
     ctx->ga.TournamentSize = tournament_size;
 
-    PGADebugExited("PGASetTournamentSize");
+    PGADebugExited ("PGASetTournamentSize");
 }
 
 /*U***************************************************************************
@@ -443,19 +443,19 @@ void PGASetTournamentSize(PGAContext *ctx, int tournament_size)
 
    Example:
       PGAContext *ctx;
-      int sz;
+      double sz;
       :
-      sz = PGAGetTournamentSize(ctx);
+      sz = PGAGetTournamentSize (ctx);
 
 ***************************************************************************U*/
-int PGAGetTournamentSize(PGAContext *ctx)
+double PGAGetTournamentSize (PGAContext *ctx)
 {
-    PGADebugEntered("PGAGetTournamentSize");
+    PGADebugEntered ("PGAGetTournamentSize");
     PGAFailIfNotSetUp("PGAGetTournamentSize");
 
-    PGADebugExited("PGAGetTournamentSize");
+    PGADebugExited ("PGAGetTournamentSize");
 
-     return ctx->ga.TournamentSize;
+    return ctx->ga.TournamentSize;
 }
 /*U****************************************************************************
    PGASetTournamentWithReplacement - Specifies if tournament is with or
@@ -740,9 +740,13 @@ int PGASelectTournamentWithReplacement (PGAContext *ctx, int pop)
 {
     int i;
     int m = PGARandomInterval (ctx, 0, ctx->ga.PopSize-1);
-    for (i=1; i<ctx->ga.TournamentSize; i++) {
+    int t = (int)ctx->ga.TournamentSize;
+    if (ctx->ga.TournamentSize - t) {
+        t += PGARandomFlip (ctx, ctx->ga.TournamentSize - t);
+    }
+    for (i=1; i<t; i++) {
         int mn = PGARandomInterval(ctx, 0, ctx->ga.PopSize-1);
-        /* use '>=' for backwards-compat with prev. binary tournament */
+        /* use '<=' for backwards-compat with prev. binary tournament */
         if (PGAEvalCompare (ctx, mn, pop, m, pop) <= 0) {
             m = mn;
         }
@@ -780,46 +784,40 @@ int PGASelectTournamentWithReplacement (PGAContext *ctx, int pop)
 
 
 /* Helper function to compute permuted list */
-/* We're using Durstenfeld's version of the Fisher-Yates shuffle */
-
-static void _shuffle (PGAContext *ctx, int *list, int n)
+static void _shuffle (PGAContext *ctx, int k, int init)
 {
-    int i = 0, j = 0, tmp = 0;
-    for (i = 0; i < n; i++)
-	list[i] = i;
-    for (i = 0; i < n-1; i++) {
-	j = PGARandomInterval (ctx, i, n - 1);
-        tmp = list[j];
-        list [j] = list [i];
-        list [i] = tmp;
+    if (init) {
+        int i = 0;
+        for (i=0; i<k; i++) {
+            ctx->scratch.permute [i] = i;
+        }
     }
+    PGAShuffle (ctx, ctx->scratch.permute, k);
 }
-#define NEXT_IDX(ctx, perm, idx, n)                           \
-    ((idx) >= (n))                                            \
-      ? (_shuffle((ctx), (perm), (n)), (idx) = 1, (perm) [0]) \
-      : (perm) [(idx)++]
+#define NEXT_IDX(ctx, k, init)                        \
+    ((ctx)->ga.perm_idx >= (k))                       \
+    ? ( _shuffle((ctx), (k), (init))                  \
+      , (ctx)->ga.perm_idx = 1                        \
+      , (ctx)->scratch.permute [0]                    \
+      )                                               \
+    : ctx->scratch.permute [ctx->ga.perm_idx++]
 
 static
 int PGASelectTournamentWithoutReplacement (PGAContext *ctx, int pop)
 {
     int m;
     int i;
-    static int *permutation = NULL;
-    static int perm_idx = 0;
+    int t = (int)ctx->ga.TournamentSize;
 
-    if (permutation == NULL) {
-        permutation  = (int *) malloc(sizeof (int) * ctx->ga.PopSize);
-        if (permutation == NULL) {
-            PGAError (ctx, "PGASelectTournamentWithoutReplacement: malloc:",
-                     PGA_FATAL, PGA_INT, (void *) &ctx->ga.PopSize );
-            return 0;
-        }
-        perm_idx = ctx->ga.PopSize;
+    if (ctx->ga.TournamentSize - t) {
+        t += PGARandomFlip (ctx, ctx->ga.TournamentSize - t);
     }
 
-    m = NEXT_IDX(ctx, permutation, perm_idx, ctx->ga.PopSize);
-    for (i=1; i<ctx->ga.TournamentSize; i++) {
-        int mn = NEXT_IDX(ctx, permutation, perm_idx, ctx->ga.PopSize);
+    m = NEXT_IDX(ctx, ctx->ga.PopSize, 1);
+    assert (0 <= m && m < ctx->ga.PopSize);
+    for (i=1; i<t; i++) {
+        int mn = NEXT_IDX(ctx, ctx->ga.PopSize, 1);
+        assert (0 <= mn && mn < ctx->ga.PopSize);
         if (PGAEvalCompare (ctx, mn, pop, m, pop) <= 0) {
             m = mn;
         }
@@ -849,16 +847,14 @@ int PGASelectTournamentWithoutReplacement (PGAContext *ctx, int pop)
 ****************************************************************************I*/
 int PGASelectLinear (PGAContext *ctx, PGAIndividual *pop)
 {
-    static int perm_idx = 0;
-    static int last_generation = -1;
     int numreplace = PGAGetNumReplaceValue (ctx);
     int popsize = PGAGetPopSize (ctx);
 
-    if (last_generation != ctx->ga.iter || perm_idx >= popsize) {
-        perm_idx = popsize - numreplace;
-        last_generation = ctx->ga.iter;
+    if (ctx->ga.last_iter != ctx->ga.iter || ctx->ga.perm_idx >= popsize) {
+        ctx->ga.perm_idx  = popsize - numreplace;
+        ctx->ga.last_iter = ctx->ga.iter;
     }
-    return perm_idx++;
+    return ctx->ga.perm_idx++;
 }
 
 /*I****************************************************************************
@@ -884,9 +880,6 @@ int PGASelectTruncation (PGAContext *ctx, int pop)
 {
     int m = -1;
     int k = (int)(ctx->ga.PopSize * ctx->ga.TruncProportion + 0.5);
-    static int *kbest = NULL;
-    static int perm_idx = 0;
-    static int last_generation = -1;
 
     if (k < 1) {
         k = 1;
@@ -895,29 +888,21 @@ int PGASelectTruncation (PGAContext *ctx, int pop)
         k = ctx->ga.PopSize;
     }
 
-    if (kbest == NULL) {
-        kbest  = (int *) malloc(sizeof (int) * k);
-        if (kbest == NULL) {
-            PGAError (ctx, "PGASelectTruncation: malloc:",
-                     PGA_FATAL, PGA_INT, (void *) &k);
-            return 0;
-        }
-        perm_idx = k;
-    }
-    if (last_generation != ctx->ga.iter) {
+    if (ctx->ga.last_iter != ctx->ga.iter) {
         int i;
-        int bestidx [ctx->ga.PopSize];
+        DECLARE_DYNARRAY (int, bestidx, ctx->ga.PopSize);
         /* Returns sorted list of indeces in bestidx, no need to initialize */
         PGAEvalSort (ctx, pop, bestidx);
         /* Copy the k first indeces */
         for (i=0; i<k; i++) {
-            kbest [i] = bestidx [i];
+            ctx->scratch.permute [i] = bestidx [i];
         }
-        last_generation = ctx->ga.iter;
-        perm_idx = k;
+        ctx->ga.last_iter = ctx->ga.iter;
+        ctx->ga.perm_idx  = k;
     }
 
-    m = NEXT_IDX(ctx, kbest, perm_idx, k);
+    m = NEXT_IDX(ctx, k, 0);
+    assert (0 <= m && m < ctx->ga.PopSize);
     return m;
 }
 
