@@ -45,9 +45,42 @@ privately owned rights.
 *              Brian P. Walenz
 *****************************************************************************/
 
-#include <assert.h>
 #include <pgapack.h>
 
+/*U****************************************************************************
+  PGAShuffle:
+    We're using Durstenfeld's version of the Fisher-Yates shuffle
+
+  Category: Utility
+
+  Inputs:
+    ctx  - Context pointer
+    list - array of integers to shuffle
+    n    - number of elements in array
+
+  Outputs:
+    Shuffled array
+
+  Example:
+    list [ctx->ga.PopSize];
+    for (i=0; i<ctx->ga.PopSize; i++) {
+        list [i] = i;
+    }
+    PGAShuffle (ctx, list, ctx->ga.PopSize);
+
+****************************************************************************U*/
+void PGAShuffle (PGAContext *ctx, int *list, int n)
+{
+    int i, j, tmp = 0;
+
+    for (i=0; i<n-1; i++) {
+        j = PGARandomInterval (ctx, i, n - 1);
+        tmp = list [j];
+        list [j] = list [i];
+        list [i] = tmp;
+    }
+}
+
 /*U****************************************************************************
   PGAMean - calculates the mean value of an array of elements
 
@@ -367,8 +400,11 @@ int PGAGetWorstIndex(PGAContext *ctx, int pop)
 /*U***************************************************************************
   PGAGetBestIndex - returns the index of the string with the best evaluation
   function value in population pop
-  Note that in the presence of multiple evaluations, calling this
-  function does not make much sense.
+  Note that in the presence of multiple evaluations, this will return
+  - If all strings violate constraints the one with the least constraint
+    violation
+  - In case there exist strings without constraint violations one
+    *randomly* chosen from rank 0.
 
   Category: Utility
 
@@ -386,25 +422,41 @@ int PGAGetWorstIndex(PGAContext *ctx, int pop)
      best = PGAGetBestIndex(ctx,PGA_OLDPOP);
 
 ***************************************************************************U*/
-int PGAGetBestIndex(PGAContext *ctx, int pop)
+int PGAGetBestIndex (PGAContext *ctx, int popidx)
 {
-    int     p, Best_indx = 0;
+    int p, Best_indx = 0;
+    DECLARE_DYNARRAY (int, bestidxs, ctx->ga.PopSize);
+    int nbest = 0;
+    int is_multi = ctx->ga.NumAuxEval > ctx->ga.NumConstraint;
+    PGAIndividual *pop = PGAGetIndividual (ctx, 0, popidx);
 
     PGADebugEntered ("PGAGetBestIndex");
 
     for (p=0; p<ctx->ga.PopSize; p++) {
-        if (!PGAGetEvaluationUpToDateFlag (ctx, p, pop)) {
+        if (!PGAGetEvaluationUpToDateFlag (ctx, p, popidx)) {
 	    PGAError
                 ( ctx, "PGAGetBestIndex: Evaluate function not up to date:"
                 , PGA_FATAL, PGA_INT, (void *) &p
                 );
         }
     }
+    if (is_multi && pop->rank == 0 && !INDGetAuxTotal (pop)) {
+        bestidxs [nbest++] = 0;
+    }
 
     for (p=1; p<ctx->ga.PopSize; p++) {
-        if (PGAEvalCompare (ctx, p, pop, Best_indx, pop) < 0) {
+        if (PGAEvalCompare (ctx, p, popidx, Best_indx, popidx) < 0) {
             Best_indx = p;
         }
+        if (is_multi && (pop + p)->rank == 0 && !INDGetAuxTotal (pop + p)) {
+            bestidxs [nbest++] = p;
+        }
+    }
+    /* Shuffle and return first */
+    if (is_multi && nbest > 1) {
+        PGAShuffle (ctx, bestidxs, nbest);
+        Best_indx = bestidxs [0];
+        assert ((pop + Best_indx)->rank == 0);
     }
 
     PGADebugExited ("PGAGetBestIndex");
@@ -414,8 +466,8 @@ int PGAGetBestIndex(PGAContext *ctx, int pop)
 
 
 /*U***************************************************************************
-  PGAGetBestReport - returns the string with the best evaluation
-  function value in population pop for the given evaluation.
+  PGAGetBestReport - returns the best evaluation value in population pop
+  for the given evaluation.
   The evaluation function index must be 0 < idx <= NumAuxEval. So for
   single evaluation only the index 0 is allowed and the return is the
   evaluation as from PGAGetBestIndex. Note that this accesses the
@@ -426,7 +478,7 @@ int PGAGetBestIndex(PGAContext *ctx, int pop)
 
   Inputs:
      ctx - context variable
-     pop - symbolic constant of the population to find the best string in
+     pop - symbolic constant of the population to find the best eval,
            only PGA_OLDPOP is allowed
      idx - Index of the evaluation function, 0 for the one return by
            your evaluation function 1 .. NumAuxEval for the auxiliary
@@ -585,7 +637,7 @@ void PGAUpdateBest (PGAContext *ctx, int popix)
 {
     PGAIndividual *ind = popix == PGA_OLDPOP ? ctx->ga.oldpop : ctx->ga.newpop;
     PGAIndividual *pop = ind;
-    PGAIndividual *best [ctx->ga.NumAuxEval + 1];
+    DECLARE_DYNARRAY (PGAIndividual *, best, ctx->ga.NumAuxEval + 1);
     int numfun = ctx->ga.NumAuxEval - ctx->ga.NumConstraint;
     int validcount = 0;
     int p, k;
@@ -598,6 +650,7 @@ void PGAUpdateBest (PGAContext *ctx, int popix)
     if (!INDGetAuxTotal (ind)) {
         validcount++;
     }
+    ctx->rep.MinSumConstr = ind->auxtotal;
     ind++;
     for (p=1; p<ctx->ga.PopSize; p++) {
 	if (!PGAGetEvaluationUpToDateFlag (ctx, p, popix)) {
@@ -609,6 +662,9 @@ void PGAUpdateBest (PGAContext *ctx, int popix)
         if (!INDGetAuxTotal (ind)) {
             validcount++;
         }
+        if (ind->auxtotal < ctx->rep.MinSumConstr) {
+            ctx->rep.MinSumConstr = ind->auxtotal;
+        }
         for (k=0; k<=ctx->ga.NumAuxEval; k++) {
             PGAIndividual *b = best [k];
             double enew;
@@ -631,8 +687,12 @@ void PGAUpdateBest (PGAContext *ctx, int popix)
                 int funcmp = ctx->ga.optdir == PGA_MINIMIZE
                            ? CMP (enew, eold)
                            : CMP (eold, enew);
-                if (  (INDGetAuxTotal (b) && !INDGetAuxTotal (ind))
-                   || (!INDGetAuxTotal (ind) && funcmp < 0)
+                if (  (  INDGetAuxTotal (b)   - ctx->ga.Epsilon  > 0
+                      && INDGetAuxTotal (ind) - ctx->ga.Epsilon <= 0
+                      )
+                   || (  INDGetAuxTotal (ind) - ctx->ga.Epsilon <= 0
+                      && funcmp < 0
+                      )
                    )
                 {
                     best [k] = ind;
@@ -645,7 +705,7 @@ void PGAUpdateBest (PGAContext *ctx, int popix)
     for (k=0; k<=ctx->ga.NumAuxEval; k++) {
         if (k<=numfun) {
             double e = (k==0) ? best [k]->evalue : best [k]->auxeval [k-1];
-            if (INDGetAuxTotal (best [k])) {
+            if (INDGetAuxTotal (best [k]) - ctx->ga.Epsilon > 0) {
                 ctx->rep.Best [k] = NAN;
             } else {
                 ctx->rep.Best [k] = e;
@@ -745,7 +805,7 @@ void PGAUpdateOnline(PGAContext *ctx, int pop)
     int validcount = 0;
     PGAIndividual *ind = pop == PGA_OLDPOP ? ctx->ga.oldpop : ctx->ga.newpop;
     int numfun = ctx->ga.NumAuxEval - ctx->ga.NumConstraint;
-    double evalsum [ctx->ga.NumAuxEval + 1];
+    DECLARE_DYNARRAY (double, evalsum, ctx->ga.NumAuxEval + 1);
     int p, k;
 
     PGADebugEntered ("PGAUpdateOnline");
@@ -963,6 +1023,16 @@ int INDEvalCompare (PGAIndividual *ind1, PGAIndividual *ind2)
         auxt1 = INDGetAuxTotal (ind1);
         auxt2 = INDGetAuxTotal (ind2);
     }
+    if (ctx->ga.Epsilon) {
+        auxt1 -= ctx->ga.Epsilon;
+        auxt2 -= ctx->ga.Epsilon;
+        if (auxt1 < 0) {
+            auxt1 = 0;
+        }
+        if (auxt2 < 0) {
+            auxt2 = 0;
+        }
+    }
     if (auxt1 || auxt2) {
         return CMP (auxt1, auxt2);
     }
@@ -1092,7 +1162,7 @@ int PGAEvalSortHelper (const void *a1, const void *a2)
 void PGAEvalSort (PGAContext *ctx, int pop, int *idx)
 {
     int i;
-    PGAIndividual *sorttmp [ctx->ga.PopSize];
+    DECLARE_DYNARRAY (PGAIndividual *, sorttmp, ctx->ga.PopSize);
     /* No need to check validity of pop, done by PGAIndividual */
     PGAIndividual *first = PGAGetIndividual (ctx, 0, pop);
 
@@ -1105,3 +1175,75 @@ void PGAEvalSort (PGAContext *ctx, int pop, int *idx)
     }
 }
 
+#define HASH_JEN_MIX(a,b,c)              \
+do {                                     \
+  a -= b; a -= c; a ^= ( c >> 13 );      \
+  b -= c; b -= a; b ^= ( a << 8 );       \
+  c -= a; c -= b; c ^= ( b >> 13 );      \
+  a -= b; a -= c; a ^= ( c >> 12 );      \
+  b -= c; b -= a; b ^= ( a << 16 );      \
+  c -= a; c -= b; c ^= ( b >> 5 );       \
+  a -= b; a -= c; a ^= ( c >> 3 );       \
+  b -= c; b -= a; b ^= ( a << 10 );      \
+  c -= a; c -= b; c ^= ( b >> 15 );      \
+} while (0)
+
+/* This is Bob Jenkins' hash function as implemented in the uthash project
+ * https://github.com/troydhanson/uthash/blob/master/src/uthash.h
+ * The last parameter is the previous hash (if hashing different pieces)
+ */
+
+PGAHash PGAUtilHash (const void *data, size_t len, PGAHash hashv)
+{
+    PGAHash _hj_i,_hj_j,_hj_k;
+    unsigned const char *_hj_key = data;
+    _hj_i = _hj_j = 0x9e3779b9u;
+    _hj_k = (PGAHash)(len);
+    while (_hj_k >= 12U) {
+        _hj_i +=    (_hj_key[0] + ( (PGAHash)_hj_key[1] << 8 )
+            + ( (PGAHash)_hj_key[2] << 16 )
+            + ( (PGAHash)_hj_key[3] << 24 ) );
+        _hj_j +=    (_hj_key[4] + ( (PGAHash)_hj_key[5] << 8 )
+            + ( (PGAHash)_hj_key[6] << 16 )
+            + ( (PGAHash)_hj_key[7] << 24 ) );
+        hashv += (_hj_key[8] + ( (PGAHash)_hj_key[9] << 8 )
+            + ( (PGAHash)_hj_key[10] << 16 )
+            + ( (PGAHash)_hj_key[11] << 24 ) );
+
+        HASH_JEN_MIX(_hj_i, _hj_j, hashv);
+
+        _hj_key += 12;
+        _hj_k -= 12U;
+    }
+    hashv += (PGAHash)(len);
+    switch ( _hj_k ) {
+      case 11: hashv += ( (PGAHash)_hj_key[10] << 24 ); /* FALLTHROUGH */
+      case 10: hashv += ( (PGAHash)_hj_key[9] << 16 );  /* FALLTHROUGH */
+      case 9:  hashv += ( (PGAHash)_hj_key[8] << 8 );   /* FALLTHROUGH */
+      case 8:  _hj_j += ( (PGAHash)_hj_key[7] << 24 );  /* FALLTHROUGH */
+      case 7:  _hj_j += ( (PGAHash)_hj_key[6] << 16 );  /* FALLTHROUGH */
+      case 6:  _hj_j += ( (PGAHash)_hj_key[5] << 8 );   /* FALLTHROUGH */
+      case 5:  _hj_j += _hj_key[4];                      /* FALLTHROUGH */
+      case 4:  _hj_i += ( (PGAHash)_hj_key[3] << 24 );  /* FALLTHROUGH */
+      case 3:  _hj_i += ( (PGAHash)_hj_key[2] << 16 );  /* FALLTHROUGH */
+      case 2:  _hj_i += ( (PGAHash)_hj_key[1] << 8 );   /* FALLTHROUGH */
+      case 1:  _hj_i += _hj_key[0];                      /* FALLTHROUGH */
+      default: ;
+    }
+    HASH_JEN_MIX(_hj_i, _hj_j, hashv);
+    return hashv;
+}
+
+size_t PGAIndividualHashIndex (PGAContext *ctx, int p, int pop)
+{
+    PGABinary hash = 0;
+    assert (ctx->ga.NoDuplicates);
+
+    if (ctx->fops.Hash) {
+	int fp = ((p == PGA_TEMP1) || (p == PGA_TEMP2)) ? p : p + 1;
+	hash = (*ctx->fops.Hash)(&ctx, &fp, &pop);
+    } else {
+	hash = (*ctx->cops.Hash)(ctx, p, pop);
+    }
+    return (size_t)(hash % ctx->ga.PopSize);
+}
